// FILE: ./bower_components/zh-customizations-service/dist/zhCustomizationsService.js

// FILE: src/config/config.js
(function(){
window.templateUrls = window.templateUrls || [];
var packageName = 'zh-customizations-service';

var hasPackageRegistered = window.templateUrls.some(function(templateUrl){
	return templateUrl.package === packageName;
});

if(!hasPackageRegistered){
	window.templateUrls.push({
		'package': packageName,
		'baseUrl': './bower_components/' + packageName + '/dist/templates/'
	});
}
})();

// FILE: src/registers/registerEvents.js
Configuration(function(eventEngine) {
  eventEngine.registerEvent("FilterPreferenceOnClick", "filterOnClick");
  eventEngine.registerEvent("VisualizationPreferenceOnClick", "visualizationOnClick");
  eventEngine.registerEvent("LayoutPreferenceOnClick", "layoutOnClick");
});

// FILE: src/PreferencesUser.js
function PreferencesUser() {

    var userId = localStorage.getItem('userId');

    this.registerUser = function(primaryKey){
        userId = primaryKey;
    };

    this.getPreferenceUserRegistered = function(){
        return userId;
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register("preferencesUserService", PreferencesUser);
});

// FILE: src/CustomizationActionInjector.js
(function() {

	function CustomizationActionInjector(templateManager, eventAggregator, ApplicationContext) {

		eventAggregator.subscribe("beforeProjectStartup", pushCustomizationAction);

		//------Dependencies
		var zhLayoutPreferences, zhFilterPreferences, zhCustomizationPreferences;

		function getDependencyByCustomizationType(type) {
			var dependency;
			if (type == "layout"){
				dependency = zhLayoutPreferences;
			} else if (type == "filter"){
				dependency = zhFilterPreferences;
			} else if (type =="view"){
				dependency = zhCustomizationPreferences;
			}
			return dependency;
		}

		// function getDefaultVisibilityByTemplate(widget, type) {
		// 	var defaults = [
		// 		{ template: 'widget/list.html', 		options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/grid.html', 		options: { search: true, filter: true, layout: true, view: true },
		// 		{ template: 'widget/tree/grid.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/kanban.html', 		options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/crosstab.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'widget/form.html', 		options: { search: false, filter: false, layout: false, view: false },
		// 		{ template: 'widget/dashboard.html', 	options: { search: true, filter: true, layout: false, view: false },
		// 		{ template: 'container/dashboard.html', options: { search: true, filter: true, layout: false, view: false }
		// 	];
		// }

		function isCustomizationActionVisible(widget, type) {
			var dependency = getDependencyByCustomizationType(type);
			return dependency && (Util.isUndefined(widget.floatingControl) || 
								  Util.isUndefined(widget.floatingControl.customizationAction) ||
								  widget.floatingControl.customizationAction[type] !== false);
		}

		function pushCustomizationAction() {
			zhLayoutPreferences = ApplicationContext.zhLayoutPreferences;
			zhFilterPreferences = ApplicationContext.zhFilterPreferences;
			zhCustomizationPreferences = ApplicationContext.zhCustomizationPreferences;

			var customFloatingActions = templateManager.project.customFloatingActions;
			var actionExistence = customFloatingActions.filter(function (action) {
				return action.id == "#hashReportsAction#";
			}).shift();
			if (!actionExistence) {
				templateManager.project.customFloatingActions.push({
					"id": "#hashFilterAction#",
					"name": "FilterAction",
					"label": "Search",
					"type": "filter",
					"order": 20,
					"icon": "search",
					"template": "zh-customizations-service#floating-filter-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'filter') || Util.isUndefined(widget.floatingControl) || widget.floatingControl.searchAction !== false; }
				});
				templateManager.project.customFloatingActions.push({
					"id": "#hashLayoutsAction#",
					"name": "LayoutAction",
					"label": "Layouts",
					"type": "layout",
					"order": 30,
					"icon": "layout",
					"template": "zh-customizations-service#floating-preferences-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'layout'); }
				});
				templateManager.project.customFloatingActions.push({
					"id": "#hashReportsAction#",
					"name": "ReportAction",
					"label": "Views",
					"type": "view",
					"order": 40,
					"icon": "view",
					"template": "zh-customizations-service#floating-preferences-card.html",
					"expandable": true,
					"isVisible": function(widget) { return isCustomizationActionVisible(widget, 'view'); }
				});
			}
		}
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("CustomizationActionInjector", CustomizationActionInjector);
	});

})();

// FILE: src/CustomizationsFloatingCardController.js
function CustomizationsFloatingCardController($scope, $rootScope, ScreenService, templateManager, ApplicationContext, restEngine, ZHPromise, eventAggregator) {
	
	var boundCustomizationUpdateEvent = eventAggregator.subscribe("OnUpdateCustomizationsRepositories", fillFloatingCardRepositories);

	$scope.$on("destroy", function(){
		eventAggregator.unSubscribe("OnUpdateCustomizationsRepositories", boundCustomizationUpdateEvent);
	});
	
	//------Dependencies
	var zhLayoutPreferences = ApplicationContext.zhLayoutPreferences;
	var zhFilterPreferences = ApplicationContext.zhFilterPreferences;
	var zhCustomizationPreferences = ApplicationContext.zhCustomizationPreferences;
	var zhCustomizationsService = ApplicationContext.zhCustomizationsService;
	
	//------ Control viariables
	var editingWidget = $scope.$parent.widget;
	var customizationIndex = 0;
	var floatingControl = editingWidget.floatingControl || {};
	var customizationsWidgetOptions = floatingControl.customizationAction;
	$scope.settingsCardLabel = ScreenService.i18n("Configurations");

	function checkFloatingCardOverflow(e) {
		var actionElement = $(e.target).closest(".float-action");
		var floatingControl = actionElement.parents(".zh-floating-control");
		var widgetContainer = actionElement.closest(".container");
		var actionTop, actionBottom;
		var widgetTop, widgetBottom;

		if (!actionElement.hasClass("opened")) {
			actionElement.removeClass("fix-position-top");
			actionElement.removeClass("fix-position-bottom");

			actionTop = actionElement.offset().top;
			widgetTop = widgetContainer.offset().top;
			widgetBottom = widgetTop + widgetContainer.height();
			if (floatingControl.hasClass("pos-top")) {
				actionTop = actionTop - 200 + actionElement.height();
				if (actionTop < widgetTop) {
					actionElement.addClass("fix-position-top");
				}
			} else {
				actionBottom = actionTop + 200;
				if (actionBottom > widgetBottom) {
					actionElement.addClass("fix-position-bottom");
				}				
			}
		}
	}

	$scope.togglePreferencesCard = function(e){
		checkFloatingCardOverflow(e);
		$scope.toggleAction(e);
	};
	editingWidget.togglePreferencesCard = $scope.togglePreferencesCard;

	var preferencesConfigsClick = function(){
		zhLayoutPreferences.startConfig(editingWidget);
	};

	var filtersConfigsClick = function(){
		zhFilterPreferences.startFilterConfig(editingWidget);
	};

	var viewsConfigsClick = function(){
		zhCustomizationPreferences.startViewSettings(editingWidget);
	};

	var preferencesRestoreClick = function(){
		zhLayoutPreferences.restoreDefault(editingWidget);
	};

	var filtersRestoreClick = function(){
		zhFilterPreferences.restoreDefault(editingWidget);
	};

	var viewsRestoreClick = function(){
		zhCustomizationPreferences.restoreDefault(editingWidget);
	};

	function applyFilter(customization){
		zhCustomizationsService.applyFilter(customization, editingWidget);
		editingWidget.customizationsApplied = editingWidget.customizationsApplied || {};
		delete editingWidget.customizationsApplied.view;
	}

	function applyLayout(customization){
		zhCustomizationsService.applyLayout(customization, editingWidget);
		editingWidget.customizationsApplied = editingWidget.customizationsApplied || {};
		delete editingWidget.customizationsApplied.view;
	}

	function applyVisualization(customization){
		zhCustomizationsService.applyVisualization(customization, editingWidget);
	}

	$scope.isCustomizationApplied = function(customization){
		if(editingWidget.customizationsApplied){
			var appliedCustomization = editingWidget.customizationsApplied[customization.customizationType];
			if(appliedCustomization == customization._id ){
				return true;
			}
		}
	};

	$scope.customizationsTypes = [
		{
			"label": ScreenService.i18n("Views"),
			"type": "view",
			"configsClick": viewsConfigsClick,
			"restoreClick": viewsRestoreClick,
			"applyOnClick": applyVisualization,
			"dataSource": {
				"data": []
			}
		},
		{
			"label": ScreenService.i18n("Layouts"),
			"type": "layout",
			"configsClick": preferencesConfigsClick,
			"restoreClick": preferencesRestoreClick,
			"applyOnClick": applyLayout,
			"dataSource": {
				"data": []
			}
		},
		{
			"label": ScreenService.i18n("Filters"),
			"type": "filter",
			"configsClick": filtersConfigsClick,
			"restoreClick": filtersRestoreClick,
			"applyOnClick": applyFilter,
			"dataSource": {
				"data": []
			}
		}
	];

	$scope.setActiveCustomization = function(type) {
		$scope.activeCustomization = $scope.customizationsTypes.filter(function(customizationType) {
			return customizationType.type == type;
		}).shift();
	};

	function fillFloatingCardRepositories(){
		return ZHPromise.all($scope.customizationsTypes.filter(function(customizationType) {
			return Util.isUndefined(editingWidget.floatingControl) || Util.isUndefined(editingWidget.floatingControl.customizationAction) || editingWidget.floatingControl.customizationAction[customizationType.type] !== false;
		}).map(function(customizationType){
			return zhCustomizationsService.getWidgetCustomizations(editingWidget, customizationType.type).then(function(custo){
				insertDefaultOption(custo, customizationType.type);
				customizationType.dataSource.data = custo;
				ScreenService.updateTemplate();
			});
		}));
	}
	fillFloatingCardRepositories();

	function insertDefaultOption(customizations, customizationType){
		var defaultOption = {
			"_id": "default",
			"label": ScreenService.i18n("Original"),
			"customizationType": customizationType,
			"isDefaultOption": true
		};
		if(defaultOption.customizationType == "filter" && zhFilterPreferences){
			defaultOption.filters = zhFilterPreferences.getDefaultFilterProperties(editingWidget) || [];
		}
		customizations.push(defaultOption);
	}

	$scope.previousCustomizationType = function(){
		if(customizationIndex === 0){
			customizationIndex = $scope.customizationsTypes.length - 1;
		} else {
			customizationIndex--;
		}
		updateCustomizationType(customizationIndex);
	};

	$scope.forwardCustomizationType = function(){
		if(customizationIndex === $scope.customizationsTypes.length - 1){
			customizationIndex = 0;
		} else {
			customizationIndex++;
		}
		updateCustomizationType(customizationIndex);
	};
}

// FILE: src/CustomizationsRequestsBuilder.js
(function() {

	function CustomizationsRequestsBuilder(templateManager) {

		this.buildRequest = function(typeRequest, method, dbProperties, dataset) {
			var request = {
				"typeRequest": typeRequest,
				"baseUrl": getbaseUrl(dbProperties),
				"service": getServiceUrl(dbProperties, method),
				"params": {
					"requestType": "DataSet",
					"dataset": dataset
				}
			};
			return request;
		};

		var getServiceUrl = function(dbProperties, method) {
			return !dbProperties.urlFullPath ? '/' + method + '/' + getRoute(dbProperties) : '';
		};

		var getbaseUrl = function(dbProperties) {
			return dbProperties.url ? dbProperties.url : templateManager.project.zhStoragePath;
		};

		var getRoute = function(dbProperties) {
			return dbProperties.dbName + "/" + dbProperties.collectionName;
		};
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("CustomizationsRequestsBuilder", CustomizationsRequestsBuilder);
	});

})();

// FILE: src/CustomizationsService.js
(function() {

	function CustomizationService($rootScope, ScreenService, templateManager, RepositoryFactory, Query, ZhUtil, restEngine, ZHPromise, CustomizationsRequestsBuilder, ApplicationContext, containerMerger, eventAggregator, eventEngine, preferencesUserService, ZhLogger, ZhKeyboardService) {

		var self = this;
		var customizationsRepositories;
		var customizationsTypes = ["layout", "filter", "view"];
		var SUFFIX_ORIGINAL_VALUE = "original";
		var userPreference;

		function extendBehavior() {
			Util.extend(self, preferencesUserService);
		}
		extendBehavior();

		var validateRegister = function(register) {
			var valid = register && register.containerId && register.widgetId;
			if (!valid) {
				ZhLogger.warning('The container and the widget must have IDs to save customizations');
			}
			return valid;
		};

		var getCustomizationsRepositories = function() {
			return templateManager.projectPromise.then(function() {
				customizationsRepositories = templateManager.project.customizationsRepositories || [];
			});
		};

		var createCustomizationsRepositories = function() {
			customizationsRepositories.forEach(function(custoRepo) {
				var repositoryName = custoRepo.dbName + custoRepo.collectionName;
				ZhUtil.registerDataSource(repositoryName, "_id");
				repositoryName = "/" + repositoryName;
				custoRepo.repository = RepositoryFactory.factory(repositoryName, "LOCAL");
			});
		};

		self.fillCustomizationsRepositories = function() {
			var promises = customizationsRepositories.map(function(custoRepo) {
				var request = CustomizationsRequestsBuilder.buildRequest("GET", "findAll", custoRepo, []);
				return restEngine.request(request).then(function(responseData) {
					return custoRepo.repository.save(responseData.dataset.foundData);
				});
			});
			return ZHPromise.all(promises);
		};

		self.getRepository = function(custoId) {
			var customizationRepository = getCustomizationRepositoryById(custoId);
			if (!Util.isEmptyOrBlank(customizationRepository) && customizationRepository.repository) {
				return customizationRepository.repository;
			}
		};

		self.removeCustomizations = function(customizationsToDelete) {
			var customizationsMappedById = groupIdsByCustomizationRepository(customizationsToDelete);
			var promises = customizationsMappedById.map(function(customization) {
				var customizationRepository = getCustomizationRepositoryById(customization.custoRepo);
				var query = Query.build()
					.where("_id").in(customization.ids);
				var request = CustomizationsRequestsBuilder.buildRequest("DELETE", "remove", customizationRepository, createCustomizationsIdArrayIndexedById(customization.ids));
				return restEngine.request(request).then(function() {
					return customizationRepository.repository.remove(query);
				});
			});
			return ZHPromise.all(promises)
				.then(function(promisesResult) {
					eventAggregator.publish("OnUpdateCustomizationsRepositories");
					return promisesResult;
				});
		};

		self.saveCustomization = function(rows, customizationType) {
			rows = Util.parseToArray(rows);
			var promises = rows.map(function(row) {
				if (validateRegister(row) && customizationType && ~customizationsTypes.indexOf(customizationType)) {
					row.customizationType = row.customizationType || customizationType;
					var customizationRepository = getCustomizationRepositoryById(row.customizationRepository);
					var request = CustomizationsRequestsBuilder.buildRequest("POST", "save", customizationRepository, [row]);

					return restEngine.request(request).then(function(responseData) {
						var savedPreferences = responseData.dataset.response.saved;

						if (savedPreferences.length === 1) {
							row._id = savedPreferences[0]._id;
							delete row.unsync_id;
						}

						return customizationRepository.repository.save([row]);
					});
				} else {
					return ZHPromise.when({});
				}
			});

			return ZHPromise.all(promises)
				.then(function(promisesResult) {
					eventAggregator.publish("OnUpdateCustomizationsRepositories");
					return promisesResult;
				});
		};

		self.getWidgetCustomizations = function(widget, customizationType) {
			var promises = customizationsRepositories.map(function(repository) {
				var query = Query.build()
					.where("containerId").equals(widget.container.id)
					.where("widgetId").equals(widget.id)
					.where("customizationType").equals(customizationType);
				var userId = preferencesUserService.getPreferenceUserRegistered();
				if (userId && repository.customizationsByUser) {
					query.where("userId").equals(userId);
				}
				return repository.repository.find(query);
			});
			return ZHPromise.all(promises).then(function(reponses) {
				return reponses.reduce(function(accumulatedCustomizations, response) {
					return accumulatedCustomizations.concat(response);
				}, []);
			});
		};

		self.transferCustomizationsRepository = function(widget, rows, destinyRepository) {
			ScreenService.showLoader();
			return self.removeCustomizations(rows).then(function() {
				var syncIds = {};
				promises = rows.map(function(row) {
					return transferViewCustomizations(row, destinyRepository, widget).then(function() {
						row.customizationRepository = destinyRepository;
						row.unsync_id = new Date().getTime();
						removeNotInterestingProperties(row);
						delete row._id;
						return self.saveCustomization(row, row.customizationType).then(function(syncId) {
							syncIds[row.customizationType] = syncId;
						});
					});
				});
				return ZHPromise.all(promises).then(function() {
					ScreenService.hideLoader();
					return syncIds;
				});
			});
		};

		var transferViewCustomizations = function(view, destinyRepository, widget) {
			if (view.customizationType == "view") {
				var viewCustomizationsMapped = [{
					"customizationType": "filter",
					"id": view.filterId
				}, {
					"customizationType": "layout",
					"id": view.preferenceId
				}];
				var viewCustomizations = [];
				var customizationRepository = view.customizationRepository;
				var promises = viewCustomizationsMapped.map(function(customization) {
					return self.getWidgetCustomizationsByRepository(widget, customization.customizationType, customizationRepository, customization.id)
						.then(function(customizationObject) {
							viewCustomizations.push(customizationObject.shift());
						});
				});
				return ZHPromise.all(promises).then(function() {
					return self.transferCustomizationsRepository(widget, viewCustomizations, destinyRepository).then(function(syncIdsMapped) {
						view.filterId = syncIdsMapped.filter;
						view.preferenceId = syncIdsMapped.layout;
					});
				});
			} else {
				return ZHPromise.when();
			}
		};

		self.getCustomizationView = function(widget, customizationRepository, customizationType, id) {
			var idPropertyName = customizationType == "filter" ? "filterId" : "preferenceId";
			return self.getWidgetCustomizationsByRepository(widget, "view", customizationRepository)
				.then(function(viewObjects) {
					var viewObject;
					if (Util.isArray(viewObjects) && viewObjects.length > 0) {
						viewObject = viewObjects.filter(function(view) {
							return view[idPropertyName] == id;
						});
					}
					return viewObject;
				});
		};

		var removeNotInterestingProperties = function(object) {
			Object.keys(object).forEach(function(key) {
				if (~key.indexOf('__')) {
					delete object[key];
				}
			});
		};

		self.getWidgetCustomizationsByRepository = function(widget, customizationType, repositoryType, customizationId) {
			var customizationRepository = getCustomizationRepositoryById(repositoryType);
			var query = Query.build()
				.where("containerId").equals(widget.container.id)
				.where("widgetId").equals(widget.id)
				.where("customizationType").equals(customizationType);
			var userId = preferencesUserService.getPreferenceUserRegistered();
			if (userId && customizationRepository.customizationsByUser) {
				query.where("userId").equals(userId);
			}
			if (customizationId) {
				query.where("_id").equals(customizationId);
			}
			return customizationRepository.repository.find(query);
		};

		self.getCustomizationsRepositoriesTypes = function() {
			return customizationsRepositories;
		};

		self.getUpdatedCustomizationsRepositoriesTypes = function() {
			return self.fillCustomizationsRepositories().then(function() {
				return customizationsRepositories;
			});
		};

		self.getLabelRepositoryType = function(repositoryTypeValue) {
			return customizationsRepositories.filter(function(repository) {
				return repository.value == repositoryTypeValue;
			}).shift().label;
		};

		self.applyLayout = function(customization, editingWidget) {
			var eventData = createEventData(customization, editingWidget);
			if (self.layoutOnClick) {
				self.layoutOnClick(eventData);
			}
		};

		self.applyFilter = function(customization, editingWidget) {
			var eventData = createEventData(customization, editingWidget);
			if (self.filterOnClick) {
				self.filterOnClick(eventData);
			}
		};

		self.applyVisualization = function(customization, editingWidget) {
			var eventData = createEventData(customization, editingWidget);
			if (self.visualizationOnClick) {
				self.visualizationOnClick(eventData);
			}
		};

		function createEventData(customization, editingWidget) {
			return {
				"customization": customization,
				"editingWidget": editingWidget
			};
		}

		var groupIdsByCustomizationRepository = function(customizations) {
			var customizationsMappedById = [];
			customizationsRepositories.forEach(function(custoRepo) {
				var sameCustoRepoIds = customizations.filter(function(customization) {
					return customization.customizationRepository == custoRepo.value;
				});
				var map = {
					"custoRepo": custoRepo.value,
					"ids": sameCustoRepoIds.map(function(customization) {
						return customization._id;
					})
				};
				if (Util.isArray(map.ids) && map.ids.length > 0) {
					customizationsMappedById.push(map);
				}
			});
			return customizationsMappedById;
		};

		function createCustomizationsIdArrayIndexedById(customizationsId) {
			var customizationsToDelete = [];
			customizationsId.forEach(function(id) {
				customizationsToDelete.push({
					"_id": id
				});
			});
			return customizationsToDelete;
		}

		var getCustomizationRepositoryById = function(custoId) {
			return customizationsRepositories.filter(function(custoRepo) {
				return custoRepo.value == custoId;
			}).shift();
		};

		//-------------Begin Execution
		getCustomizationsRepositories().then(function() {
			createCustomizationsRepositories();
			self.fillCustomizationsRepositories().then(function() {
				eventAggregator.publish("OnUpdateCustomizationsRepositories");
			});
		});
	}

	Configuration(function(ContextRegister) {
		ContextRegister.register("zhCustomizationsService", CustomizationService);
	});

})();

// FILE: ./bower_components/zh-filter-preferences/dist/zhFilterPreferences.js

// FILE: src/registers/registerEvents.js
Configuration(function(eventEngine) {
  eventEngine.registerEvent("GridFilterBeforeApply", "beforeGridFilterApply");
  eventEngine.registerEvent("GridFilterAfterApply", "afterGridFilterApply");
  eventEngine.registerEvent("GridFilterAfterCancel", "afterGridFilterCancel");
});

// FILE: src/FilterBuilder.js
(function () {

function FilterBuilder(Query, ZHPromise) {
	var _filterType = {};
	var skipFiltering = [];
	var self = this;

	var _buildFilterTypeMap = function (fields, currentRow) {
		fields.forEach(function (field) {
			if (!field.filterProperties) {
				setOperatorFromFieldTemplate(field, currentRow);
			} else {
				var filterProperties = field.filterProperties;
				if (filterProperties.operator) {
					setOperatorFromFieldOperator(_filterType, field);
				} else {
					setOperatorFromFieldTemplate(field, currentRow);
				}
				if (filterProperties.outFilters) {
					Object.keys(filterProperties.outFilters).forEach(function (key) {
						var operator = filterProperties.outFilters[key];
						_filterType[key] = operator ? operator : _filterType[field.name];
					});
				}
			}
		});
	};

	function setOperatorFromFieldOperator(_filterType, field) {
		if (fieldHasSelectTemplate(field)) {
			_filterType[field.outData[field.valueField]] = field.filterProperties.operator;
		} else {
			_filterType[field.name] = field.filterProperties.operator;
		}
	}

	function fieldHasSelectTemplate(field) {
		var template = (field.filterProperties && field.filterProperties.template) || field.template;
		return field.outData && template && (~template.indexOf('select.html') || ~template.indexOf('select-autocomplete.html'));
	}

	var setOperatorFromFieldTemplate = function (field, currentRow) {
		var template = (field.filterProperties && field.filterProperties.template) || field.template;

		if (template && (template.indexOf('between-dates.html') !== -1 || template.indexOf('interval.html') !== -1)) {
			_filterType[field.name] = "BETWEEN";
			var fieldValue = currentRow[field.name];
			if (fieldValue) {
				if (!Object.keys(fieldValue).some(function (element) {
					return fieldValue[element];
				})) {
					if (!~skipFiltering.indexOf(field.name)) {
						skipFiltering.push(field.name);
					}
				}
			}
		} else if (template && (~template.indexOf('number.html') || ~template.indexOf('datetime.html') || ~template.indexOf(
			'calendar.html') || ~template.indexOf('select-native.html'))) {
			_filterType[field.name] = "=";
		} else if (fieldHasSelectTemplate(field)) {
			_filterType[field.outData[field.valueField]] = "=";
		} else if (template && ~template.indexOf('select-multiple.html')) {
			_filterType[field.name] = "IN";
		} else {
			_filterType[field.name] = "LIKE_I";
		}
	};

	var _filterTypeFlow = {
		"BETWEEN": function (query, value) {
			query.between([value.start, value.end]);
		},
		"NOT BETWEEN": function (query, value) {
			query.notBetween([value.start, value.end]);
		},
		"LIKE": function (query, value) {
			query.like('%' + value + '%');
		},
		"LIKE_I": function (query, value) {
			query.likeInsensitive('%' + value + '%');
		},
		"NOT LIKE": function (query, value) {
			query.notLike('%' + value + '%');
		},
		"LIKE ALL": function (query, value) {
			query.likeAll('%' + value + '%');
		},
		"<>": function (query, value) {
			query.notEquals(value);
		},
		"=": function (query, value) {
			query.equals(value);
		},
		">": function (query, value) {
			query.greater(value);
		},
		"<": function (query, value) {
			query.lower(value);
		},
		">=": function (query, value) {
			query.greaterOrEqual(value);
		},
		"<=": function (query, value) {
			query.lowerOrEqual(value);
		},
		"IN": function (query, value) {
			query.in(value);
		},
		"NOT IN": function (query, value) {
			query.notIn(value);
		}
	};

	this.buildFilter = function (fields, currentRow) {
		_filterType = {};
		skipFiltering = [];
		_buildFilterTypeMap(fields, currentRow);
		var query = Query.build();
		for (var key in currentRow) {
			if (typeof currentRow[key] == "boolean") {
				currentRow[key] = currentRow[key].toString();
			}
			if (currentRow[key] && _filterType[key] && (!Array.isArray(currentRow[key]) || currentRow[key].length > 0)) {
				if (!~skipFiltering.indexOf(key)) {
					query.where(key);
					_filterTypeFlow[_filterType[key]](query, currentRow[key]);
				}
			}
		}
		return query.where();
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register("ZhFilterBuilder", FilterBuilder);
});

})();

// FILE: src/FilterWidgetsCreator.js
function FilterWidgetsCreator(metaDataFactory, ScreenService) {

	this.createFiltersWidget = function (gridWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 1,
			"showAsModal": true,
			"container": gridWidget.container,
			"name": "popUpFiltersList",
			"label": ScreenService.i18n("Filters"),
			"isVisible": true,
			"template": "widget/grid.html",
			"showCheckbox": true,
			"actions": actionsFiltersListWidget,
			"styles": [],
			"floatingControl": {
				"refreshAction": false,
				"customizationAction": {
					"layout": false,
					"filter": false,
					"view": false
				},
				"actions": []
			},
			"events": [
				{
					"id": 2,
					"name": "WidgetOnEnter",
					"code": "args.owner.fillFilterList(args.owner);"
				},
				{
					"id": 3,
					"name": "WidgetAfterMoveRow",
					"code": "args.owner.openEditFilter(args.owner, args.row);"
				}
			],
			"dataSource": {
				"localStorage": false,
				"rest": false,
				"data": []
			},
			"fields": [
				{
					"id": 1,
					"name": "label",
					"label": ScreenService.i18n("Filter name"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"id": 2,
					"name": "filterTypeLabel",
					"label": ScreenService.i18n("Type"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"id": 4,
					"name": "defaultPreference",
					"isVisible": false,
					"showOnList": true,
					"label": ScreenService.i18n("Default"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"hideCheckAllButton": true,
					"events": [{
						"id": 1,
						"name": "FieldOnChange",
						"code": "args.owner.field.widget.setAsDefaultPreference(args.row, args.owner.field.widget);"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

	var actionsFiltersListWidget = [
		{
			"id": 2,
			"name": "NEW",
			"isVisible": true,
			"showAsAction": "always",
			"icon": "plus icon-white",
			"tooltip": ScreenService.i18n("New"),
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.openCreateNewFilterWidget(args.owner.widget)"
			}]
		},
		{
			"id": 3,
			"name": "cancelAction",
			"label": ScreenService.i18n("Close"),
			"showAsAction": "back",
			"hideIcon": true,
			"isVisible": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.closeFilterPopup()"
			}]
		},
		{
			"id": 4,
			"name": "deletePreference",
			"isVisible": true,
			"icon": "trash",
			"showAsAction": "checked_rows",
			"condition": "row.acl_edit",
			"tooltip": ScreenService.i18n("Delete preference"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.deletePreferenceClick(args.owner.widget);"
				}
			]
		}
	];

	this.createNewFilterWidget = function(gridWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": "335689",
			"isVisible": true,
			"name": "saveFilterPopup",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Create a new filter"),
			"template": "widget/form.html",
			"currentRow": {},
			"descriptionOrder": false,
			"fields": [
				{
					"id": 1,
					"label": ScreenService.i18n("Name"),
					"name": "name",
					"showOnList": true,
					"class": "6",
					"showOnForm": true,
					"events": [],
					"template": "field/text-edit.html",
					"validations": {
						"required": {
							"message": ScreenService.i18n("Required Field")
						}
					}
				},
				{
					"name": "type",
					"label": ScreenService.i18n("Type"),
					"class": "6",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data": []
					},
					"validations": {
						"required": {
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"actions": [{
					"id": 2,
					"name": "cancelAction",
					"label": ScreenService.i18n("Cancel"),
					"showAsAction": "back",
					"hideIcon": true,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
				}]
				}, {
					"id": 2,
					"name": "saveFilterAction",
					"label": ScreenService.i18n("Continue"),
					"showAsAction": "never",
					"hideIcon": true,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.createNewFilter(args.owner.widget)"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

	this.createChangeFilterTypeWidget = function(gridWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 8998,
			"name": "changePreferenceTypeWidget",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Change filter type"),
			"template": "widget/form.html",
			"showDescriptionOrder": false,
			"isVisible": true,
			"fields": [
				{
					"name": "filterType",
					"label": ScreenService.i18n("New filter type"),
					"class": "12",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data":[]
					},
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"events": [],
			"actions":[
				{
					"id": 1,
					"name": "canceladd",
					"label": ScreenService.i18n("Cancel"),
					"isVisible": true,
					"hideIcon": true,
					"showAsAction": "back",
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 1,
					"name": "tranferFilter",
					"label": ScreenService.i18n("Save"),
					"isVisible": true,
					"showAsAction": "never",
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "args.owner.widget.saveTransferFilterClick(args.owner.widget)"
						}
					]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

	var actionsPropertiesWidget = [
		{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.backToPreviousWidget(args.owner.widget)"
			}]
		}, {
			"label": ScreenService.i18n("OK"),
			"showAsAction": "never",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.saveFieldChanges(args.owner.widget, args.row)"
			}]
		}
	];

	this.createFormFilterWidget = function(gridWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": "31231231231",
			"isVisible": true,
			"name": "filterPopup",
			"orderFieldsBySequenceInForm": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Filter"),
			"template": "widget/form.html",
			"fieldGroups": (gridWidget.fieldGroups  || []).concat([{
				"id": 1,
				"name": "COLUMNS_FILTER",
				"opened": false,
				"collapsible": true,
				"label": ScreenService.i18n("More fields")
			}]),
			"fields": [],
			"actions": [
				{
					"id": 1,
					"name": "cancelAction",
					"label": ScreenService.i18n("Cancel"),
					"showAsAction": "back",
					"hideIcon": false,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 4,
					"name": "saveAndApplyFilterAction",
					"showAsAction": "never",
					"label": ScreenService.i18n("Save & Apply"),
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.saveAndApplyFilter(args.owner.widget)"
					}]
				}
			]
		});

		return Util.extend(widget, functionsToInject);
	};

	this.createCurrentFormFilterWidget = function(gridWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": "8514214420",
			"orderFieldsBySequenceInForm": true,
			"isVisible": true,
			"name": "filterPopup",
			"container": gridWidget.container,
			"label": ScreenService.i18n("Filter"),
			"template": "widget/form.html",
			"fieldGroups": (gridWidget.fieldGroups  || []).concat([{
				"id": 1,
				"name": "COLUMNS_FILTER",
				"opened": false,
				"collapsible": true,
				"label": ScreenService.i18n("More fields")
			}]),
			"events": Util.getNestedProperty(gridWidget, ['filterProperties', 'events']),
			"fields": [],
			"actions": [
				{
					"id": 1,
					"name": "cancelAction",
					"label": ScreenService.i18n("Back"),
					"showAsAction": "back",
					"hideIcon": false,
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "ScreenService.closePopup();"
					}]
				},
				{
					"id": 4,
					"name": "saveAndApplyFilterAction",
					"showAsAction": "never",
					"label": ScreenService.i18n("Filter"),
					"isVisible": true,
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.currentFilterClickPopup(args.owner.widget)"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	};

}

Configuration(function(ContextRegister) {
	ContextRegister.register("filterWidgetsCreator", FilterWidgetsCreator);
});

// FILE: src/FilterFloatingCardController.js
var FilterFloatingCardController = (function(_super) {
	__extends(FilterFloatingCardController1, _super);

	function FilterFloatingCardController1($scope, $rootScope, ScreenService, templateManager, ApplicationContext, restEngine, ZHPromise, eventAggregator, $timeout, ZhKeyboardService) {
		var _ref = FilterFloatingCardController.__super__.constructor.apply(this, arguments);

		var zhFilterPreferences = ApplicationContext.zhFilterPreferences;

		function getFilterFromSearchCriteria(filters, searchCriteria) {
			// remove itens starting with "SEARCHLIST_"
			filters = (filters || []).filter(function(filter) {
				return !~filter.name.indexOf("SEARCHLIST_");
			});
			if (searchCriteria) {			
				Object.keys(searchCriteria).forEach(function(key) {
					if (searchCriteria[key]) {				
						filters.push({
							name: "SEARCHLIST_"+key,
							operator: "LIKE",
							value: '%'+searchCriteria[key]+'%'
						});
					}
				});
			}
			return filters;
		}

		$scope.$watchCollection(function() { return $scope.widget.searchCriteria; }, function(current, old) {
			var newFilterApplied = getFilterFromSearchCriteria($scope.widget.filterApplied, current);
			if (!angular.equals($scope.widget.filterApplied, newFilterApplied)) {
				if ($scope.widget.customizationsApplied) {
					delete $scope.widget.customizationsApplied.filter;
				}

				$scope.widget.filterApplied = newFilterApplied;
				zhFilterPreferences.updateFilterInfo($scope.widget, newFilterApplied);
			}
		});

		$scope.clearSearch = function(event, fieldName) {
			if (!fieldName) {
				$scope.widget.searchCriteria = {};
			} else {
				delete $scope.widget.searchCriteria[fieldName];				
			}

			if (event) {
				Util.selectElement($(event.target).closest(".floating-card-input").find("input"));
			}
		};

		$scope.startFiltering = function() {
			zhFilterPreferences.startFiltering($scope.widget);
		};
		$scope.$on('control-info-icon-click', function(event, data) {
			if (data == 'filter') {
				$scope.startFiltering();
			} else {
				$scope.clearSearch();
			}
		});

		$scope.isSearchFieldSelect = function(field) {
			return field && ((field.filterProperties && field.filterProperties.template && ~field.filterProperties.template.indexOf('select-native.html')) || 
							 (field.template && ~field.template.indexOf('select-native.html')));
		};

		$scope.prepareField = function(field) {
			return zhFilterPreferences.prepareFields(Util.parseToArray(field))[0];
		};

		function isMobiScrollOpen() {
			return $('.mbsc-mobiscroll').length > 0;
		}

		$scope.openWidgetFilter = function(event, key) {
			if (!ScreenService.isSideMenuOpen() && !isMobiScrollOpen() && !ScreenService.isAlertShown()) {
				var widget = angular.element('.zh-floating-control:visible').last().scope().widget;
				if (widget.floatingControl && widget.floatingControl.customizationAction && widget.floatingControl.customizationAction.filter) {
					ApplicationContext.zhFilterPreferences.startFiltering(widget);
				}
			}

			// prevent default behaviour
			return false;
		};

		$scope.openWidgetSearch = function(event, key) {
			if (!ScreenService.isSideMenuOpen() && !isMobiScrollOpen() && !ScreenService.isAlertShown()) {
				var scope = angular.element('.zh-floating-control:visible').last().scope();
				scope.openSearchAction();
			}

			// prevent default behaviour
			return false;
		};

		$scope.initKeyboardShortCuts = function() {
			ZhKeyboardService.unbind(templateManager.project.shortcutKeys.openWidgetFilter || 'mod+l');
			ZhKeyboardService.unbind(templateManager.project.shortcutKeys.openWidgetSearch || 'mod+f');
			
			ZhKeyboardService.bind(templateManager.project.shortcutKeys.openWidgetFilter || 'mod+l', $scope.openWidgetFilter);
			ZhKeyboardService.bind(templateManager.project.shortcutKeys.openWidgetSearch || 'mod+f', $scope.openWidgetSearch);
		};
		$scope.initKeyboardShortCuts();

		return _ref;
	}

	return FilterFloatingCardController1;
})(CustomizationsFloatingCardController);

// FILE: src/FilterFloatingControlDirective.js
(function () {
	var ZeedhiDirectives = angular.module("ZeedhiDirectives");

	ZeedhiDirectives.directive(
		'zhFloatingControl',
		["templateManager", "ScreenService", "$timeout", "eventEngine", "ApplicationContext", "$window", "$rootScope", "ZhUtil", "$compile", "ZhKeyboardService", FilterFloatingControlDirective]
	);

	function FilterFloatingControlDirective(templateManager, ScreenService, $timeout, eventEngine, ApplicationContext, $window, $rootScope, zhUtil, $compile, ZhKeyboardService) {
		return {
			priority: 10,
			link: function($scope, element, attrs, controller) {
				var widget = $scope.widget;
				if (widget.floatingControl === false) return;

				var defaultOptions = {
					searchAction: Util.isUndefined(element.parent().attr('data-hide-search')) && Util.isUndefined(element.parent().attr('hide-search')),
					openSearch: false
				};
				widget.floatingControl = Util.extend(defaultOptions, widget.floatingControl);

				widget.floatingControl.addStatus({
					priority: 20,
					classString: 'searching',
					icon: 'search',
					conditionFunction: function() {
						return (widget.filterApplied && widget.filterApplied.length > 0) || Util.objectHasAnyValue($scope.searchCriteria);
					},
					infoTemplate: function() {
						var template;
						if (widget.floatingControl.customizationAction && widget.floatingControl.customizationAction.filter) {
							template = '<div class="searching-control-info"><span data-zh-icon="filter no-border icon-white" data-zh-touchstart="ControlInfoIconClick(\'filter\')"></span><div class="searching-control-info-details" data-ng-if="widget._filterInfo"><span data-ng-bind-html="widget._filterInfo"></span></div></div><div class="searching-control-info-text" data-ng-if="widget._filterInfo" data-zh-bind-translate="\'Filter applied\'"></div>';
						} else {
							template = '<div class="searching-control-info"><span data-zh-icon="close-x no-border icon-white" data-zh-touchstart="ControlInfoIconClick(\'clear\')"></span><div class="searching-control-info-details" data-ng-if="widget._filterInfo"><span data-ng-bind-html="widget._filterInfo"></span></div></div><div class="searching-control-info-text" data-ng-if="widget._filterInfo" data-zh-bind-translate="\'Filter applied\'"></div>';
						}
						return template;
					},
					scope: $scope
				});

				var isTheOnlyActionAvailable = function(propertyName){
					var actionsToCheck= ["configAction", "refreshAction", "xlsAction", "pdfAction", "actions", "searchAction"];
					var filtered = actionsToCheck.filter(function(property){
						var action = widget.floatingControl[property];
						return Util.isArray(action) ? action.length : action;
					});
					return (filtered.length == 1 && filtered[0] == propertyName);
				};

				$scope.$on('floating-control-before-close', function() {
					element.find('.search-action input').blur();
				});

				$scope.$on('floating-control-after-close', function() {
					if (element.hasClass('searching') || isTheOnlyActionAvailable("searchAction")) {
						element.find('.search-action').addClass('opened');
						Util.selectElement(element.find('.search-action input'));
					}
				});

				$scope.openSearchAction = function() {
					$timeout.cancel($scope._closeTimeout);
					if (!element.hasClass('opened')) {
						$scope.toggleActionMenu();
					}
					$scope.closeOpenedActions();
					var searchAction = element.find('.search-action').closest('.float-action');
					if (searchAction && searchAction.length > 0) {
						searchAction.find("span.search-action-icon").click();
						Util.selectElement(searchAction.find('input'));
					}
				};

				if (widget.floatingControl.openSearch) {
					$timeout(function() {
						var floatingControl = (ScreenService.isPopupOpen() ? $('.popup-container .container:visible') : $('.container:visible')).last().parent().find('.zh-floating-control');
						$scope.openSearchAction(floatingControl);
					}, 100);
				}

				$(window).off('click.floatingControl');
				$(window).on('click.floatingControl', function(e) {
					if (!element.find('.search-action').closest('.float-action').hasClass('opened')) {			
						if ($(e.target).parents('.zh-floating-control').length === 0) {
							widget.closeActionMenu();
						}
					}
				});

				$scope.filterFields = function(field) {
					return field.aclIsVisible && !field.skipSearch;
				};

				$scope.toggleSearchFieldsList = function(event) {
					// if filter action exists user cannot choose search field... it will be done in filter popup
					if (!widget.floatingControl.customizationAction || !widget.floatingControl.customizationAction.filter) {
						var el = $(event.target).closest(".floating-card").find('.floating-card-search-field-select');
						el.toggleClass("opened");

						if (el.hasClass("opened")) {
							$(event.target).closest(".floating-card").find('.floating-card-select-field-options').removeClass("opened");
							$(event.target).closest(".floating-card").find('.zh-select-search-floating').removeClass("opened");
						}
					}
				};

				$scope.toggleSelectFieldValueList = function(event) {
					var el = $(event.target).closest(".zh-select-search-floating");
					el.toggleClass("opened");
					var list = $(event.target).closest(".floating-card").find('.floating-card-select-field-options');
					list.toggleClass("opened");

					if (list.hasClass("opened")) {
						$(event.target).closest(".floating-card").find('.floating-card-search-field-select').removeClass("opened");
					}
				};

				$scope.selectSearchField = function(field) {
					$scope.searchField = field;
					element.find('.floating-card-search-field-select').removeClass('opened');
					element.find('input.zh-input-search-floating').focus();
				};

				$scope.selectSearchFieldValue = function(widget, event) {
					var dataValue = $(event.target).attr('data-value');
					var field = $scope.searchField;
					if (dataValue) {
						widget.searchCriteria = widget.searchCriteria || {};
						widget.searchCriteria[field.name] = field.dataSource.data[parseInt(dataValue)][field.valueField].toString();
					} else {
						delete widget.searchCriteria[field.name];
					}

					$(event.target).closest('.floating-card').find('.floating-card-select-field-options').removeClass('opened');
					$(event.target).closest('.floating-card').find('.zh-select-search-floating').removeClass('opened');
				};

				$scope.getSearchFieldValue = function(widget) {
					var field = $scope.searchField;
					var fieldValue;
					var row = field.dataSource.data.filter(function(row) {
						return widget.searchCriteria && widget.searchCriteria[field.name] == row[field.valueField].toString();
					});
					if (row.length > 0) {
						fieldValue = row[0][field.descriptionField];
					}
					return fieldValue;
				};

				if (widget.floatingControl.defaultSearchField) {
					$scope.searchField = widget.getField(widget.floatingControl.defaultSearchField);
				}
			}
		};
	}
})();


// FILE: src/ZhFilterPreferences.js
function ZhFilterPreferences(ScreenService, templateManager, ZhFilterBuilder, ZhUtil, RepositoryFactory, Query, eventEngine, ZHPromise, ZhLogger, ApplicationContext, zhCustomizationsService, filterWidgetsCreator, eventAggregator, preferencesUserService, DecodeValueFieldService) {

	var self = this;
	var gridWidget, filterDescription = [];

	eventAggregator.subscribe("OnListStartUp", getAndApplyDefaultPreference);

	function getAndApplyDefaultPreference(eventData) {
		if (eventData.widget) {
			var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
			self.getDefaultPreferenceByUser(eventData.widget, userRegistered).then(function(defaultPreference) {
				if (defaultPreference) {
					self.applyFilter(defaultPreference, eventData.widget);
				} else {
					eventData.widget.customizationsApplied = eventData.widget.customizationsApplied || {};
					eventData.widget.customizationsApplied.filter = "default";
				}
			});
		}
	}

	this.getDefaultPreferenceByUser = function(widget, userId) {
		return zhCustomizationsService.getWidgetCustomizations(widget, "filter").then(function(preferences) {
			return preferences.filter(function(preference) {
				return preference.mapUserIdDefault && preference.mapUserIdDefault[userId];
			}).shift();
		});
	};

	this.startFilterConfig = function startFiltering(widget) {
		gridWidget = widget;
		gridWidget.__filterConfig = filterWidgetsCreator.createFiltersWidget(gridWidget, functionsFilterListWidget);
		fillFilterList(gridWidget.__filterConfig).then(function() {
			ScreenService.openPopup(gridWidget.__filterConfig);
		});
	};

	this.restoreDefault = function(widget) {
		this.restoreDefaultFilter(widget);
	};

	function getFilterWidget(widget) {
		if (widget.__filterPopup === undefined) {
			widget.__filterPopup = filterWidgetsCreator.createCurrentFormFilterWidget(widget, functionsFormCurrentFilterWidget);
			widget.__filterPopup.showAsModal = false;
			widget.__filterPopup.label = ScreenService.i18n("Filter");
			widget.__filterPopup.fields = prepareFieldstoFormFilter(factoryFields(widget.__filterPopup));
			var searchFields = widget.hideSearchFieldsOnFilter ? [] : prepareFieldstoFormFilter(factorySearchFields(widget.__filterPopup));
			widget.__filterPopup.fields = widget.__filterPopup.fields.concat(searchFields);
			widget.__filterPopup.applyDefaultValue();
			var filters = widget.filterApplied;
			if (filters && filters.length || widget._filterRow) {
				widget.__filterPopup.currentRow = buildCurrentRowByFilters(filters, widget.__filterPopup, widget._filterRow);
			}
		}

		return widget.__filterPopup;
	}

	this.startFiltering = function(widget) {
		gridWidget = widget;
		ScreenService.openPopup(getFilterWidget(gridWidget));
	};

	var filterVisibleFields = function(fields) {
		return fields.filter(function(field) {
			return field.isVisible;
		});
	};

	function currentFilterClickPopup(widget) {
		if (widget.isValid()) {
			var currentRow = widget.currentRow;
			var fields = filterVisibleFields(gridWidget.__filterPopup.fields);
			var filter = {
				_filterRow: ZhUtil.clearRow(currentRow),
				filters: ZhFilterBuilder.buildFilter(fields, currentRow),
				_id: "edited"
			};
			self.applyFilter(filter, gridWidget);
			ScreenService.closePopup();
		}
	}

	function fillFilterList() {
		return zhCustomizationsService.getWidgetCustomizations(gridWidget, "filter").then(function(filters) {
			filters.forEach(function(filter) {
				filter.filterTypeLabel = zhCustomizationsService.getLabelRepositoryType(filter.customizationRepository);
			});
			gridWidget.__filterConfig.dataSource.data = filters;
			ScreenService.updateTemplate();
		});
	}

	this.restoreDefaultFilter = function(widget) {
		var editingWidget = widget || gridWidget;
		var filter = {
			"_id": "default",
			filters: this.getDefaultFilterProperties(editingWidget)
		};
		this.applyFilter(filter, editingWidget);
		this.updateFilterInfo(editingWidget, []);
	};

	function applyFilterEventCallback(eventData) {
		var filter = eventData.e.customization;
		var editingWidget = eventData.e.editingWidget;
		self.applyFilter(filter, editingWidget);
	}

	var fireBeforeBuildFilterEvent = function(editingWidget, filter) {
		var event = editingWidget.beforeGridFilterApply && editingWidget.beforeGridFilterApply(filter);
		return ZHPromise.when(event);
	};

	function _getFilterFields(filters) {
		return filters.filter(function(filter) {
			return !~filter.name.indexOf("SEARCHLIST_");
		});
	}

	function _getSearchFields(filters) {
		var searchFields = Util.clone(filters);
		searchFields = searchFields.filter(function(filter) {
			return ~filter.name.indexOf("SEARCHLIST_");
		});
		searchFields.map(function(filter) {
			filter.name = filter.name.replace("SEARCHLIST_","");
			filter.static = true;
		});
		return searchFields;
	}

	function _applySearchCriteria(searchFields, editingWidget) {
		if (editingWidget.setSearchCriteria) {
			editingWidget.setSearchCriteria(searchFields);
		}
	}

	this.applyFilter = function(filter, editingWidget, callback) {
		editingWidget._filterRow = filter._filterRow;
		var originalFilter = Util.clone(filter.filters);
		fireBeforeBuildFilterEvent(editingWidget, filter).then(function(event) {
			if (!event || !event.isDefaultPrevented()) {
				if (!editingWidget.customizationsApplied) {
					editingWidget.customizationsApplied = {};
				}
				createFilterDescription(editingWidget, originalFilter);
				editingWidget.filterApplied = originalFilter;
				editingWidget.customizationsApplied.filter = filter._id;

				var filterFields;
				var searchFields;
				if (filter.filters) {
					filterFields = _getFilterFields(filter.filters);
					searchFields = _getSearchFields(filter.filters);
				}

				if (filterFields && filterFields.length > 0) {
					filterFields.forEach(function(filter) {
						filter.static = true;
					});
				}
				if (editingWidget.dataSourceFilter) {
					editingWidget.dataSourceFilter = self.getStaticDataSourceFilters(editingWidget.dataSourceFilter);
					if (filterFields && filterFields.length > 0) {
						editingWidget.dataSourceFilter = editingWidget.dataSourceFilter.concat(filterFields);
					}
				}

				_applySearchCriteria(searchFields, editingWidget);

				if (editingWidget.afterGridFilterApply) {
					editingWidget.afterGridFilterApply();
				}

				editingWidget.reload(ZhUtil.getLikeAllFilter(editingWidget, editingWidget.searchCriteria)).then(function() {
					if (originalFilter) {
						self.updateFilterInfo(editingWidget, originalFilter);
					}
					if (callback) {
						callback();
					}
				});
			}
		});
	};

	this.preferencesListOnEnter = function(widgetPreferenceList) {
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		return zhCustomizationsService.getWidgetCustomizations(gridWidget, "filter").then(function(preferences) {
			preferences.forEach(function(preference) {
				preference.defaultPreference = preference.mapUserIdDefault[userRegistered] || false;
				preference.preferenceTypeLabel = zhCustomizationsService.getLabelRepositoryType(preference.customizationRepository);
			});
			widgetPreferenceList.dataSource.data = preferences;
		});
	};

	eventEngine.bindEvent(zhCustomizationsService, "FilterPreferenceOnClick", applyFilterEventCallback);

	var handleSelectMultipleDescription = function(field, currentRow) {
		if (field.template == "field/select-multiple.html" && field.dataSource.data.length > 0) {
			if (currentRow && currentRow[field.name]) {
				field.value(currentRow[field.name]);
			}
			if (field && Util.isArray(field.value())) {
				registers = field.dataSource.data.filter(function(register) {
					return ~field.value().indexOf(register[field.valueField]);
				});
			}
		} else {
			if (field.value() && Util.isArray(field.value()) && field.dataSource.data.length > 0) {
				registers = field.dataSource.data.filter(function(register) {
					return field.value() && register[field.valueField] == field.value();
				}).shift();
				if (registers && Util.isArray(registers)) {
					var teste = [];
					registers.forEach(function(register) {
						teste.push(register[field.descriptionField]);
					});
					return teste.join(", ") || currentRow[field.descriptionField];
				}
			}
		}
	};

	var oldDescriptionCreator = function(field, currentRow) {
		var labelDescription;
		if (field.descriptionField && field.valueField) {
			labelDescription = handleSelectMultipleDescription(field, currentRow);
		} else {
			if (field.outData) {
				labelDescription = currentRow[field.outData[field.valueField]];
			} else {
				labelDescription = currentRow[field.name];
			}
		}
		if (Util.isArray(labelDescription)) {
			labelDescription = labelDescription.map(function(value) {
				if (Util.isString(value)) {
					return value.replace(/^%|%$/g, "");
				}
				return value;
			}).join(" ");
		} else {
			if (Util.isString(labelDescription)) {
				labelDescription = labelDescription.replace(/^%|%$/g, "");
			}
		}
		return labelDescription;
	};

	var createFilterDescription = function(gridWidget, filters) {
		var fields = gridWidget.fields;
		var currentRow = buildCurrentRowByFilters(filters, gridWidget);
		filterDescription = [];
		if (!Util.isEmptyObject(currentRow)) {
			filterVisibleFields(fields).forEach(function(field) {
				var _field = Util.clone(field, ['widget', 'element', 'owner', 'selectWidget', 'intervalWidget']);
				_field.widget = field.widget;
				_field.template = (_field.filterProperties && _field.filterProperties.template) || _field.template;
				var labelDescription = DecodeValueFieldService.model(_field, currentRow[_field.name]) || oldDescriptionCreator(_field, filters);
				if (labelDescription) {
					filterDescription[field.name] = labelDescription;
				}
			});
		}
	};

	var openCreateNewFilterWidget = function(popupWidget) {
		var createNewFilterWidget = filterWidgetsCreator.createNewFilterWidget(gridWidget, functionsCreateNewFilterWidget);
		createNewFilterWidget.getField('type').dataSource.data = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		createNewFilterWidget.newRow();
		ScreenService.openPopup(createNewFilterWidget);
	};

	var createNewFilter = function(widget) {
		if (widget.isValid()) {
			var filterName = widget.currentRow.name;
			var customizationRepository = widget.currentRow.type;
			var preferenceHeader = createFilterHeader(filterName, customizationRepository);

			ScreenService.closePopup();
			templateManager.updateTemplate();
			var preferencesListWidget = gridWidget.__filterConfig;

			self.preferencesListOnEnter(preferencesListWidget).then(function() {
				openEditFilter(preferencesListWidget, preferenceHeader);
			});
		}
	};

	var createFilterHeader = function(filterName, customizationRepository) {
		var header = {
			"label": filterName,
			"widgetId": gridWidget.id,
			"containerId": gridWidget.container.id,
			"customizationRepository": customizationRepository,
			"unsync_id": new Date().getTime(),
			"customizationType": "filter",
			"mapUserIdDefault": {
				"default": false
			},
			"defaultPreference": false
		};
		return header;
	};

	var deletePreferenceClick = function(widget) {
		return zhCustomizationsService.removeCustomizations(widget.getCheckedRows()).then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				templateManager.updateTemplate();
			});
		});
	};

	var openPopupSelectDestinyRepository = function(filterListWidget) {
		var traferFilterTypeWidget = filterWidgetsCreator.createChangeFilterTypeWidget(gridWidget, functionsTranferFilterType);
		traferFilterTypeWidget.newRow();
		traferFilterTypeWidget.getField('filterType').dataSource.data = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		var toTransferLayouts = filterListWidget.getCheckedRows();
		var isSomeLayoutConnectedToAnVisuzalization = false;
		var promises = toTransferLayouts.map(function(filter) {
			return zhCustomizationsService.getCustomizationView(gridWidget, filter.customizationRepository, filter.customizationType, filter._id).then(function(connectedView) {
				if (connectedView) {
					isSomeLayoutConnectedToAnVisuzalization = true;
				}
			});
		});
		ZHPromise.all(promises).then(function() {
			if (!isSomeLayoutConnectedToAnVisuzalization) {
				ScreenService.openPopup(traferFilterTypeWidget);
			} else {
				ScreenService.showMessage(ScreenService.i18n("Filter connected with a customizations. It can't be transferred."));
			}
		});
	};

	var saveTransferFilterClick = function(widgetChangePreferenceType) {
		var newFilterType = widgetChangePreferenceType.getField("filterType").value();
		zhCustomizationsService.transferCustomizationsRepository(gridWidget, gridWidget.__filterConfig.getCheckedRows(), newFilterType).then(function() {
			fillFilterList(gridWidget.__filterConfig);
			ScreenService.closePopup();
		});
	};

	var openEditFilter = function(filterListWidget, row) {
		var filterFormWidget = filterWidgetsCreator.createFormFilterWidget(gridWidget, functionsFormFilterWidget);
		filterFormWidget.showAsModal = true;
		filterFormWidget.label = ScreenService.i18n("Filter") + " - " + row.label;
		filterFormWidget.filterSelected = row;
		filterFormWidget.fields = prepareFieldstoFormFilter(factoryFields(filterFormWidget));
		var searchFields = gridWidget.hideSearchFieldsOnFilter ? [] : prepareFieldstoFormFilter(factorySearchFields(filterFormWidget));
		filterFormWidget.fields = filterFormWidget.fields.concat(searchFields);
		filterFormWidget.currentRow = buildCurrentRowByFilters(row.filters, gridWidget, row._filterRow);
		ScreenService.openPopup(filterFormWidget);
	};

	var removeConflictingProperties = function(field) {
		var conflictingProperties = ['selectWidget'];
		conflictingProperties.forEach(function(property) {
			delete field[property];
		});
	};

	var factoryFields = function(filterForm) {
		return gridWidget.fields.filter(function(field) {
			return !Util.isUndefined(field.template);
		}).map(function(field) {
			var factoredField = metaDataFactory.fieldFactory(field);
			removeConflictingProperties(factoredField);
			factoredField.widget = filterForm;
			return factoredField;
		});
	};

	var factorySearchFields = function(filterForm) {
		var fields = [];
		fields.push(metaDataFactory.fieldFactory({
			"id": "SEARCHLIST__ALL",
			"label": ScreenService.i18n("All Fields"),
			"name": "SEARCHLIST__ALL",
			"class": 12,
			"template": "field/text-edit.html",
			"isVisible": true,
			"showOnList": true,
			"descending": true,
			"fieldGroup": "COLUMNS_FILTER"
		}, filterForm));

		fields = fields.concat(gridWidget.fields.filter(function(field) {
			return !field.skipSearch && Util.isUndefined(field.template) && !filterForm.getField(field.name);
		}).map(function(field) {
			var factoredField = metaDataFactory.fieldFactory(field);
			delete factoredField.$$hashKey;
			factoredField.id = "SEARCHLIST_"+factoredField.id;
			factoredField.name = "SEARCHLIST_"+factoredField.name;
			factoredField.class = 6;
			factoredField.isVisible = true;
			factoredField.readOnly = false;
			factoredField.validations = {};
			factoredField.template = "field/text-edit.html";
			factoredField.fieldGroup = "COLUMNS_FILTER";
			factoredField.widget = filterForm;
			return factoredField;
		}));

		return fields;
	};

	var prepareFieldstoFormFilter = this.prepareFields = function(fields) {
		return fields.map(function(field) {
			return field.filterProperties ? mergeFieldNewProperties(field, field.filterProperties) : field;
		});
	};

	var saveAndApplyFilter = function(filterFormWidget) {
		if (filterFormWidget.isValid()) {
			saveFilter(filterFormWidget).then(function() {
				self.applyFilter(filterFormWidget.filterSelected, gridWidget, function() {
					ScreenService.closePopup();
				});
			});
		}
	};

	var saveFilter = function(filterFormWidget) {
		filterFormWidget.filterSelected._filterRow = ZhUtil.clearRow(filterFormWidget.currentRow);
		filterFormWidget.filterSelected.filters = ZhFilterBuilder.buildFilter(filterFormWidget.fields.slice(), filterFormWidget.currentRow);
		removeNotInterestingProperties(filterFormWidget.filterSelected);
		return zhCustomizationsService.saveCustomization(filterFormWidget.filterSelected, 'filter').then(function() {
			self.preferencesListOnEnter(gridWidget.__filterConfig).then(function() {
				ScreenService.closePopup();
				templateManager.updateTemplate();
			});
		});
	};

	var removeNotInterestingProperties = function(object) {
		Object.keys(object).forEach(function(key) {
			if (~key.indexOf('__')) {
				delete object[key];
			}
		});
	};

	var applyFilterActionOnClick = function(filter) {
		self.applyFilter(filter, gridWidget);
		ScreenService.closePopup(true);
		gridWidget.activate();
	};

	var mergeFieldNewProperties = function(field, filterProperties) {
		var keys = Object.keys(field);
		keys = keys.concat(
			Object.keys(filterProperties).filter(function(key) {
				return keys.indexOf(key) == -1;
			})
		);
		keys.forEach(function(property) {
			field[property] = filterProperties.hasOwnProperty(property) ? filterProperties[property] : field[property];
		});
		if (field.dataSource && !(field.dataSource instanceof DataSource)) {
			field.dataSource = metaDataFactory.factoryDataSource(field.dataSource, field.dataSourceFilter, field);
		}
		return field;
	};

	var buildCurrentRowByFilters = function(filters, gridWidget, descriptionValues) {
		filters = Util.clone(filters) || [];
		var builtCurrentRow = descriptionValues || {};

		if (gridWidget && gridWidget.fields) {
			gridWidget.fields.forEach(function(field) {
				var fieldFilter = getFieldFilter(field.name, filters);
				if (fieldFilter) {
					if (fieldFilter.operator === 'LIKE' || fieldFilter.operator === 'LIKE_I') {
						if (fieldFilter.value[fieldFilter.value.length - 1] == '%') {
							fieldFilter.value = fieldFilter.value.substring(0, (fieldFilter.value.length - 1));
						}
						if (fieldFilter.value[0] == '%') {
							fieldFilter.value = fieldFilter.value.substring(1, fieldFilter.value.length);
						}
					} else if (fieldFilter.operator && ~fieldFilter.operator.indexOf("BETWEEN")) {
						fieldFilter.value = {
							start: fieldFilter.value[0],
							end: fieldFilter.value[1],
						};
					}
					builtCurrentRow[field.name] = fieldFilter.value;
					/*
					 dumb reload, should remove this in the next release!
					*/
					field.reload();
				}
			});
		}

		// add SearchList values
		filters.filter(function(filter) {
			return ~filter.name.indexOf("SEARCHLIST_");
		}).forEach(function(filter) {
			var filterValue = filter.value;
			if (filterValue[filterValue.length - 1] == '%') {
				filterValue = filterValue.substring(0, (filterValue.length - 1));
			}
			if (filterValue[0] == "%") {
				filterValue = filterValue.substring(1, filterValue.length);
			}
			builtCurrentRow[filter.name] = filterValue;
		});
		return builtCurrentRow;
	};

	this.inArray = function(elem, array) {
		return -1 != (Util.isArray(array) ? array.indexOf(elem) : -1);
	};

	var getFieldFilter = function(fieldName, filters) {
		return filters.filter(function(filter) {
			return filter.name == fieldName;
		}).shift();
	};

	var closeFilterPopup = function() {
		ScreenService.closePopup();
		gridWidget.activate();
		if (gridWidget.afterGridFilterCancel) {
			gridWidget.afterGridFilterCancel();
		}
	};

	this.updateFilterInfo = function(gridWidget, filters) {
		var stripeFilterFields = [];
		var currentFilter = buildCurrentRowByFilters(filters, gridWidget);
		var filterLabel = "";
		gridWidget._filterInfo = filterLabel;

		if (gridWidget.customizationsApplied && gridWidget.customizationsApplied.filter && gridWidget.customizationsApplied.filter != "edited" && gridWidget.customizationsApplied.filter != "default") {
			zhCustomizationsService.getWidgetCustomizations(gridWidget, "filter").then(function(filters) {
				filters.filter(function(filter) {
					if (filter._id == gridWidget.customizationsApplied.filter) {
						filterLabel = "<b>" + ScreenService.i18n("Filter") + ":</b> " + filter.label;
						gridWidget._filterInfo = filterLabel;
						ScreenService.updateTemplate();
					}
				});
			});
		} else {
			filterLabel = [];
			if (Util.isArray(filters)) {
				filters.forEach(function(filter) {
					var field = gridWidget.getField ? gridWidget.getField(filter.name) : undefined;
					if (field) {
						if (hasSelectTemplate(field)) {
							currentFilter[filter.name] = getSelectFieldDescriptionValue(field, currentFilter[filter.name]);
						}
						var fieldLabel = ScreenService.i18n(field.label);
						var nextField;
						if (filterDescription[field.name]) {
							nextField = "<b>" + fieldLabel + ":</b> " + filterDescription[field.name];
						} else if (filter.operator == "BETWEEN") {
							nextField = "<b>" + fieldLabel + ":</b> " + currentFilter[filter.name].start + ' - ' + currentFilter[filter.name].end;
						} else {
							nextField = "<b>" + fieldLabel + ":</b> " + currentFilter[filter.name];
						}

						filterLabel.push(nextField);
					} else if (~filter.name.indexOf("SEARCHLIST_")) {
						if (filter.name == "SEARCHLIST__ALL") {
							filterLabel.push("<b>" + ScreenService.i18n("All Fields") + ":</b> " + currentFilter[filter.name]);
						} else {
							field = gridWidget.getField ? gridWidget.getField(filter.name.substring(11)) : undefined;
							if (field) {
								if (hasSelectTemplate(field)) {
									currentFilter[filter.name] = getSelectFieldDescriptionValue(field, currentFilter[filter.name]);
								}
								filterLabel.push("<b>" + ScreenService.i18n(field.label) + ":</b> " + currentFilter[filter.name]);
							}
						}
					}
				});
			} else if (Util.isObject(filters)) {
				Object.keys(filters).forEach(function(key) {
					if (key == "_ALL") {
						filterLabel.push("<b>" + ScreenService.i18n("All Fields") + ":</b> " + filters[filter.name]);
					} else {
						field = gridWidget.getField(key);
						if (field) {
							if (field.template && hasSelectTemplate(field)) {
								filters[key] = getSelectFieldDescriptionValue(field, filters[key]);
							}
							filterLabel.push("<b>" + ScreenService.i18n(field.label) + ":</b> " + filters[key]);
						}
					}
				});
			}

			filterLabel = filterLabel.join("<br />");
			gridWidget._filterInfo = filterLabel;
			ScreenService.updateTemplate();
		}
	};

	var hasSelectTemplate = function(field) {
		return field.template && ~field.template.indexOf('select-native.html');
	};

	var getSelectFieldDescriptionValue = function(field, value) {
		if (field.dataSource && field.dataSource.data) {
			var selectedRow = field.dataSource.data.filter(function(row) {
				return value == row[field.valueField].toString();
			});
			if (selectedRow.length > 0) {
				value = selectedRow[0][field.descriptionField];
			}
		}
		return value;
	};

	var factoryStripeFields = function(stripe, field) {
		factoredField = metaDataFactory.fieldFactory(field);
		factoredField.widget = stripe;
		if (stripe.__originalFields) {
			stripe.fields = stripe.__originalFields.concat([factoredField]);
		} else {
			stripe.fields = factoredField;
		}
	};

	var defaultFilterProperties = {};

	this.setFilterProperties = function(gridWidget, currentRow) {
		var filter = {
			"_id": "default",
			filters: ZhFilterBuilder.buildFilter(gridWidget.fields, currentRow)
		};
		self.applyFilter(filter, gridWidget);
	};

	this.setDefaultFilterProperties = function(widget, currentRow) {
		gridWidget = widget;
		defaultFilterProperties[gridWidget.id] = ZhFilterBuilder.buildFilter(gridWidget.fields, currentRow);
		if (!gridWidget.filterApplied || !gridWidget.filterApplied.length) {
			self.setFilterProperties(gridWidget, currentRow);
		}
	};

	this.getDefaultFilterProperties = function(gridWidget) {
		return defaultFilterProperties[gridWidget.id];
	};

	this.getStaticDataSourceFilters = function(dataSourceFilter) {
		return dataSourceFilter.filter(function(filter) {
			return !filter.static;
		});
	};

	var setAsDefaultPreference = function(row, widget) {
		var toUpdate = [];
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		var preferenceSetAsDefault = widget.dataSource.data.filter(function(preference) {
			return preference.mapUserIdDefault[userRegistered] === true && preference._id !== row._id;
		}).shift();
		if (preferenceSetAsDefault) {
			preferenceSetAsDefault.mapUserIdDefault[userRegistered] = false;
			removeNotInterestingProperties(preferenceSetAsDefault);
			toUpdate.push(preferenceSetAsDefault);
		}
		removeNotInterestingProperties(row);
		row.mapUserIdDefault[userRegistered] = !row.mapUserIdDefault[userRegistered];
		toUpdate.push(row);
		zhCustomizationsService.saveCustomization(toUpdate, 'filter').then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				ScreenService.hideLoader();
				ScreenService.updateTemplate();
			});
		});
	};

	var functionsFilterListWidget = {
		"fillFilterList": fillFilterList,
		"openEditFilter": openEditFilter,
		"openCreateNewFilterWidget": openCreateNewFilterWidget,
		"deletePreferenceClick": deletePreferenceClick,
		"openPopupSelectDestinyRepository": openPopupSelectDestinyRepository,
		"applyFilterActionOnClick": applyFilterActionOnClick,
		"closeFilterPopup": closeFilterPopup,
		"setAsDefaultPreference": setAsDefaultPreference,
		"restoreDefaultFilter": self.restoreDefaultFilter,
		"preferencesListOnEnter": self.preferencesListOnEnter
	};

	var functionsCreateNewFilterWidget = {
		"createNewFilter": createNewFilter
	};

	var functionsTranferFilterType = {
		"saveTransferFilterClick": saveTransferFilterClick
	};

	var functionsFormFilterWidget = {
		"saveAndApplyFilter": saveAndApplyFilter,
		"saveFilter": saveFilter
	};

	var functionsFormCurrentFilterWidget = {
		"currentFilterClickPopup": currentFilterClickPopup
	};

}

Configuration(function(ContextRegister) {
	var zhFilterPreferences;
	ContextRegister.register("zhFilterPreferences", function(ScreenService, templateManager, ZhFilterBuilder, ZhUtil, RepositoryFactory, Query, eventEngine, ZHPromise, ZhLogger, ApplicationContext, zhCustomizationsService, filterWidgetsCreator, eventAggregator, preferencesUserService, DecodeValueFieldService) {
		zhFilterPreferences = new ZhFilterPreferences(ScreenService, templateManager, ZhFilterBuilder, ZhUtil, RepositoryFactory, Query, eventEngine, ZHPromise, ZhLogger, ApplicationContext, zhCustomizationsService, filterWidgetsCreator, eventAggregator, preferencesUserService, DecodeValueFieldService);
		return zhFilterPreferences;
	});
	ContextRegister.register("ZhGridFilter", zhFilterPreferences);
});

// FILE: ./bower_components/zh-preferences/dist/preferences.js
// FILE: src/ZhPreferences.js
// FILE: src/ZhPreferences.js
/**
 * Service to handle preferences storage
 *
 * @class
 * @param {RepositoryFactory} RepositoryFactory Service to factory repositories
 * @param {RequestEngine}     requestEngine     Service to make http requests
 * @param {RequestFactory}    requestFactory    Service to factory requests
 * @param {ZHPromise}         ZHPromise         Service to manipulate promises
 * @param {Query}             Query             Service to build queries
 */
function ZhPreferences(RepositoryFactory, requestEngine, requestFactory, ZHPromise, Query, $q) {

    var repositoryMap = {};

    var userWidgetConfiguration = {};

    /**
     * Register new repository to save preferences
     *
     * @param  {String} name Repository name used to save a preference
     */
    this.registerRepository = function(name) {
        var repositoryName = name.split('#').pop();
        repositoryMap[repositoryName] = RepositoryFactory.factory(name, 'MEMORY');
    };


    /**
     * Persist preferences
     *
     * @param  {String}   preferenceType Repository name
     * @param  {Object[]} preferences    Array of preferences to persist
     *
     * @return {Promise}                 Preferences saved
     */
    this.savePreferences = function(preferenceType, preferences) {
        return requestEngine.doRequest(requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType + '/save',
            'requestType': 'dataSource',
            'dataSource': preferences
        })).then(null, function(response) {
            throw new Error(response.data.error);
        }).then(function(response) {
            var returnedIds = response.dataset[preferenceType];
            return preferences.map(function(preference, index) {
                preference.__is_new = false;
                preference.changed = false;
                preference.ID = returnedIds[index].ID;
                return preference;
            });
        }).then(function(preferences) {
            return repositoryMap[preferenceType].save(preferences);
        });
    };

    /**
     * Delete preferences
     *
     * @param  {String}   preferenceType Repository name
     * @param  {Object[]} preferences    Array of preferences to delete
     *
     * @return {Promise}                 Preferences deleted
     */
    this.deletePreferences = function(preferenceType, preferences) {
        return requestEngine.doRequest(requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType + '/remove',
            'requestType': 'dataSource',
            'dataSource': preferences
        })).then(null, function(response) {
            throw new Error(response.data.error);
        }).then(function(response) {
            var deletedIds = response.dataset[preferenceType].map(function(preference) {
                return preference.ID;
            });
            var query = Query.build().where('ID').in(deletedIds);

            return repositoryMap[preferenceType].remove(query);
        });
    };

    /**
     * Get local saved preferences from repository
     *
     * @param  {String} preferenceType Repository name
     * @param  {Query}  query          Conditions to filter the preferences
     *
     * @return {Promise}               Array of preferences
     */
    this.getPreferences = function(preferenceType, query) {
        var filterRequest = requestFactory.factory({
            'serviceName': 'zh-preferences#/' + preferenceType,
            'requestType': 'filterData',
            'filter': query.where()
        });
        return requestEngine.doRequest(filterRequest)
            .then(function(response) {
                var preferences = response.dataset[preferenceType];
                var removePreferences = repositoryMap[preferenceType].remove(query);
                var savePreferences = repositoryMap[preferenceType].save(preferences);
                return ZHPromise.all([removePreferences, savePreferences]).then(function(){
                    return preferences;
                });
            })
            .catch(function(response) {
                throw new Error(response.data.error);
            });
    };

    /**
     * Get local saved preferences from repository filtering by widget and logged user
     *
     * @param  {String} preferenceType Repository name
     * @param  {String} widgetID       Widget identified
     *
     * @return {Promise}               Array of preferences
     */
    this.getWidgetPreferences = function(preferenceType, widgetID) {
        var query = Query.build().where('WIDGET_ID').equals(widgetID);
        return this.getPreferences(preferenceType, query);
    };

    /**
     * Get default preferences for user on specified widget
     *
     * @param  {String} widgetID Widget to get default preferences
     *
     * @return {Promise}         Widget default preferences
     */
    this.getDefaultWidgetPreferences = function(widgetID) {
        if (userWidgetConfiguration[widgetID] === undefined) {
            var query = Query.build().where('WIDGET_ID').equals(widgetID);

            userWidgetConfiguration[widgetID] = requestEngine.doRequest(requestFactory.factory({
                'serviceName': 'zh-preferences#/default-preferences',
                'requestType': 'filterData',
                'filter': query.where()
            })).then(function(response) {
                return response.dataset.user_widget.pop() || {
                    '__is_new': true,
                    'ID': null,
                    'WIDGET_ID': widgetID,
                    'DEFAULT_FILTER': null,
                    'DEFAULT_LAYOUT': null,
                    'DEFAULT_VIEW': null,
                    'LAST_MODIFIED_DATE': null
                };
            });
        }

        return userWidgetConfiguration[widgetID];
    };

    /**
     * Change default preference for user on specified widget
     *
     * @param  {String}  configWidget        Preference configuration widget
     * @param  {String}  property            Property that contains the default preference ID
     * @param  {Integer} preference          Preference configuration widget changed row
     *
     * @return {Promise}                     Widget default preferences
     */
    this.changeDefaultPreferences = function(configWidget, property, preference){
        var widgetID = configWidget.listWidget.id;
        var preferenceID = preference[property] ? preference.ID : null;

        userWidgetConfiguration[widgetID] = this.getDefaultWidgetPreferences(widgetID)
            .then(function(defaultPreferences) {
                defaultPreferences[property] = preferenceID;
                return ZHPromise.all([
                    defaultPreferences,
                    requestEngine.doRequest(requestFactory.factory({
                        'serviceName': 'zh-preferences#/default-preferences/save',
                        'requestType': 'dataSource',
                        'dataSource': [defaultPreferences]
                    }))
                ]);
            }).then(function(args) {
                var defaultPreferences = args[0];
                var response = args[1];
                defaultPreferences.ID = Util.getNestedProperty(response, ['dataset', 'user_widget', '0', 'ID']);

                configWidget.dataSource.data.forEach(function(row){
                    row.__is_new = false;
                    row[property] = row.ID == defaultPreferences[property];
                });
                return defaultPreferences;
            });

        return userWidgetConfiguration[widgetID];
    };

    function isMasterDetail(template){
        return template.indexOf('master_detail/grid.html') !== -1 ||
               template.indexOf('master_detail/list.html') !== -1 ||
               template.indexOf('master_detail/tree/grid.html') !== -1;
    }

    function isMasterDetailView(widget){
        return isMasterDetail(widget.template) && widget.mode == ViewMode.VIEW;
    }

    this.validateWidget = function(widget){
        if (!widget.id) {
            throw new Error('Widget\'s must have "id" to use Preferences. '+
                'Widget with name "'+widget.name+'" does not have an id.');
        }
        if (!widget.container.id) {
            throw new Error('Container\'s must have "id" to use Preferences.');
        }

        return !isMasterDetailView(widget);
    };

    var userPromise = ZHPromise.defer();

    /**
     * Invalidate logged user and clear preferences loaded
     */
    this.unsetUser = function() {
        userPromise = ZHPromise.defer();
        userWidgetConfiguration = {};

        Object.keys(repositoryMap).forEach(function(repositoryName) {
            repositoryMap[repositoryName].clearAll();
        });
    };

    /**
     * Set logged user
     *
     * @param {String} userID Logged user
     */
    this.setUserID = function(userID) {
        userPromise.resolve(userID);
    };

    /**
     * Get logged user ID
     *
     * @return {Promise} User identifier
     */
    this.getUserID = function() {
        return userPromise.promise;
    };


    /**
     * Generate preference file to download
     *
     * @param {Object} preference       Preference to be exported
     * @param {String} fileName         File name
     *
     * @return {Promise<Object>}        Preference exported
     */
    this.downloadPreferenceFile = function(preference, fileName) {
        var dataStr = "data:text/json;charset=utf-8," + encodeURIComponent(JSON.stringify(preference,false,4));

        var linkElement = document.createElement('a');
        linkElement.setAttribute("href",     dataStr);
        linkElement.setAttribute("download", fileName);
        linkElement.click();
        linkElement.remove();

        return ZHPromise.when(preference);
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhPreferences', ZhPreferences);
});

// FILE: src/Config.js
(function(){

window.metaDataUrls = window.metaDataUrls || [];
window.serviceUrls = window.serviceUrls || [];

var PACKAGE_NAME = 'zh-preferences';

var hasMetaData = window.metaDataUrls.some(function(metaDataUrl){
	return metaDataUrl.package === PACKAGE_NAME;
});

var hasServiceUrl = window.metaDataUrls.some(function(metaDataUrl){
    return metaDataUrl.package === PACKAGE_NAME;
});

if (!hasMetaData) {
    window.metaDataUrls.push({
        package: PACKAGE_NAME,
        baseUrl: 'bower_components/zh-preferences/dist/assets/'
    });
}

if (!hasServiceUrl) {
    window.serviceUrls.push({
        package: PACKAGE_NAME,
        baseUrl: '../backend/service/index.php/zh-preferences'
    });
}

})();


// FILE: ./bower_components/zhuaclfrontend/assets/dist/main.js
// FILE: assets/js/util/ZhuAclConfig.js
/* global Configuration */
function ZhuAclConfig() {
	var self = this;
	this.RULE_ALLOW = 'A';
	this.RULE_READONLY = 'R';
	this.RULE_INVISIBLE = 'I';

	this.exportRulesModal = 'exportRulesModal';
	this.menuElementsListWidgetName = 'elementsListWidget';
	this.containerElementsListWidgetName = 'containerElementsModal';
	this.containerElementsTreeWidgetName = 'containerElementsTree';

	this.containerType = 'container';

	this.getDefaultDataSourceFilter = function(groupIds, projectId, organizationId){
		var dataFilter = [];
		var groupFilter = {
			"name": "GROUP_ID",
			"operator": "=",
			"value": groupIds
		};
		var projectFilter = {
			"name": "PROJECT_ID",
			"operator": "=",
			"value": projectId
		};

		if(organizationId !== undefined){
			var organizationFilter = {
				"name": "ORGANIZATION_ID",
				"operator": "=",
				"value": organizationId
			};
			dataFilter.push(organizationFilter);
		}

		dataFilter.push(groupFilter);
		dataFilter.push(projectFilter);
		return dataFilter;
	};

	this.getAclMap = function(type){
		var aclMap = {
			"container": {
				"aclTypeCod": "C",
				"childs": ["widgets"]
			},
			"widgets": {
				"aclTypeCod": "W",
				"childs": ["widgets", "fields", "actions"]
			},
			"fields": {
				"aclTypeCod": "F",
				"childs": []
			},
			"actions": {
				"aclTypeCod": "A",
				"childs": []
			}
		};
		return aclMap[type] || {};
	};

	this.getRuleType = function(isVisible, isEditable){
		var ruleType;
		if(!isVisible){
			ruleType = self.RULE_INVISIBLE;
		} else if(!isEditable){
			ruleType = self.RULE_READONLY;
		} else {
			ruleType = self.RULE_ALLOW;
		}
		return ruleType;
	};

	this.getRuleConfiguration = function(ruleType){
		var ruleConfig = {
			isVisible: ruleType != self.RULE_INVISIBLE,
			isEditable: ruleType == self.RULE_ALLOW
		};
		return ruleConfig;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('ZhuAclConfig', ZhuAclConfig);
});

// FILE: assets/js/util/ZhuAclMessages.js
var ZhuAclMessages = {
	CANCEL: 'Cancelar',
	YES: 'Sim',
	NO: 'Não',
	VISIBLE: 'visíveis',
	EDITABLE: 'editáveis',
	INVISIBLE: 'invisíveis'
};
(function(){
	var mappedMessages = [
		"ACCESS_DENIED",
		"RULE_NOT_SAVED",
		"SUC_RULE_INSERTED",
		"ERR_RULE_INSERTED",
		"SUC_RULES_EXPORTED",
		"ERR_RULES_EXPORTED",
		"CANNOT_LOAD_RULES",
		"CONFIRMATION_MESSAGE",
		"CONFIRM_CHILDS_QUESTION",
		"CONFIRM_EXPORT",
		"NSHOW_WITHOUT_ID",
		"NSHOW_WITHOUT_CONTAINER_CONFIG",
		"NSHOW_WITHOUT_ID_AND_CONTAINER_CONFIG"
	];
	mappedMessages.forEach(function addMappedMessages(code){
		ZhuAclMessages[code] = 'zhuacl#' + code;
	});
})();

// FILE: assets/js/services/ZhuAccessButtonService.js
/* global _Notification, Util, ZeedhiServices */
function ZhuAccessButtonService(){
	this.insertButtonInProject = function(project){
		var notification = {
			"label": "Access Control",
			"name": "accessControl",
			"icon": "block",
			"isVisible": false,
			"tooltip": "Access Control",
			"isAclNotification": true,
			"events": [
				{
					"name": "NotificationEvent",
					"code": "ZhuAclController.openAclModalWidget();"
				}
			]
		};
		try {
			notification = Util.extend(new _Notification(), notification);
			project.notifications = project.notifications || [];
			project.notifications.push(notification);
		} catch (e){
			console.error('O Controle de acesso à elementos só funciona em versões acima da 1.40.x do Zeedhi Framework');
		}
	};

	this.disableButtonInProject = function(project){
		if(project.notifications){
			project.notifications = project.notifications.filter(function(notification){
				return !notification.isAclNotification;
			});
		}
	};

	this.enableButtonInProject = function(project){
		if(project.notifications){
			project.notifications.forEach(function(notification){
				if(notification.isAclNotification){
					notification.isVisible = true;
				}
			});
		}
	};
}

ZeedhiServices.service("ZhuAccessButtonService", function(ApplicationContext){
	var instance = new ZhuAccessButtonService();
	ApplicationContext.registerInstance("ZhuAccessButtonService", instance);
	return instance;
});


// FILE: assets/js/services/ZhuAclService.js
/* global Util, Configuration, ZhuAclMessages*/
function ZhuAclService(requestDataSourceEngine, restEngine, ZhuAclConfig){

	this.getZhuAclRules = function(projectId, groupIds){
		return requestDataSourceEngine.getDataSource('zhuacl#/acl_zhu_acl_rules', ZhuAclConfig.getDefaultDataSourceFilter(groupIds, projectId), 1, undefined, undefined, undefined, 120000).then(function(response){
			var dataset = response.dataset || {};
			var rules = dataset.zhu_access_control_rule_login;
			if(Util.isArray(rules)){
				var objToReturn = {
					aclRules: rules,
					defaultRule: dataset.zhu_access_control_default_rule.defaultRule,
				};
				return objToReturn;
			} else {
				throw {code: ZhuAclMessages.CANNOT_LOAD_RULES};
			}
		});
	};

	this.getWidgetMetadata = function(widgetName){
		var widgetUrl = 'zhuacl#json/'+ widgetName +'.json';
		return restEngine.requestMetaData(widgetUrl).then(function(metadata){
			return metadata.data;
		});
	};

	this.exportRules = function(profileFrom, profilesTo){
		var rows = [{
				changed: true,
				profileFrom: profileFrom,
				profilesTo: profilesTo
		}];
		return requestDataSourceEngine.setDataSource('zhuacl#/acl_zhu_acl_element_exportRules', rows);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclService', ZhuAclService);
});

// FILE: assets/js/controllers/ZhuAclController.js
/* global openWindow, Util, Configuration, ZhuAclMessages */
function ZhuAclController(ScreenService, templateManager, metaDataFactory, ZHPromise, ZhuAclService, ZhuAccessButtonService, ZhuAclConfig, MessageService, ApplicationContext, eventAggregator){
	var self = this;
	var user;
	var validProperties = ['widgets', 'fields', 'actions'];
	var loadedContainers = {};

	function User(userId, projectId, organizationId, defaultRule){
		var currentGroupId;
		var userDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;

		this.setCurrentGroupId = function(id){
			currentGroupId = id;
		};

		this.getDefaultRule = function(){
			return userDefaultRule;
		};

		this.getCurrentGroupId = function(){
			return currentGroupId;
		};

		this.getProjectId = function(){
			return projectId;
		};

		this.getUserId = function(){
			return userId;
		};

		this.getOrganizationId = function(){
			return organizationId;
		};
	}

	this.getDefaultRule = function(){
		return user.getDefaultRule();
	};

	this.getCurrentGroupId = function(){
		return user.getCurrentGroupId();
	};

	this.getProjectId = function(){
		return user.getProjectId();
	};

	var prepareLabelToName = function(label){
		var name = label.toLowerCase();
		name = name.replace(/\ /g, "_");
		return name.replace(/[^\w\s]/gi, '');
	};

	var translateLabel = function(label){
		return ScreenService.i18n(label);
	};

	this.getRulesFromDatasource = function(rulesData){
		var userId = user.getUserId();
		var rules = {};
		rulesData.forEach(function(row){
			var obj = {
				ruleRow: row,
				userGroups: {}
			};
			obj.userGroups[userId] = {
				rule: row.RULE
			};
			rules[row.ELEMENT_UNIQUE_ID] = obj;
		});
		return rules;
	};

	this.getElementRule = function(rules, itemId){
		var itemRule = {};
		if(rules[itemId]){
			itemRule = rules[itemId].ruleRow;
		}
		return itemRule;
	};

	this.getPlannedItemToInsertInDataSet = function(rules, item, itemType, parentId){
		var uniqueId = item.id;
		var itemRule = self.getElementRule(rules, uniqueId);
		var itemToInsert = {
			"UNIQUE_ID": uniqueId,
			"NAME": item.name || prepareLabelToName(item.label),
			"LABEL": translateLabel(item.label || item.name),
			"TYPE": itemType,
			"RULE": itemRule.RULE || this.getDefaultRule(),
			"RULE_ID": itemRule.ID,
			"PARENT_ID": parentId,
			"USER_ID": user.getUserId(),
			"PROJECT_ID": user.getProjectId(),
			"ORGANIZATION_ID": user.getOrganizationId()
		};

		itemToInsert.isVisible = item.aclIsVisible;
		itemToInsert.isEditable = !item.aclReadOnly && item.aclIsVisible;

		return itemToInsert;
	};

	var insertContainerElementsListWidget = function(container){
		return getFactoredWidget(container, ZhuAclConfig.containerElementsListWidgetName, true).then(function(factoredWidget){
			container.actions = container.actions || [];
			return factoredWidget;
		});
	};

	this.openAclModalWidget = function(){
		insertContainerElementsListWidget(templateManager.container).then(function(widget){
			ScreenService.openPopup(widget);
		});
	};

	var redirectToMainWindow = function(){
		var dashboardMenu = templateManager.project.getMenu('dashboard');
		var openLinkByMenu = ApplicationContext.LibUtilities && ApplicationContext.LibUtilities.openLinkByMenu;
		if(dashboardMenu && openLinkByMenu){
			openLinkByMenu(dashboardMenu);
		} else {
			openWindow(templateManager.project.mainWindow);
		}
	};

	var verifyContainerRule = function(rule){
		var containerIsRestrict = rule === ZhuAclConfig.RULE_INVISIBLE;
		if(containerIsRestrict){
			var actions = [{
				label: ScreenService.i18n("Ok"),
				code: redirectToMainWindow
			}];
			var dialogConfig = {
				type: 'ALERT',
				icon: 'exclamation',
				actions: actions
			};
			MessageService.showCustomDialogByCode(ZhuAclMessages.ACCESS_DENIED, {}, dialogConfig);
		}
		return !containerIsRestrict;
	};

	var mergeUndefinedRules = function(groupIds, itemRule, parentRules){
		parentRules = parentRules || {};

		var definedRule = itemRule.rule;
		var groupRules = parentRules.groupRules || {};

		if(!definedRule){
			definedRule = parentRules.rule || templateManager.project.aclDefaultRule;
		} else {
			var undefinedGroups = itemRule.undefinedGroups || [];
			groupIds.forEach(function(groupId){
				var parentRule = groupRules[groupId] || templateManager.project.aclDefaultRule;
				var isUndefinedGroup = undefinedGroups.indexOf(groupId) !== -1;
				var groupRule = isUndefinedGroup ? parentRule : definedRule;
				groupRules[groupId] = groupRule;

				if(definedRule !== ZhuAclConfig.RULE_ALLOW && groupRule !== ZhuAclConfig.RULE_INVISIBLE){
					definedRule = groupRule;
				}
			});
			itemRule.rule = definedRule;
		}

		return {rule: definedRule, groupRules: groupRules};
	};

	var getUserRule = function(rules, userId, elementId){
		return Util.getNestedProperty(rules, [elementId, 'userGroups', userId]);
	};

	var initObject = function(obj, objKeys, lastKeyValue){
		var objKeysLength = objKeys.length;
		objKeys.reduce(function(prev, next, index){
			if(index == objKeysLength - 1){
				prev[next] = lastKeyValue;
			} else {
				prev[next] = prev[next] || {};
			}
			return prev[next];
		}, obj);
	};

	var updateContainerAcl = function(rules, groupIds, userId){
		var applyRule = function(item, parentRules){
			var id = item.id;
			var userRule = getUserRule(rules, userId, id) || {};
			var mergedItem = mergeUndefinedRules(groupIds, userRule, parentRules);
			var itemRule = mergedItem.rule;

			item.aclIsVisible = itemRule != ZhuAclConfig.RULE_INVISIBLE;
			item.aclReadOnly = itemRule != ZhuAclConfig.RULE_ALLOW;

			return mergedItem;
		};

		var updateItemAcl = function(item, type, parentRules){
			var mergedItem = applyRule(item, parentRules);

			ZhuAclConfig.getAclMap(type).childs.forEach(function(type){
				if(Util.isArray(item[type])){
					item[type].forEach(function(child){
						updateItemAcl(child, type, mergedItem);
					});
				}
			});
		};

		var changeNotificationVisibility = function(isVisible){
			templateManager.project.notifications.some(function(notification){
				var isAclNotification = notification.name === "accessControl";
				if(isAclNotification){
					notification.isVisible = isVisible;
				}
				return isAclNotification;
			});
		};

		var updateUserRules = function(container){
			var containerId = container.id;
			var parentMenuId = container.parentMenuId;
			var userRule = getUserRule(rules, userId, containerId);
			if(!userRule && parentMenuId) {
				userRule = getUserRule(rules, userId, parentMenuId);
				if(userRule){
					initObject(rules, [containerId, 'userGroups', userId], userRule);
				}
			}
		};

		var container = templateManager.container;
		var aclEnable = container.aclEnable;
		aclEnable = aclEnable !== undefined ? aclEnable : true;
		changeNotificationVisibility(aclEnable);

		if(aclEnable){
			updateUserRules(container);
			var containerRule = applyRule(container);
			if(verifyContainerRule(containerRule.rule)){
				updateItemAcl(container, ZhuAclConfig.containerType);
			}
			loadedContainers[container.id] = true;
		}
	};

	var convertObjectRulesToArray = function(userId, objRules){
		var arrayRules = [];
		Object.keys(objRules).forEach(function(ruleId){
			var objRule = objRules[ruleId];
			var userRule = objRule.userGroups[userId];
			if(userRule){
				var rule = {
					"UNIQUE_ID": ruleId,
					"NAME": objRule.itemName,
					"RULE": userRule.rule
				};
				arrayRules.push(rule);
			}
		});
		return arrayRules;
	};

	var convertArrayRulesToObject = function(userId, arrayRules){
		var objRules = {};
		arrayRules.forEach(function(rule){
			var profileObjects = {};
			profileObjects[userId] = {
				'rule': rule.RULE,
				'undefinedGroups': rule.GROUP_ID || []
			};
			objRules[rule.UNIQUE_ID] = {
				'itemName': rule.NAME,
				'userGroups': profileObjects
			};
		});
		return objRules;
	};

	var subscribeContainerEvent = function(rules, groupIds, userId){
		eventAggregator.subscribe(EventAggregator.ON_CONTAINER_OPEN, function applyACL(){
			var containerId = templateManager.container.id;
			if(!loadedContainers[containerId]){
				updateContainerAcl(rules, groupIds, userId);
			}
		});

	};

	var updateProjectAcl = function(loginRules, defaultRule, groupIds){
		var rules = {};
		var userId = user.getUserId();
		if(Util.isObject(loginRules)){
			rules = loginRules;
			loginRules = convertObjectRulesToArray(userId, rules);
		} else if(Util.isArray(loginRules)){
			rules = convertArrayRulesToObject(userId, loginRules);
		}

		templateManager.project.aclDefaultRule = defaultRule || ZhuAclConfig.RULE_INVISIBLE;
		templateManager.userId = userId;

		templateManager.aclRules = rules;
		updateContainerAcl(rules, groupIds, userId);
		subscribeContainerEvent(rules, groupIds, userId);
	};

	var createUser = function createUser(userId, projectId, organizationId, defaultRule){
		user = new User(userId, projectId, organizationId, defaultRule);
	};

	this.setUserRules = function(groupIds, projectId, userId, organizationId, options){
		var returnPromise;
		loadedContainers = {};
		options = options || {};

		if(options.canConfigureAcl){
			ZhuAccessButtonService.enableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		} else {
			ZhuAccessButtonService.disableButtonInProject(templateManager.project);
			templateManager.updateTemplate();
		}

		if(options.rules){
			createUser(userId, projectId, organizationId, options.defaultRule);
			updateProjectAcl(options.rules, options.defaultRule, groupIds);
			returnPromise = ZHPromise.when(templateManager.aclRules);
		} else {
			returnPromise = ZhuAclService.getZhuAclRules(projectId, groupIds).then(function(response){
				createUser(userId, projectId, organizationId, response.defaultRule);
				updateProjectAcl(response.aclRules, response.defaultRule, groupIds);
				return templateManager.aclRules;
			}).catch(function(err){
				MessageService.showMessageByCode(err.code);
			});
		}
		return returnPromise;
	};

	this.setElementsListFilter = function(container, groupId, widgetName){
		widgetName = widgetName || ZhuAclConfig.menuElementsListWidgetName;
		var elementsWidget = container.getWidget(widgetName);
		elementsWidget.dataSourceFilter = ZhuAclConfig.getDefaultDataSourceFilter(groupId, user.getProjectId(), user.getOrganizationId());
		user.setCurrentGroupId(groupId);
	};

	var getFactoredWidget = function(container, widgetName, pushInContainer){
		var widget = container.getWidget(widgetName);
		if(widget){
			return ZHPromise.when(widget);
		} else {
			return ZhuAclService.getWidgetMetadata(widgetName).then(function(metadata){
				var factoredWidget = metaDataFactory.widgetFactory(metadata, container);
				if(pushInContainer){
					container.widgets.push(factoredWidget);
				}
				return factoredWidget;
			});
		}
	};
	this.getFactoredWidget = getFactoredWidget;

	this.insertElementsListWidget = function(container, parentWidget, extraElementsPath){
		getFactoredWidget(container, ZhuAclConfig.menuElementsListWidgetName).then(function(factoredWidget) {
			if(parentWidget && !factoredWidget.parent){
				factoredWidget.parent = parentWidget;
				parentWidget.widgets = parentWidget.widgets || [];
				parentWidget.widgets.push(factoredWidget);
				factoredWidget.extraElementsPath = extraElementsPath;
			}
		});
	};

}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclController', ZhuAclController);
});


// FILE: assets/js/controllers/ZhuAclCRUDController.js
/* global Configuration, ZhuAclMessages */
function ZhuAclCRUDController(ScreenService, ZHPromise, ZhuAclConfig, ZhuAclController, MessageService){

	function defineQuestionToUse(rule, messageConfig){
		var question = ZhuAclMessages.VISIBLE;
		if(rule == ZhuAclConfig.RULE_INVISIBLE){
			question = ZhuAclMessages.INVISIBLE;
		} else if(rule == ZhuAclConfig.RULE_ALLOW){
			question = ZhuAclMessages.EDITABLE;
		}
		messageConfig.newVisibility = ScreenService.i18n(question);
		return messageConfig;
	}

	function deleteChildrensRules(elementsWidget, parentRow, groupId, parentRule){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					if(row.extraElement){
						row.RULE = parentRule;
					} else {
						row.__isDeleted = true;
						row.RULE_SAVE_CONTAINER = parentRule;
					}
					row.changed = true;
					row.GROUP_ID = groupId;
					deleteChildrensRules(elementsWidget, row, groupId, parentRule);
				}
			});
		}
	}

	function confirmChildsVisibility(elementsWidget, ruleType, messageConfig){
		var returnPromise = ZHPromise.when();
		if(elementsWidget.currentRow._childCount){
			var currentRow = elementsWidget.currentRow;
			var defer = ZHPromise.defer();
			var actions = [
				{
					label: ScreenService.i18n(ZhuAclMessages.YES),
					code: function(){
						currentRow.changed = true;
						deleteChildrensRules(elementsWidget, currentRow, currentRow.GROUP_ID, currentRow.RULE);
						defer.resolve();
					}
				},
				{
					label: ScreenService.i18n(ZhuAclMessages.NO),
					code: function(){
						var messageConfig = {
							type: 'confirmation',
							icon: 'question'
						};
						MessageService.showMessageByCode(ZhuAclMessages.CONFIRMATION_MESSAGE, {}, messageConfig);
						defer.reject();
					}
				}
			];
			var message = ZhuAclMessages.CONFIRM_CHILDS_QUESTION;
			var dialogConfig = {
				type: 'confirmation',
				icon: 'question',
				actions: actions
			};

			MessageService.showCustomDialogByCode(message, defineQuestionToUse(ruleType, messageConfig), dialogConfig);
			returnPromise = defer.promise;
		}

		return returnPromise;
	}

	function reloadChildrensRules(elementsWidget, parentRow, updatedRules, deletedRules){
		if(parentRow._childCount){
			elementsWidget.dataSource.data.forEach(function(row){
				if(row._parent == parentRow){
					var rowId = row.UNIQUE_ID;
					row.isVisible = parentRow.isVisible;
					row.isEditable = parentRow.isEditable;

					if(deletedRules.indexOf(rowId) != -1){
						delete row.RULE_ID;
					}
					if(updatedRules[rowId] && updatedRules[rowId].containerId){
						row.CONTAINER_RULE_ID = updatedRules[rowId].containerId;
					}

					reloadChildrensRules(elementsWidget, row, updatedRules, deletedRules);
				}
				delete row.__isDeleted;
				delete row.changed;
			});
		}
	}

	function fixZeedhiBehavior(widget){
		widget.dataSource.data.forEach(function(data){
			data.isEditable = (data._childCount && data.isVisible) || data.isEditable;
		});
	}

	function isFatherVisible(parentRow, widget){
		return widget.dataSource.data.some(function(row){
			return (row.PARENT_ID == parentRow.UNIQUE_ID) && (row.isVisible || row.isEditable);
		});
	}

	function reloadParentsRules(row, widget){
		var parentRow = row._parent;
		if(parentRow){
			if(row.isVisible){
				parentRow.isEditable = parentRow.isEditable || row.isEditable || row.isVisible;
				parentRow.isVisible = true;
			} else {
				var updateFather = isFatherVisible(parentRow, widget);
				parentRow.isEditable = updateFather;
				parentRow.isVisible = updateFather;
			}
			reloadParentsRules(row._parent, widget);
		}
	}

	function clearRowState(row){
		row.changed = false;
		row.__isDeleted = false;
	}

	function reloadDataSourceAfterSave(elementsWidget, newRules){
		var currentRow = elementsWidget.currentRow;
		var currentRowId = currentRow.UNIQUE_ID;
		var updatedRule = newRules.updated[currentRowId];
		var deletedRules = newRules.deleted || [];
		if(updatedRule){
			if(Util.isObject(updatedRule)){
				currentRow.RULE_ID = updatedRule.menuId;
				currentRow.CONTAINER_RULE_ID = updatedRule.containerId;
			} else {
				currentRow.RULE_ID = updatedRule;
			}
			reloadChildrensRules(elementsWidget, currentRow, newRules.updated, deletedRules);
			if(!currentRow.extraElement){
				reloadParentsRules(currentRow, elementsWidget);
				fixZeedhiBehavior(elementsWidget);
			}
		}

		var updatedRules = Object.keys(newRules.updated);
		if(updatedRules.length){
			//updated all extraElements
			elementsWidget.dataSource.data.forEach(function(row){
				var ruleId = newRules.updated[row.UNIQUE_ID];
				if(ruleId && row.extraElement){
					row.RULE_ID = ruleId;
					row.RULE = currentRow.RULE;
					row.isVisible = currentRow.isVisible;
					row.isEditable = currentRow.isEditable;
				}
				clearRowState(row);
			});
		} else {
			elementsWidget.dataSource.data.forEach(clearRowState);
		}
	}

	function cancelSave(widget){
		widget.reload();
	}

	function changeElementRule(currentRow, isVisible, isEditable){
		currentRow.RULE = ZhuAclConfig.getRuleType(isVisible, isEditable);
		currentRow.isVisible = isVisible;
		currentRow.isEditable = isEditable;
		currentRow.GROUP_ID = currentRow.GROUP_ID || ZhuAclController.getCurrentGroupId();
	}

	function clearInlineChangedStatus(row){
		delete row.changed_isEditable;
		delete row.changed_isVisible;
	}

	function saveRule(widget, isEditable, isVisible, messageConfig){
		var currentRow = widget.currentRow;
		var ruleType = ZhuAclConfig.getRuleType(isVisible, isEditable);

		clearInlineChangedStatus(currentRow);
		changeElementRule(currentRow, isVisible, isEditable);

		confirmChildsVisibility(widget, ruleType, messageConfig).then(function(){
			widget.saveRow().then(function(response){
				reloadDataSourceAfterSave(widget, response.dataset.aclZhuAccessControlRule);
				MessageService.successNotificationByCode(ZhuAclMessages.SUC_RULE_INSERTED);
			}).catch(function(){
				cancelSave(widget);
				MessageService.notificationMessageByCode(ZhuAclMessages.ERR_RULE_INSERTED, {}, {type: "alert"});
			});
		}).catch(function(){
			cancelSave(widget);
			MessageService.notificationMessageByCode(ZhuAclMessages.RULE_NOT_SAVED, {}, {type: "alert"});
		});
	}

	this.changeIsEditable = function changeIsEditable(widget, row, messageConfig){
		var isEditable = row.isEditable;
		var isVisible = isEditable || row.isVisible;
		saveRule(widget, isEditable, isVisible, messageConfig);
	};

	this.changeIsVisible = function changeIsVisible(widget, row, messageConfig){
		var isVisible = row.isVisible;
		var isEditable = isVisible && row.isEditable;
		saveRule(widget, isEditable, isVisible, messageConfig);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclCRUDController', ZhuAclCRUDController);
});


// FILE: assets/js/controllers/ZhuAclExportRulesController.js
function ZhuAclExportRulesController(templateManager, MessageService, ScreenService, ZhuAclConfig, ZhuAclController, ZhuAclService){
	var self = this;
	var requestToExportRules = function(profileFrom, profilesToExport) {
		ZhuAclService.exportRules(profileFrom, profilesToExport)
			.then(self.closeExportRulesPopup)
			.catch(function(response){
				var errorCode = (response && (response.errorCode || response.error)) || ZhuAclMessages.ERR_RULES_EXPORTED;
				MessageService.errorNotificationByCode(errorCode);
			});
	};

	this.exportRules = function() {
		var exportRulesWidget = templateManager.container.getWidget(ZhuAclConfig.exportRulesModal);
		if(exportRulesWidget.isValid()){
			var currentRow = exportRulesWidget.currentRow;
			var dialogConfig = {
				type: 'CONFIRMATION',
				actions: [
					{
						label: ScreenService.i18n(ZhuAclMessages.YES),
						code: function() {
							requestToExportRules(currentRow.profileFrom, currentRow.profilesToExport);
						}
					},
					{ label: ScreenService.i18n(ZhuAclMessages.NO) }
				]
			};
			MessageService.showCustomDialogByCode(ZhuAclMessages.CONFIRM_EXPORT, {}, dialogConfig);
		}
	};

	this.profileActionEventExportRules = function(groupId) {
		var container = templateManager.container;
		return ZhuAclController.getFactoredWidget(container, ZhuAclConfig.exportRulesModal, true).then(function(exportRulesWidget){
			ScreenService.openPopup(exportRulesWidget);
			exportRulesWidget.isVisible = true;
			exportRulesWidget.currentRow = {
				profileFrom: groupId
			};
			exportRulesWidget.fields[0].dataSourceFilter = [{
				name: 'ID',
				operator: '<>',
				value: groupId
			}];
		});
	};

	this.closeExportRulesPopup = function() {
		ScreenService.closePopup();
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuAclExportRulesController', ZhuAclExportRulesController);
});

// FILE: assets/js/controllers/ZhuContainerElementsController.js
/* global Util, Configuration, ZhuAclMessages */
function ZhuContainerElementsController(templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService){
	var self = this;
	var idErrorMessages = "";

	var getContainerElementsTreeWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsTreeWidgetName);
	};

	var getContainerElementsListWidget = function(){
		return templateManager.container.getWidget(ZhuAclConfig.containerElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = "";
	};

	var showIdErrorsMessage = function(){
		if(idErrorMessages){
			var messageConfig = {
				type: 'alert',
				icon: 'exclamation'
			};
			MessageService.showMessageByCode(ZhuAclMessages.NSHOW_WITHOUT_ID, {elementsIds: idErrorMessages}, messageConfig);
		}
	};

	var addParentMenuProperties = function(rules, item, itemToInsert){
		var parentMenuId = item.parentMenuId;
		var parentMenuLabel = item.parentMenuLabel || item.parentMenuName;
		if(parentMenuId && parentMenuLabel){
			var parentMenuRule = ZhuAclController.getElementRule(rules, parentMenuId);
			itemToInsert.PARENT_MENU_ID = parentMenuId;
			itemToInsert.PARENT_MENU_NAME = item.parentMenuName || ZhuAclController.prepareLabelToName(parentMenuLabel);
			itemToInsert.PARENT_MENU_LABEL = parentMenuLabel;
			itemToInsert.PARENT_MENU_RULE_ID = parentMenuRule.ID;
		}
	};

	var getPlannedContainer = function(rules, item, itemType, parentId){
		var plannedItems = [];
		itemType = itemType || ZhuAclConfig.containerType;
		if(item && !item.isZhuAclElement){
			var uniqueId = item.id;
			if(uniqueId){
				var aclType = ZhuAclConfig.getAclMap(itemType);
				var itemToInsert = ZhuAclController.getPlannedItemToInsertInDataSet(rules, item, aclType.aclTypeCod, parentId);
				addParentMenuProperties(rules, item, itemToInsert);
				plannedItems.push(itemToInsert);
				aclType.childs.forEach(function(type){
					if(Util.isArray(item[type])){
						item[type].forEach(function(child){
							plannedItems = plannedItems.concat(getPlannedContainer(rules, child, type, uniqueId));
						});
					}
				});
			} else {
				var nameToShow = item.label || item.name;
				idErrorMessages += "<br> " + nameToShow + ";";
			}
		}
		return plannedItems;
	};

	this.reloadContainerElementsModal = function(){
		getContainerElementsListWidget().reload();
	};

	this.reloadContainerElementsTree = function(){
		getContainerElementsTreeWidget().reload();
	};

	this.selectAccessOnChange = function(selectedRow){
		if(selectedRow && selectedRow.SELECT_ACCESS){
			ZhuAclController.setElementsListFilter(templateManager.container, selectedRow.ID, ZhuAclConfig.containerElementsTreeWidgetName);
		}
		self.reloadContainerElementsTree();
	};

	var copyItem = function(itemToCopy, itemType, parentId){
		var item;
		if(itemToCopy && !itemToCopy.isZhuAclElement && !itemToCopy.ignoreAclElementsConfig){
			var uniqueId = itemToCopy.id;
			item = {
				id: uniqueId,
				name: itemToCopy.name,
				label: itemToCopy.label,
				parentMenuId: itemToCopy.parentMenuId,
				parentMenuName: itemToCopy.parentMenuName,
				parentMenuLabel: itemToCopy.parentMenuLabel,
			};
			itemType = itemType || ZhuAclConfig.containerType;
			var aclType = ZhuAclConfig.getAclMap(itemType);
			aclType.childs.forEach(function(type){
				if(Util.isArray(itemToCopy[type])){
					item[type] = itemToCopy[type].map(function(child){
						return copyItem(child, type, uniqueId);
					}).filter(function(copiedChild){
						return copiedChild;
					});
				}
			});
		}
		return item;
	};

	this.containerElementsTreeAfterLoadDataSource = function(){
		clearIdErrorsMessage();
		var widget = getContainerElementsTreeWidget();
		var elementsListWidget = getContainerElementsListWidget();
		var elementsListRow = elementsListWidget && elementsListWidget.currentRow;
		var rulesData = Util.clone(widget.dataSource.data);
		widget.dataSource.data = [];

		if(elementsListRow && elementsListRow.SELECT_ACCESS){
			var rules = ZhuAclController.getRulesFromDatasource(rulesData);
			var container = copyItem(templateManager.container);
			if(container){
				aclEngine.processAcl(container, rules, templateManager.userId, templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule());
				widget.dataSource.data = getPlannedContainer(rules, container);
				showIdErrorsMessage();
			}
		}
	};

	this.isEditableOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		var messageConfig = {
			type: 'container',
			subType: 'elementos filhos'
		};
		ZhuAclCRUDController.changeIsEditable(widget, currentRow, messageConfig);
	};

	this.isVisibleOnChange = function(currentRow){
		var widget = getContainerElementsTreeWidget();
		var messageConfig = {
			type: 'container',
			subType: 'elementos filhos'
		};
		ZhuAclCRUDController.changeIsVisible(widget, currentRow, messageConfig);
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuContainerElementsController', ZhuContainerElementsController);
});

// FILE: assets/js/controllers/ZhuMenuElementsController.js
function ZhuMenuElementsController(ScreenService, templateManager, ZhuAclController, ZhuAclConfig, aclEngine, ZhuAclCRUDController, MessageService, restEngine, ZHPromise){
	var MENU = "M";
	var GROUP_MENU = "G";
	var IGNORED_MENUS = ['zhHistory'];
	var idErrorMessages = {"errorMenus": "", "errorContainers": ""};

	var getWidgetElements = function(){
		return templateManager.container.getWidget(ZhuAclConfig.menuElementsListWidgetName);
	};

	var clearIdErrorsMessage = function(){
		idErrorMessages = {"errorMenus": "", "errorContainers": ""};
	};

	var showIdErrorsMessage = function(){
		var messageCode;
		var errorMenus = idErrorMessages.errorMenus;
		var errorContainers = idErrorMessages.errorContainers;
		var messageParameters = {
			elementsIds: errorMenus,
			elementsWithoutContainer: errorContainers
		};

		if(errorMenus && errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID_AND_CONTAINER_CONFIG;
		} else if(errorMenus){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_ID;
		} else if(errorContainers){
			messageCode = ZhuAclMessages.NSHOW_WITHOUT_CONTAINER_CONFIG;
		}

		if(messageCode){
			MessageService.showMessageByCode(messageCode, messageParameters, { type: 'alert' });
		}
	};

	var addErrorMenuId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorMenus += "<br> " + nameToShow + ";";
	};

	var addErrorContainerId = function(item){
		var nameToShow = item.label || item.name;
		idErrorMessages.errorContainers += "<br> " + nameToShow + ";";
	};

	this.elementsActionEventEdit = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			var messageConfig = {
				type: 'menu',
				subType: 'submenus'
			};
			currentRow.isEditable = !currentRow.isEditable;
			ZhuAclCRUDController.changeIsEditable(elementsWidget, currentRow, messageConfig);
		}
	};

	this.elementsActionEventView = function(currentRow){
		if(!currentRow.buildedMenu){
			var elementsWidget = getWidgetElements();
			var messageConfig = {
				type: 'menu',
				subType: 'submenus'
			};
			currentRow.isVisible = !currentRow.isVisible;
			currentRow.isEditable = currentRow.isVisible;
			ZhuAclCRUDController.changeIsVisible(elementsWidget, currentRow, messageConfig);
		}
	};

	this.elementsActionEventBack = function(){
		var elementsWidgetParent = getWidgetElements().parent;
		return ScreenService.closeSwipe().then(function(){
			elementsWidgetParent.list();
			elementsWidgetParent.activate();
		});
	};

	var addContainerProperties = function(item, menuItem, rules){
		var containerId = menuItem.containerId;
		var containerLabel = menuItem.containerLabel || menuItem.containerName;
		if(containerId && containerLabel){
			var containerRule = ZhuAclController.getElementRule(rules, containerId);
			item.CONTAINER_ID = containerId;
			item.CONTAINER_NAME = menuItem.containerName || ZhuAclController.prepareLabelToName(containerLabel);
			item.CONTAINER_LABEL = containerLabel;
			item.CONTAINER_RULE_ID = containerRule.RULE_ID;
		} else if(menuItem.windowName && !menuItem.buildedMenu){
			addErrorContainerId(menuItem);
		}
	};

	var planningMenuItem = function(menuItem, parentMenuItem, rules){
		var item;
		var uniqueId = menuItem.id;
		if(uniqueId){
			var parentId = parentMenuItem && parentMenuItem.id;
			var itemType = !parentId ? GROUP_MENU : MENU;
			item = ZhuAclController.getPlannedItemToInsertInDataSet(rules, menuItem, itemType, parentId);
			item.buildedMenu = menuItem.buildedMenu;
			addContainerProperties(item, menuItem, rules);
		} else {
			addErrorMenuId(menuItem);
		}

		return item;
	};

	var addMenusInDataSource = function(widget, rules, menus, parentMenu){
		menus.forEach(function(menu, index){
			var item = planningMenuItem(menu, parentMenu, rules);
			if(item){
				widget.dataSource.data.push(item);
				if (menu.menus && menu.menus.length > 0){
					addMenusInDataSource(widget, rules, menu.menus, menu);
				}
			}
		});
	};

	var getProjectMenu = function(){
		var projectMenu = Util.clone(templateManager.project.groupMenu);
		return projectMenu.filter(function(menu){
			return IGNORED_MENUS.indexOf(menu.name) == -1;
		});
	};

	var extraRulesPromise;

	var loadExtraRules = function(extraElementsPath){
		if(!extraRulesPromise){
			if(extraElementsPath){
				extraRulesPromise = restEngine.requestMetaData(Util.getRightPath(extraElementsPath), {});
			} else {
				extraRulesPromise = ZHPromise.when();
			}
		}
	};

	var addExtraElements = function(widget, projectRules, defaultRule){
		var getRule = function(id){
			return (projectRules[id] && projectRules[id].ruleRow) || {};
		};

		var getVisibility = function(rule){
			return (rule || defaultRule) !== 'I';
		};

		extraRulesPromise.then(function(response){
			if(response && response.data){
				response.data.forEach(function(row){
					var rule = getRule(row.UNIQUE_ID);
					row.RULE = rule.RULE;
					row.RULE_ID = rule.ID;
					row.isVisible = getVisibility(rule.RULE);
					row.extraElement = true;
				});
				widget.dataSource.data = widget.dataSource.data.concat(response.data);
			}
		});
	};

	this.loadPlanningTree = function(){
		var elementsWidget = getWidgetElements();
		var userId = templateManager.userId;
		var defautRule = templateManager.project.elementListDefaultRule || ZhuAclController.getDefaultRule();
		var rules = ZhuAclController.getRulesFromDatasource(elementsWidget.dataSource.data);
		var newMenu = getProjectMenu();

		clearIdErrorsMessage();
		loadExtraRules(elementsWidget.extraElementsPath);
		aclEngine.processMenuAcl(newMenu, rules, templateManager.containers, userId, defautRule);
		elementsWidget.dataSource.data = [];
		addMenusInDataSource(elementsWidget, rules, newMenu);
		addExtraElements(elementsWidget, rules, defautRule);
		showIdErrorsMessage();
	};

	this.reloadElementsWidget = function(){
		getWidgetElements().reload();
	};
}

Configuration(function(ContextRegister){
	ContextRegister.register('ZhuMenuElementsController', ZhuMenuElementsController);
});

// FILE: assets/js/adapter/config.js
ZeedhiApp.run(function(templateManager, ZhuAccessButtonService) {
	templateManager.projectPromise.then(function(project){
		if(templateManager.project.aclEnable){
			ZhuAccessButtonService.insertButtonInProject(templateManager.project);
		}
	});
});


// FILE: ./bower_components/zhulog/dist/main.js

// FILE: src/services/ZhuLogService.js
function ZhuLogService() {

    this.getPlainContentRows = function(value) {
        var data = [];
        var objValue = JSON.parse(value);

        createRows(data, objValue);
        return data;
    };

    function createRows(data, value, parent) {
        Object.keys(value).forEach(function(key) {
            createRow(data, key, value[key], parent);
        });
    }

    function createRow(data, key, value, parent) {
        var method;
        if (Util.isObject(value)) {
            method = factoryCreateObjectRowsMethod();
        } else if (Array.isArray(value)) {
            method = factoryCreateArrayRowsMethod();
        } else {
            method = createSimpleRow;
        }

        method(data, key, value, parent);
    }

    function factoryCreateObjectRowsMethod() {
        return factoryCreateObjectRows(': {}');
    }

    function factoryCreateArrayRowsMethod() {
        return factoryCreateObjectRows(': []');
    }

    function createSimpleRow(data, key, value, parent) {
        if (typeof value === 'string') {
            value = '"' + value + '"';
        }

        value = key + ': ' + value;
        addData(key, value, parent, data);
    }

    function addData(key, value, parent, data) {
        var id = parent + '#' + key;
        data.push({
            'ID': id,
            'VALUE': value,
            'PARENT': parent
        });

        return id;
    }

    function factoryCreateObjectRows(emptySufix) {
        return function createObjectRows(data, key, rowContent, parent) {
            var rowValue = key;
            if (Object.keys(rowContent).length === 0) {
                rowValue += emptySufix;
            }

            var id = addData(key, rowValue, parent, data);
            createRows(data, rowContent, id);
        };
    }

}

Configuration(function (ContextRegister) {
    ContextRegister.register('ZhuLogService', ZhuLogService);
});

// FILE: src/controllers/ZhuLogController.js
function ZhuLogController(ZhuLogService, ScreenService, templateManager, ZhGridFilter, metaDataFactory) {

    var LOG_WIDGET_NAME = 'zhuLog';
    var CONTENT_WIDGET = 'content';
    var CONTENT_FORM_WIDGET = 'contentForm';
    var CONTENT_GRID_WIDGET = 'contentGrid';
    var CONTENT_FILTER_WIDGET = 'select';

    function getContentWidget() {
        return templateManager.container.getWidget(CONTENT_WIDGET);
    }

    function getContentFormWidget() {
        return templateManager.container.getWidget(CONTENT_FORM_WIDGET);
    }

    function getContentGridWidget() {
        return templateManager.container.getWidget(CONTENT_GRID_WIDGET);
    }

    function getFilterWidget() {
        return templateManager.container.getWidget(CONTENT_FILTER_WIDGET);
    }

    function getZhuLogWidget() {
        return templateManager.container.getWidget(LOG_WIDGET_NAME);
    }

    this.openContent = function(widget) {
        var contentFormWidget = getContentFormWidget();
        contentFormWidget.currentRow = widget.currentRow;

        var rows = ZhuLogService.getPlainContentRows(widget.currentRow.CONTENT);
        getContentGridWidget().dataSource.data = rows;

        ScreenService.openSwipe(getContentWidget());
    };

    var startFilter = function(widget, currentRow) {
        ZhGridFilter.startFiltering(widget);
        var filterPopup = widget.__filterPopup;
        filterPopup.currentRow = currentRow;
    };

    var isAppliedFilter = function(widget){
        return widget.filterApplied && widget.filterApplied.length > 0;
    };

    this.zhuLogWidgetOnEnter = function() {
        var widget = getZhuLogWidget();
        if(!widget.hasFilterApplied){
            if (isAppliedFilter(widget)) {
                widget.reload();
            } else {
                if(widget.__filterPopup ) {
                    startFilter(widget, widget.__filterPopup.currentRow);
                } else if(widget.currentRowDefaultFilter){
                    startFilter(widget, widget.currentRowDefaultFilter);
                } else {
                    startFilter(widget,{});
                }
            }
        }
    };

    this.openFilterPopUp = function() {
        ScreenService.openPopup(getFilterWidget());
    };

    this.setSelectValues = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var select2 = templateManager.container.getWidget('select').fields[1];

        if (selectWidget.currentRow.SNATIVE === 'REQUEST') {
            select2.label = 'Resquest type';
            select2.readOnly = false;

            select2.dataSource.data = [];
            select2.dataSource.data.push(
                {'value': 'Dataset'},
                {'value': 'Row'},
                {'value': 'Empty'},
                {'value': 'Filter'}
            );
        } else {
            if (selectWidget.currentRow.SNATIVE === 'RESPONSE') {
                select2.label = 'Response type';
                select2.readOnly = false;

                select2.dataSource.data = [];
                select2.dataSource.data.push(
                    {
                        'value': 'Dataset'
                    },
                    {
                        'value': 'Message'
                    }
                );
            } else {
                select2.dataSource.data = [];
                this.clearFieldsValue(1);
                select2.readOnly = true;
            }
        }
    };

    this.setTextEdit = function() {
        var selectWidget = templateManager.container.getWidget('select');

        selectWidget.actions[2].isVisible = true;
        selectWidget.actions[3].isVisible = true;
        // selectWidget.fields[2].isVisible = true;
        // selectWidget.fields[3].isVisible = true;

        if (selectWidget.currentRow.SNATIVE === 'REQUEST') {
            if (selectWidget.currentRow.SNATIVE2 === 'Dataset' || selectWidget.currentRow.SNATIVE2 === 'Row') {
                selectWidget.fields[2].label = 'Column name';
                selectWidget.fields[3].label = 'Operator';
                selectWidget.fields[3].template = 'field/select-native.html';
                selectWidget.fields[4].label = 'Value';
                this.addFields();
            }

            if (selectWidget.currentRow.SNATIVE2 === 'Empty') {
                selectWidget.fields[2].label = 'Parameter';
                selectWidget.fields[3].label = 'Operator';
                selectWidget.fields[3].template = 'field/select-native.html';
                selectWidget.fields[4].label = 'Value';

                this.addFields();
            }

            if (selectWidget.currentRow.SNATIVE2 === 'Filter') {
                selectWidget.fields[2].label = 'Column name';
                selectWidget.fields[3].label = 'Column value';
                selectWidget.fields[3].template = 'field/text-edit.html';
                selectWidget.fields[4].label = 'Value';

                this.addFields();
            }

            if (selectWidget.currentRow.SNATIVE2 === null) {
                this.removeExtraFields();
            }
        } else {
            if (selectWidget.currentRow.SNATIVE === 'RESPONSE') {
                if (selectWidget.currentRow.SNATIVE2 === 'Dataset') {
                    selectWidget.fields[2].label = 'Column name';
                    selectWidget.fields[3].label = 'Operator';
                    selectWidget.fields[3].template = 'field/select-native.html';
                    selectWidget.fields[4].label = 'Value';
                    this.addFields();
                }

                if (selectWidget.currentRow.SNATIVE2 == 'Message'){
                    selectWidget.fields[2].label = 'Column name';
                    selectWidget.fields[3].label = 'Operator';
                    selectWidget.fields[3].template = 'field/select-native.html';
                    selectWidget.fields[4].label = 'Value';
                    this.addFields();
                }

                if (selectWidget.currentRow.SNATIVE2 === null) {
                    this.removeExtraFields();
                }
            }
        }
    };

    this.removeExtraFields = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var fields = selectWidget.fields;

        for (var i = (fields.length - 1); i > 6; i--) {
            if (!Util.isUndefined(fields[i])) {
                delete selectWidget.currentRow[fields[i].name];
                delete fields[i];
            }
        }
    };

    this.addFields = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var fields = templateManager.container.getWidget('select').fields;
        var field;
        var dataSource;

        for (var i = 2; i <= 6; i++) {
            if (i !== 5) {
                var data;

                if (!Util.isUndefined(fields[i].dataSource)) {
                    data = fields[i].dataSource.data;
                    dataSource = {
                        'data': data
                    };
                }

                field = {
                    label: fields[i].label,
                    template: fields[i].template,
                    name: 'extraField' + fields.length,
                    class: fields[i].class,
                    events: fields[i].events,
                    dataSource: dataSource,
                    valueField: fields[i].valueField,
                    descriptionField: fields[i].descriptionField,
                    isVisible: true,
                    breakline: fields[i].breakline
                };

                metaDataFactory.fieldFactory(field, selectWidget, selectWidget);
            } else {
                field = {
                    label: fields[i].label,
                    template: fields[i].template,
                    name: 'extraField' + fields.length,
                    class: fields[i].class,
                    events: fields[i].events,
                    dataSource: dataSource,
                    valueField: fields[i].valueField,
                    descriptionField: fields[i].descriptionField,
                    isVisible: false,
                    breakline: fields[i].breakline
                };

                metaDataFactory.fieldFactory(field, selectWidget, selectWidget);
            }
        }
    };

    this.clearAllFields = function() {
        this.removeExtraFields();
        this.clearFieldsValue(0);
    };

    this.clearFieldsValue = function(startingValue) {
        var selectWidget = templateManager.container.getWidget('select');

        for (var i = startingValue; i < selectWidget.fields.length; i++) {
            if(!Util.isUndefined(selectWidget.fields[i])){
                if(selectWidget.fields[i].value !== ''){
                    selectWidget.fields[i].value('');
                }
            }
        }

        selectWidget.fields[1].readOnly = true;
    };

    this.clearFields = function(args) {
        var fieldIndex = args.owner.field.sequenceInList;
        fieldIndex--;

        var selectWidget = templateManager.container.getWidget('select');
        var fields = selectWidget.fields;

        for (var i = fieldIndex; i >= fieldIndex - 4; i--) {

            delete selectWidget.currentRow[fields[i].name];
            delete fields[i];
        }

        templateManager.updateTemplate();
    };

    this.manageFields = function(args) {
        var operatorIndex = args.owner.field.sequenceInList;
        operatorIndex--;

        var selectWidget = templateManager.container.getWidget('select');
        var selectedfield = selectWidget.fields[operatorIndex];
        var selectedValue = selectedfield.value();

        if (selectedValue === 'IS_NULL' || selectedValue === 'IS_NOT_NULL'){
            selectWidget.fields[operatorIndex + 1].readOnly = false;
            selectWidget.fields[operatorIndex + 1].class = 4;
            selectWidget.fields[operatorIndex + 2].isVisible = false;
            selectWidget.fields[operatorIndex + 1].readOnly = true;
        } else if (selectedValue === 'BETWEEN' || selectedValue === 'NOT_BETWEEN') {
            selectWidget.fields[operatorIndex + 1].class = 2;
            selectWidget.fields[operatorIndex + 1].label = 'Begin';
            selectWidget.fields[operatorIndex + 2].isVisible = true;
        } else {
            selectWidget.fields[operatorIndex + 1].readOnly = false;
            selectWidget.fields[operatorIndex + 1].class = 4;
            selectWidget.fields[operatorIndex + 1].label = 'Value';
            selectWidget.fields[operatorIndex + 2].isVisible = false;
        }
    };

    this.startFiltering = function() {
        var selectWidget = templateManager.container.getWidget('select');
        var fields = selectWidget.fields;

        var type = fields[0].value();
        var req_type = fields[1].value();
        var container = fields[2].value();
        var widget = fields[3].value();

        var params = [];
        var lastposition = fields.length - 1;

        var i = 9;
        var j = 0;

        while (i < fields.length) {
            if (!Util.isUndefined(fields[i])) {
                var name = fields[i].value();
                i++;
                var operator = fields[i].value();
                i++;

                var value;
                if(operator === 'IS_NULL' || operator === 'IS_NOT_NULL') {

                    value = '';
                    i += 3;

                } else if(operator === 'BETWEEN' || operator === 'NOT_BETWEEN') {
                    value = [];
                    value[0] = fields[i].value();
                    i++;
                    value[1] = fields[i].value();
                    i+= 2;
                } else {
                    value = fields[i].value();
                    i += 3;
                }

                params[j] = {
                    name: name,
                    operator: operator,
                    value: value
                };
                j++;
            } else {
                i+= 5;
            }
        }

        var resultObject = [
            {
                type: type,
                re_type: req_type,
                container: container,
                widget: widget,
                params: params
            }
        ];

        var result = JSON.stringify(resultObject[0]);
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('ZhuLogController', ZhuLogController);
});

// FILE: src/ZhuLogDecorator.js
function ZhuLogDecorator(ApplicationContext, templateManager) {

    var LOG_CONTAINER_NAME = 'zhulog#log';
    var GRID_WIDGET_INDEX = 0;

    var logDecorator;
    function getLogDecorator() {
        if (logDecorator === undefined) {
            logDecorator = templateManager.projectPromise.then(function() {
                var decoratorServiceName = templateManager.project.zhuLogDecoratorService;
                var logDecorator = ApplicationContext[decoratorServiceName];

                return logDecorator;
            });
        }
        return logDecorator;
    }

    this.afterLoadDataSource = function(args) {
        getLogDecorator().then(function(logDecorator) {
            if (logDecorator) {
                var dataSource = args.owner.dataSource;
                dataSource.data = logDecorator.normalizeRows(dataSource.data);
            }
        });
    };

    this.beforeGridFilterApply = function(args) {
        getLogDecorator().then(function(logDecorator) {
            if (logDecorator) {
                var _event = args.e;
                _event.filters = logDecorator.normalizeFilters(_event.filters);
            }
        });
    };

    this.apply = function(containerName, container) {
        return getLogDecorator().then(function(logDecorator) {
            if (logDecorator && containerName === LOG_CONTAINER_NAME) {
                var logWidget = container.widgets[GRID_WIDGET_INDEX];

                logWidget.events.push({
                    "name": "WidgetAfterLoadDataSource",
                    "code": "ZhuLogDecorator.afterLoadDataSource(args);"
                });

                logWidget.events.push({
                    "name": "GridFilterBeforeApply",
                    "code": "ZhuLogDecorator.beforeGridFilterApply(args);"
                });

                logDecorator.decorateWidget(logWidget);
            }

            return container;
        });
    };

}

Configuration(function (ContainerDecoratorProvider, ContextRegister, ApplicationContext, templateManager) {
    ContextRegister.register('ZhuLogDecorator', ZhuLogDecorator);
    ContainerDecoratorProvider.registerProvider(new ZhuLogDecorator(ApplicationContext, templateManager));
});

// FILE: src/ZhuLogHelper.js
function ZhuLogHelper() {

    this.moveField = function(widget, fieldName, position) {
        var index = -1;
        widget.fields.find(function(field) {
            index++;
            return field.name === fieldName;
        });
        widget.fields.splice(position - 1, 0, widget.fields.splice(index, 1)[0]);
    };

    this.getField = function(widget, fieldName) {
        var field = widget.fields.find(function(field) {
            return field.name === fieldName;
        });

        if (field === null) {
            throw 'Field "'+fieldName+'" not found';
        }

        return field;
    };

    this.addField = function(widget, field, position) {
        position = (position || widget.fields.length) - 1;
        widget.fields.splice(position, 0, field);
    };

    this.removeField = function(widget, fieldName) {
        widget.fields = widget.fields.filter(function(field) {
            return field.name !== fieldName;
        });
    };

    this.editField = function(widget, fieldName, editings) {
        var field = this.getField(widget, fieldName);
        Util.extend(field, editings);
    };

    this.getFilters = function(filters, filterName) {
        return filters.filter(function(filter) {
            return filter.name === filterName;
        });
    };

    this.addFilter = function(filters, filter) {
        filters.push(filter);
    };

    this.removeFilter = function(filters, filterName) {
        filters = filters.filter(function(filter) {
            return filter.name !== filterName;
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ZhuLogHelper', ZhuLogHelper);
});

// FILE: ./bower_components/tek-libraries/dist/main.js

// FILE: config/LibConfig.js
var loginBaseUrl = "https://rc-authentication.teknisa.com";

var LibConfig = {
    loginClientURL : 'http://libraries.teknisa.com/',
    loginURL       : loginBaseUrl + "/?redirectUri=",
    loginURI       : loginBaseUrl + "/backend/index.php"
};

// FILE: js/Util/Colors.js
function Colors() {

	this.TColorToHex = function(TColor) {
		var rgb = this.TColorToRGB(TColor);
		return this.rgbToHex(rgb);
	};

	this.TColorToRGB = function(TColor) {
        var r = TColor & 0xFF;
        var g = (TColor >> 8) & 0xFF;
        var b = (TColor >> 16) & 0xFF;
        return [r,g,b];
	};

	this.rgbToHex = function (rgb) {
	    return "#" + ((1 << 24) + (rgb[0] << 16) + (rgb[1] << 8) + rgb[2]).toString(16).slice(1);
	};
	
	this.RGBToTColor = function(r, g, b){
		var Tcolor = r | (g << 8) | (b << 16);
		return Tcolor;
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Colors', Colors);
});

// FILE: js/Util/Endereco.js
var Endereco = function (restEngine, ScreenService) {
    var self = this;

    //Busca pelo CEP na API dos correios e retorna o resultado por promise
    self.buscarCep = function(CEP) {
        CEP = CEP.replace(/[\D]+/g,'');

        if(CEP.length == 8) {
            return restEngine.post('/lib_findAddress', {
                    'requestType':'Row',
                    'row': {'CEP': CEP}
            });
        } else {
            ScreenService.showMessage('CEP Inválido.');
        }
    };

    /* Busca pelo CEP na API dos correios e trata o retorno.
     * Usar apenas de exemplo para fazer um método próprio!
     */
    self.validarCep = function(widget, CEP) {
        self.buscarCep(CEP).then(function(response) {
            var endereco = response.dataset.ENDERECO;

            if(endereco) {
                widget.currentRow.CEP = CEP;
                widget.currentRow.CDPAIS = (endereco.CDPAIS)? endereco.CDPAIS : '';
                widget.currentRow.NMPAIS = (endereco.NMPAIS)? endereco.NMPAIS : '';
                widget.currentRow.SGESTADO = (endereco.SGESTADO)? endereco.SGESTADO : '';
                widget.currentRow.NMESTADO = (endereco.NMESTADO)? endereco.NMESTADO : '';
                widget.currentRow.CDMUNICIPIO = (endereco.CDMUNICIPIO)? endereco.CDMUNICIPIO : '';
                widget.currentRow.NMMUNICIPIO = (endereco.NMMUNICIPIO)? endereco.NMMUNICIPIO : '';
                widget.currentRow.CDBAIRRO = (endereco.CDBAIRRO)? endereco.CDBAIRRO : '';
                widget.currentRow.NMBAIRRO = (endereco.NMBAIRRO)? endereco.NMBAIRRO : '';
                widget.currentRow.CDLOGRADOURO = (endereco.CDLOGRADOURO)? endereco.CDLOGRADOURO : '';
                widget.currentRow.DSLOGRADOURO = (endereco.DSLOGRADOURO)? endereco.DSLOGRADOURO : '';
                widget.currentRow.DSENDERECO = (endereco.DSENDERECO)? endereco.DSENDERECO : '';
            } else {
                widget.currentRow.CEP = '';
            }
        }, function(error) {
            var errorMessage = 'Falha na conexão. Configure o proxy para desenvolvimento!';

            widget.currentRow.CEP = '';

            if(error.data)
                errorMessage = error.data.error;

            ScreenService.showMessage("Erro na requisição do CEP '" + CEP + "': " + errorMessage, "warning");
        });
    };
};

Configuration(function (ContextRegister) {
    ContextRegister.register('Endereco', Endereco);
});

// FILE: js/Util/General.js
var General = function (ScreenService, templateManager, localStorageEngine, restEngine, maskEngine, ZhUtil, ZHPromise, ValidationEngine, metaDataFactory) {
	var searchListDeleted;
	var parent = this;

	//@TODO Criar objeto de mensagens para tradução.

	/*
	 * Função que atualiza o dataSourceFilter de um field que depende do resultado de
	 * outro. Realiza o reload apenas ao clicar no field, evitando requisições desnecessárias.
	 * OBS.: para o funcionamento correto, é necessário limpar o value do field dependente
	 * ao alterar o valor do field "pai".
	 *
	 * @param field dependentField
	 * @param array parameters
	 *
	 * Exemplo:
	 * General.onClickDependentField(meuField, {nomeParametro1: valorParametro1, nomeParametro2: valorParametro2});
	 */
	this.onClickDependentField = function (dependentField, parameters) {

		var dataSourceFilterNew = this.buildDataSourceFilter(angular.copy(dependentField.dataSourceFilter), parameters);

		var changedDataSourceFilter = compareDataSourceFilters(dependentField.selectWidget, dataSourceFilterNew, parameters);

		if (changedDataSourceFilter) {
			changeDependetFieldFilter(dependentField, dataSourceFilterNew);
		}

		if ((dependentField.selectWidget && changedDataSourceFilter) || (!dependentField.dataSource.data || !dependentField.dataSource.data.length) || searchListDeleted) {
			dependentField.reload();
		}
	};

	this.buildDataSourceFilter = function (dataSourceFilter, parameters) {
		if (!dataSourceFilter) {
			throw "DataSourceFilter is not defined!";
		}

		for (var position in dataSourceFilter) {
			if (parameters.hasOwnProperty(dataSourceFilter[position].name)) {
				dataSourceFilter[position].value = parameters[dataSourceFilter[position].name];
			}
		}

		return dataSourceFilter;
	};

	var changeDependetFieldFilter = function (dependentField, dataSourceFilterNew) {

		dependentField.dataSourceFilter = dataSourceFilterNew;
		dependentField.dataSource.lazyLoad = false;
		if (dependentField.selectWidget) {
			dependentField.selectWidget.dataSourceFilter = dataSourceFilterNew;
		}

		if (dependentField.selectWidget && dependentField.selectWidget.searchList) {

			searchListDeleted = true;
			delete dependentField.selectWidget.searchList;

		} else if (dependentField.selectWidget) {

			searchListDeleted = false;

		}

	};

	var compareDataSourceFilters = function (selectWidget, dataSourceFilterNew, parameters) {
		var changedDataSourceFilter = false;
		if (selectWidget) {
			var selectWidgetDataSourceFilter = selectWidget.dataSourceFilter;
			for (var position = 0; position < dataSourceFilterNew.length; position++) {
				if (parameters.hasOwnProperty(dataSourceFilterNew[position].name)) {
					if (selectWidgetDataSourceFilter[position].value != dataSourceFilterNew[position].value) {
						changedDataSourceFilter = true;
					}
				}
			}
		} else {
			changedDataSourceFilter = true;
		}
		return changedDataSourceFilter;
	};

	/*
	 * Função que gera e retorna a totalização e o nível para agrupamento dos pratos
	 * ou produtos correspondentes à máscara utilizada. Através de cada nível
	 * da máscara, é gerado o número correspondente à ele.
	 *
	 * @param string maskName
	 * @param int firstLevel
	 *
	 * @return array
	 *
	 * Exemplo:
	 * General.getLevelByMask('CDPICTPRAT', 1);
	 */
	this.getLevelByMask = function (maskName, firstLevel) {

		firstLevel = toNumber(firstLevel);

		var mask = getLocalVar(maskName);
		var splitMask = mask.split('.');
		var biggestLevel = splitMask.length;

		var level = [];
		var objLevel = {};

		for (var i = 0; i < biggestLevel; i++) {

			objLevel = {
				nivel: i + firstLevel,
				caracteres: i > 0 ? parseInt(level[i - 1].caracteres) + parseInt(splitMask[i].length) : parseInt(splitMask[i].length)
			};

			objLevel.nivel = objLevel.nivel.toString();
			objLevel.caracteres = objLevel.caracteres.toString();

			level.push(objLevel);
		}

		return level;
	};

	var toNumber = function (value) {

		if (!value && value !== 0) {

			throw 'firstLevel is not defined';

		} else if (typeof value !== 'number' && value) {

			value = parseInt(value);

			if (isNaN(parseInt(value))) {

				throw 'Invalid firstLevel';

			} else {

				return value;
			}
		}
		return value;
	};

	this.toggleReadOnlySelect = function (checkboxField, selectField) {

		if (checkboxField.checked) {
			selectField.readOnly = true;
			selectField.validations = {};
			selectField.setValue('');
		} else {
			selectField.readOnly = false;
			selectField.validations = {
				required: true
			};
		}
	};

	this.setLoggedUnit = function (widget) {
		widget.currentRow.CDFILIAL = getLocalVar('CDFILIAL');
		widget.currentRow.NMFILIAL = getLocalVar('NMFILIAL');
	};

	var currentIdgrupoper;
	var currentCallback;
	var currentParameters;
	var gruposDeOperador = {
		LCMR: 'A liberação de cardápio fora da meta dependente de autorização'
	};
	var fields = [
		{
			"name": "email",
			"label": "E-mail",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"template": "field/text-edit.html",
			"events": [],
			"fieldGroup": "AUTH",
			"validations": {
				"required": true,
				"mail": true
			}
        },
		{
			"name": "senha",
			"label": "Senha",
			"class": 12,
			"isVisible": true,
			"readOnly": false,
			"fieldGroup": "AUTH",
			"template": "field/password-edit.html",
			"events": [],
			"validations": {
				"required": true
			}
        }
    ];
	var actions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.validateUser(args.owner.widget);"
                }
            ]
        }
    ];

	this.openAuthPopup = function (idgrupoper, message, callback, parameters) {

		if (!idgrupoper) {
			throw 'idgrupoper is not defined';
		}

		initAuthVariables(idgrupoper, callback, parameters);

		if (!message) {
			message = gruposDeOperador[currentIdgrupoper] || '';
		}

		var widgetAuth = metaDataFactory.widgetFactory({
			"id": 0,
			"name": "authWidget",
			"label": "Autenticação",
			"template": "widget/form.html",
			"container": templateManager.container,
			"isVisible": true,
			"popupNoBlock": false,
			"fieldGroups": [
				{
					"id": 100000,
					"name": "AUTH",
					"opened": true,
					"collapsible": false,
					"label": message
                }
            ],
			"fields": fields,
			"actions": actions,
			"events": []
		});

		ScreenService.openPopup(widgetAuth);

	};

	var initAuthVariables = function (idgrupoper, callback, parameters) {
		currentIdgrupoper = idgrupoper;
		currentCallback = callback || null;
		currentParameters = parameters || [];
	};

	this.validateUser = function (widget) {

		var self = this;

		if (widget.isValid()) {

			var email = widget.currentRow.email;
			var senha = widget.currentRow.senha;

			autenticar(email, senha).then(function (callback) {

				getPermission(callback.dataset.user.cdoperador).then(function (permission) {

					if (permission) {

						if (typeof currentCallback == 'function') {

							ScreenService.closePopup();

							currentCallback.apply(null, currentParameters);

						} else {

							throw 'callback is not a function';

						}

					} else {

						var message = 'Operador não pertence ao grupo responsável pela autorização.';

						ScreenService.showMessage(ScreenService.i18n(message), 'error');

					}

				}, function (callback) {

					this.callbackError(callback);

				}.bind(self));

			}, function (callback) {

				this.callbackError(callback);

			}.bind(self));
		}

	};

	var autenticar = function (email, senha) {

		return restEngine.request('/Autenticar', {
			"requestType": "FilterData",
			"filter": [
				{
					name: "email",
					operator: "=",
					value: email
                },
				{
					name: "senha",
					operator: "=",
					value: senha
                }
            ]
		});

	};

	var getPermission = function (cdoperador) {

		return restEngine.request('/validateUserGroup', {
			"requestType": "Row",
			"row": {
				"idgrupoper": currentIdgrupoper,
				"cdoperador": cdoperador
			}
		}).then(function (callback) {

			return callback.dataset.permission.isValid;
		});

	};

	this.callbackError = function (callback) {

		if (callback.data) {

			ScreenService.showMessage(ScreenService.i18n(callback.data.error));

		} else {

			ScreenService.showMessage(ScreenService.i18n('Timeout'));
		}
	};

	this.comparaPratoProduto = function (pinicial, pfinal, mascara) {
		var mask = {
			type: "fix",
			params: {
				mask: mascara
			}
		};
		pinicial = Util.applyMask(pinicial, mask, maskEngine, ZhUtil).split('.');
		pfinal = Util.applyMask(pfinal, mask, maskEngine, ZhUtil).split('.');
		var nvinicial = pinicial.length;
		var nvfinal = pfinal.length;
		var menornv;
		if (nvinicial > nvfinal) {
			menornv = nvfinal;
		} else {
			menornv = nvinicial;
		}

		var resultado = true;
		var i = 0;

		while (pinicial[i] == pfinal[i] && i < menornv) {
			i++;
		}

		if ((pinicial.hasOwnProperty(i) && !pfinal.hasOwnProperty(i)) || (pinicial[i] > pfinal[i])) {
			resultado = false;
		} else {
			resultado = true;
		}

		return resultado;
	};

	var reportWidget = {
		"id": 0,
		"name": "reportWidget",
		"label": "Tipo de Relatório",
		"template": "widget/form.html",
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};
	var reportFields = [
		{
			"id": 99999999,
			"name": "QR2TYPE",
			"label": "",
			"class": 12,
			"readOnly": false,
			"isVisible": true,
			"isVisibleForm": true,
			"isVisibleGrid": false,
			"template": "field/radio-group.html",
			"events": [],
			"valueField": "value",
			"descriptionField": "name",
			"defaultOption": 0,
			"dataSource": {
				"data": [
					{
						"name": "pdf",
						"value": "PDF"
                    },
					{
						"name": "xls",
						"value": "XLS"
                    },
					{
						"name": "rtf",
						"value": "RTF"
                    },
					{
						"name": "txt",
						"value": "TXT"
                    }
                ]
			}
        }
    ];
	var reportActions = [
		{
			"name": "cancelar",
			"label": "Cancelar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "back",
			"events": [
				{
					"name": "ActionEvent",
					"code": "ScreenService.closePopup();"
                }
            ]
        },
		{
			"name": "confirmar",
			"label": "Confirmar",
			"idVisible": true,
			"hideIcon": true,
			"showAsAction": "never",
			"events": [
				{
					"name": "ActionEvent",
					"code": "General.generateReport(args.owner.widget, args);"
                }
            ]
        }
    ];

	var reportCallback;
	var reportParameters;
	var reportWidgetParentName;

	this.openReportPopup = function (widget, callback, parameters) {

		reportCallback = callback;
		reportParameters = parameters || [];
		reportWidgetParentName = widget.name;
		reportWidget.container = widget.container;
		reportWidget.fields = reportFields;
		reportWidget.actions = reportActions;

		var reportWidgetInitialized = metaDataFactory.widgetFactory(reportWidget);

		ScreenService.openPopup(reportWidgetInitialized);
	};

	this.generateReport = function (widget, args) {

		var reportWidgetParentCurrentRow = widget.container.getWidget(reportWidgetParentName).currentRow;

		reportWidgetParentCurrentRow.QR2TYPE = widget.currentRow.QR2TYPE;

		args.row = reportWidgetParentCurrentRow;

		if (typeof reportCallback == 'function') {

			reportCallback.apply(null, reportParameters);
		}

		ScreenService.closePopup();

	};

	this.saveRow = function (args, closeSwipe) {
		args.owner.widget.saveRow().then(function () {
			var currentRow = args.owner.widget.currentRow;
			currentRow.__is_new = false;
			currentRow.changed = false;
			args
				.owner
				.widget
				.reload();
			if (closeSwipe) {
				ScreenService.closeSwipe();
			}
		});
	};

	this.openSelectionOnce = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		if (!filterWidget.wasOpened) {
			filterWidget.edit();
			ScreenService.openPopup(filterWidget);
			filterWidget.wasOpened = true;
		}
	};

	this.openSelection = function (name) {
		var filterWidget = templateManager.container.getWidget(name + "_FILTER");
		filterWidget.edit();
		ScreenService.openPopup(filterWidget);
	};

	this.getContainerName = function () {
		return templateManager.container.name;
	};

	this.backOrCancel = function (args) {
		var deferred = ZHPromise.defer();
		if (args.row.__is_new) {
			args.owner.widget.list(true);
			args.owner.widget.dataSource.data.pop();
			args.owner.widget.currentRow = {};
			deferred.resolve();
		} else if (args.owner.widget.mode === 'edit') {
			args.owner.widget.view();
			for (var i in args.owner.widget.backupRow) {
				args.owner.widget.currentRow[i] = args.owner.widget.backupRow[i];
			}
			deferred.resolve();
			return deferred.promise;
		}
		ScreenService.goBack();
		return deferred.promise;
	};

	this.edit = function (widget) {
		widget.backupRow = angular.copy(widget.currentRow);
		widget.edit();
	};

	this.modEdit = function (widget) {
		parent.edit(widget);
		var widgetParent = widget.widgets;
		if (widgetParent.length)
			parent.setVisibleforTabbed(widgetParent, false, widget);
		// widget.template = widget.formTemplate = widget.templateToForm;
	};

	this.cancelModEdit = function (args, widgetParent) {
		parent.backOrCancel(args).then(function (success) {
			var widget = args.owner.widget;
			var widgetParent = widget.widgets;
			if (widgetParent.length)
				parent.setVisibleforTabbed(widgetParent, true);
			// widget.template = widget.formTemplate = widget.templateToView;
		});
	};

	this.setVisibleforTabbed = function (widgets, isVisible, exception) {
		widgets.forEach(function (widget) {
			if (exception) {
				if (exception.name != widget.name)
					widget.isVisible = isVisible;
			} else {
				widget.isVisible = isVisible;
			}
		});
	};

	this.setContextVar = function (name, containerName, value) {
		localStorageEngine.setLocalVar(containerName + '_' + name, value);
	};

	this.getContextVar = function (name, containerName) {
		return localStorageEngine.getLocalVar(containerName + '_' + name);
	};

	var _getWidget = function (widgetName) {
		var isPopup = arguments[1] || null;
		if (!!templateManager.container) {
			var widget;
			if (!isPopup) {
				widget = templateManager.container.getWidget(widgetName);
			} else {
				widget = templateManager.container.getWidget(widgetName);
				if(widget)
					if(widget.hasOwnProperty('__filterPopup')) 
						widget = widget.__filterPopup;
					else{
						widget = widget.appliedFilter;
					}
			}
			if (widget) {
				return widget;
			}
		}
		return null;
	};

	var _getField = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var field = widget.getField(name);
		if (field) {
			return field;
		}
		return null;
	};

	var _getAction = function (name, widgetName) {
		var widget = _getWidget(widgetName);
		var action = widget.getAction(name);
		if (action) {
			return action;
		}
		return null;
	};

	this.getFieldValue = function (name, widgetName) {
		var isPopup = arguments[2] || null;
		var widget = _getWidget(widgetName, isPopup);
		if (widget && widget.currentRow) {
			return widget.currentRow[name];
		} else {
			return "";
		}
	};

	this.setEnableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = false;
	};

	this.setDisableField = function (name, widgetName) {
		_getField(name, widgetName).readOnly = true;
	};

	this.setVisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = true;
	};

	this.setInvisibleField = function (name, widgetName) {
		_getField(name, widgetName).isVisible = false;
	};

	this.setEnableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = false;
	};

	this.setDisableAction = function (name, widgetName) {
		_getAction(name, widgetName).readOnly = true;
	};

	this.setVisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = true;
	};

	this.setInvisibleAction = function (name, widgetName) {
		_getAction(name, widgetName).isVisible = false;
	};

	this.now = function () {
		return moment(new Date()).format('DD/MM/YYYY HH:mm:ss');
	};

	this.today = function () {
		return moment(new Date()).format('DD/MM/YYYY');
	};

	this.openReport = function (route, reportName, params) {
		params.__report_name = reportName;
		return restEngine.post(route, {
			"requestType": "Row",
			"row": params
		}, null, null, null, null, 3000000).then(function (response) {
			if (response && response.messages) {
				var report = response.messages.pop();
				var reportOpened = window.open(report.message);
				if(!reportOpened){
				    ScreenService.showMessage(ScreenService.i18n('Pop-up Blocker is enabled! Please add this site to your exception list.'));
				}
			}
		}).catch(function (response) {
			if (response.data && response.data.error) {
				ScreenService.showMessage(ScreenService.i18n(response.data.error), 'E');
			}
		});
	};

	// CRUD
	/**
     * O segundo parâmetro é opcional, sendo ele um objeto contendo os atributos
     * callback (function) e params (array). Após a validação da widget,
     * o método callback é executado.
     * Ex: var afterValid = {
     *      callback: function(a) {console.log(a);},
            params: ['valorDeA']
     * };
     */
	this.saveCrud = function (widget, afterValid) {
		var deferred = ZHPromise.defer();
		if (widget.isValid()) {
			if (afterValid && angular.isObject(afterValid)) {
				if (angular.isFunction(afterValid.callback))
					afterValid.callback.apply(null, afterValid.params);
			}

			var paramsNewCode = widget.paramsNewCode;
			if (paramsNewCode && paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = paramsNewCode.__table_pk;
				widget.currentRow.__new_code_length = paramsNewCode.__new_code_length || 5;
				widget.currentRow.__cast_number = paramsNewCode.__cast_number || false;
			}

			var fieldsToValidateDuplication = widget.fieldsToValidateDuplication;
			if (fieldsToValidateDuplication && fieldsToValidateDuplication.length){
				setDuplicationValidateParamsToCurrentRow(widget);
			}

			widget.saveRow().then(function (success) {
				widget.activate();
				if (widget.currentRow.__is_new) {
					widget.list(true);
					ScreenService.goBack();
					widget.reload().then(function () {
						deferred.resolve(success);
					});
				} else {
					widget.computeDescriptionOrder();
					widget.view();
					deferred.resolve(success);
				}
			}, function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else {
			// @todo create messages class
			deferred.reject("Preencha todos os campos corretamente!");
		}

		return deferred.promise;
	};

	/* setDuplicationValidateParamsToCurrentRow
	 *
	 * Caso o widget contenha o array 'fieldsToValidateDuplication', será verifi-
	 * cado, antes de salvar, se os valores dos campos a serem validados já exis-
	 * tem no banco.
	 *
	 * O array deve conter objeto(s) com o nome da coluna na tabela e a descricao
	 * do campo que sera exibida na resposta em caso de duplicacao.
	 *
	 * Exemplo:
	 *      "fieldsToValidateDuplication": [
	 *        {
	 *	        "name": "NMCAMPO",
	 *	        "description": "campo"
	 *        }
	 *      ]
	 */
	var setDuplicationValidateParamsToCurrentRow = function (widget) {
		try {
			var fieldsToValidate = widget.fieldsToValidateDuplication;
			var validationArray = [];

			checkValidateFieldsArray(fieldsToValidate);

			for (var index in fieldsToValidate) {
				var field = fieldsToValidate[index];

				if (!field.description) {
					field.description = field.name;
					var warningMsg = "Missing description for " + field.name +
					" field in 'fieldsToValidateDuplication' of " + widget.name;
					console.warn(warningMsg);
				}
				field.description = field.description.toLowerCase();

				validationArray.push(field);
			}

			if (validationArray.length) {
				widget.currentRow.__fieldsToValidateDuplication = validationArray;
			}
		} catch (e) {
			ScreenService.showMessage(e);
		}
	};

	var checkValidateFieldsArray = function (arrayToCheck) {
		for (var index in arrayToCheck) {
			var objectToCheck = arrayToCheck[index];

			if (typeof(objectToCheck) == "object") {
				if (!objectToCheck.name) {
					throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has elements without name.";
				}
			} else {
				throw "Error in data save. Property 'fieldsToValidateDuplication' of widget has non-object elements.";
			}
		}
	};

	this.deleteCrud = function (widget) {
		var deferred = ZHPromise.defer();
		ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
			widget.removeRow().then(function () {
				ScreenService.closeSwipe().then(function () {
					widget.activate();
					deferred.resolve();
				});
			}, function (error) {
				if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
					ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
				} else {
					ScreenService.showMessage(error.data.error);
				}
				deferred.reject();
			});
		});
		return deferred.promise;
	};

    this.deleteCrudCheckedRows = function (widget) {
        var deferred = ZHPromise.defer();
        ScreenService.confirmMessage(ScreenService.i18n('Delete the selected registers?'), 'question', function (success) {
            widget.removeCheckedRows().then(function () {
                widget.reload();
                widget.unSelectAll();
                deferred.resolve();
            }, function (error) {
                if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
                    ScreenService.showMessage('Exclusão não permitida! O(s) registro(s) possui(em) registros filhos.');
                } else {
                    ScreenService.showMessage(error.data.error);
                }
                deferred.reject();
            });
        });
        return deferred.promise;
    };

	this.justInteger = function (selector, retryCount) {
		if (!retryCount || retryCount > 5)
			retryCount = 5;
		if (!selector) {
			throw {
				"message": "Selector is not defined!"
			};
		}
		setTimeout(function () {
			var elem = $(selector);
			if (elem[0]) {
				var length = selector.split(',').length;
				for (var i = 0; i < length; i++) {
					if (elem.hasOwnProperty(i)) {
						elem[i].onkeypress = avoidNonInteger;
						elem[i].onpaste = preventEvent;
					}
				}
			} else {
				if (!retryCount) {
					console.error("Selector not found!");
				} else {
					retryCount--;
					this.justInteger(selector, retryCount);
				}
			}
		}.bind(this), 500);
	};

	function avoidNonInteger(e) {
		var integerNumbers = [48, 49, 50, 51, 52, 53, 54, 55, 56, 57];
		var numberCode = e.which;
		if (integerNumbers.indexOf(numberCode) < 0) {
			preventEvent(e);
		}
	}

	function preventEvent(e) {
		e.preventDefault();
	}

	function padLeft(value, length) {
		return (value.toString().length < length) ? padLeft("0" + value, length) : value;
	}

	function padRight(value, length) {
		return (value.toString().length < length) ? padRight(value + "0", length) : value;
	}

	this.getCurrentDate = function (format) {
		return moment().format(format);
	};

	this.setDateAttrToVarcharOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, currentRow[attrName].replace(':', ''));
	};

	this.setAttrOnCurrentRow = function (currentRow, attrName, value) {
		currentRow[attrName] = value;
	};

	this.setFloatAttrOnCurrentRow = function (currentRow, attrName) {
		this.setAttrOnCurrentRow(currentRow, attrName, this.valueToFloat(currentRow[attrName]));
	};

	this.valueToFloat = function (value) {
		if (typeof value == 'string' && !Util.isEmptyOrBlank(value)) {
			if (value.indexOf(',') == -1 && value.indexOf('.') == -1) {
				return parseFloat(value);
			} else if (value.indexOf(',') > value.indexOf('.')) {
				return parseFloat(value.replace(/\./g, '').replace(/\,/g, '.'));
			} else if (value.indexOf(',') < value.indexOf('.')) {
				return parseFloat(value.replace(/\,/g, ''));
			}
		} else {
			return value;
		}
	};

	this.toLocalDate = function (element, viewValue, params, callBack) {
		var value = viewValue || '';
		element.val(value);
		params.callback = callBack;
		$.currency.setMask(params, element);
	};

	this.toLocalDateFormat = function (value, params) {
		if (value) {
			var valueDate = moment(value, params.format);
			value = valueDate.format(params.toFormat);
		}

		return value;
	};

	maskEngine.registerMask('dateFormat', this.toLocalDate, this.toLocalDateFormat);

	var maskFunction = function (element, viewValue, params, callBack) {
		// do something
	};

	var maskFormatFunction = function (value, params) {
		if (value) {
			var length = value.length;
			if (length === 11)
				value = value.replace(/^(\d{3})(\d{3})(\d{3})(\d{2})/, "$1.$2.$3-$4");
			else if (length === 14)
				value = value.replace(/^(\d{2})(\d{3})(\d{3})(\d{4})(\d{2})/, "$1.$2.$3/$4-$5");
			return value;
		}
	};

	maskEngine.registerMask('CPFCNPJ', maskFunction, maskFormatFunction);

	/*
	 * Função que verifica quais widgets serão exibidas baseado em quais projetos ela pode ser exibida.
	 * A função deve ser executada no 'ContainerAfterInit' mas deve ser estudado a possibilidade de ser executado
	 * no 'EventAgregatorOnOpenContainer' para que assim essa configuração possa ser aplicada em multiplas telas.
	 */

	var constAllWidgets;

	this.setVisibilityByProjectID = function (widgets) {
		if (widgets.length > 1) {
			widgets.forEach(function (childWidget) {
				parent.setVisibilityByProjectID(childWidget.widgets);
				if (childWidget.projects) {
					childWidget.projects.forEach(function (proj) {
						childWidget.isVisible = proj.projectID == templateManager.project.id || proj.projectID == "allProjects" ? true : false;
					});
				}
			});

		}

	};

	this.validateFilterPopup = function (args) {
		if(args.owner.hasOwnProperty('__filterPopup')){
			var filterPopup = args.owner.__filterPopup;
		
		if (!filterPopup.isValid())
			args.e.preventDefault();
		}	
	};

	var timeFormats = {
		"hh": isValidHour,
		"mm": isValidMinutesOrSeconds,
		"ss": isValidMinutesOrSeconds,
		"hh:mm": isValidHourAndMinutes,
		"mm:ss": isValidMinutesAndSeconds,
		"hh:mm:ss": isValidTime,
	};

	function isValidHour(hour) {
		if (parseInt(hour) > 23 || parseInt(hour) < 0)
			return false;
		else
			return true;
	}

	function isValidMinutesOrSeconds(minuteOrSecond) {
		if (parseInt(minuteOrSecond) > 59 || parseInt(minuteOrSecond) < 0)
			return false;
		else
			return true;
	}

	function isValidHourAndMinutes(hourAndMinute) {
		if (hourAndMinute) {
			var timeArray = hourAndMinute.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidMinutesAndSeconds(minutesAndSeconds) {
		if (minutesAndSeconds) {
			var timeArray = minutesAndSeconds.split(':');
			return isValidMinutesOrSeconds(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]);
		}
		return true;
	}

	function isValidTime(time) {
		if (time) {
			var timeArray = time.split(':');
			return isValidHour(timeArray[0]) && isValidMinutesOrSeconds(timeArray[1]) && isValidMinutesOrSeconds(timeArray[2]);
		}
		return true;
	}

	var validTime = function (value, format) {
		var validFunction;
		if (!format)
			throw 'format is not defined';
		if (typeof format == 'string' && timeFormats.hasOwnProperty(format.toLowerCase()))
			validFunction = timeFormats[format.toLowerCase()];
		else
			throw 'time format is not registered';
		return {
			"valid": validFunction(value),
			"message": ScreenService.i18n("Is not a valid time")
		};
	};

	this.registerTimeFormat = function (format, formatFunction) {
		if (typeof format !== 'string')
			throw 'time format must be an string';
		if (timeFormats.hasOwnProperty(format.toLowerCase()))
			throw 'time format already registered';
		timeFormats[format] = formatFunction;
	};

	ValidationEngine.registerValidation('isValidTime', validTime);

	/*
	 * Função que preenche um valor com um único caractere até um certo tamanho.
	 * Faz zerofill pela esquerda por default.
	 *
	 * @param string value
	 * @param int length
	 * @param boolean toRight
	 * @param char filler
	 *
	 * @return string
	 *
	 * Exemplos:
	 * General.fill('123', 10);            //'0000000123'
	 * General.fill('123', 10, true);      //'1230000000'
	 * General.fill('123', 10, null, '_'); //'_______123'
	 */
	this.fill = function (value, length, toRight, filler) {
		if (value) {
			value = value.toString();

			if (filler) {
				filler = filler.toString()[0];
			} else filler = '0';

			if (toRight)
				while (value.length + filler.length <= length) value = value + filler;
			else
				while (value.length + filler.length <= length) value = filler + value;
		}
		return value;
	};

	/*
	 * Função que faz a validação de CNPJ.
	 *
	 * @param string CNPJ
	 *
	 * @return boolean
	 */
	function validCNPJ(CNPJ) {
		if (CNPJ === undefined || CNPJ === null)
			return false;

		CNPJ = CNPJ.toString().replace(/[\D]+/g, '');

		if (CNPJ.length != 14)
			return false;

		if (CNPJ == "00000000000000" ||
			CNPJ == "11111111111111" ||
			CNPJ == "22222222222222" ||
			CNPJ == "33333333333333" ||
			CNPJ == "44444444444444" ||
			CNPJ == "55555555555555" ||
			CNPJ == "66666666666666" ||
			CNPJ == "77777777777777" ||
			CNPJ == "88888888888888" ||
			CNPJ == "99999999999999")
			return false;

		var tamanho = CNPJ.length - 2;
		var numeros = CNPJ.substring(0, tamanho);
		var digitos = CNPJ.substring(tamanho);
		var soma = 0;
		var pos = tamanho - 7;

		for (var i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		var resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(0)) //debugger;
			return false; //'resultado' é o 1º digito de validação do CNPJ

		tamanho = tamanho + 1;
		numeros = CNPJ.substring(0, tamanho);
		soma = 0;
		pos = tamanho - 7;

		for (i = tamanho; i >= 1; i--) {
			soma += numeros.charAt(tamanho - i) * pos--;
			if (pos < 2)
				pos = 9;
		}

		resultado = soma % 11 < 2 ? 0 : 11 - soma % 11;
		if (resultado != digitos.charAt(1)) //debugger;
			return false; //'resultado' é o 2º digito de validação do CNPJ

		return true;
	}

	this.validateCNPJ = validCNPJ;

	/*
	 * Função que faz a validação de CPF.
	 *
	 * @param string CPF
	 *
	 * @return boolean
	 */
	function validCPF(CPF) {
		if (CPF === undefined || CPF === null)
			return false;

		CPF = CPF.toString().replace(/[\D]+/g, '');

		if (CPF.length != 11)
			return false;

		if (CPF == "00000000000" ||
			CPF == "11111111111" ||
			CPF == "22222222222" ||
			CPF == "33333333333" ||
			CPF == "44444444444" ||
			CPF == "55555555555" ||
			CPF == "66666666666" ||
			CPF == "77777777777" ||
			CPF == "88888888888" ||
			CPF == "99999999999")
			return false;

		var soma = 0;
		var resto;

		for (var i = 1; i <= 9; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (11 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(9, 10))) //debugger;
			return false; //'resto' é o 1º dígito de validação do CPF

		soma = 0;
		for (i = 1; i <= 10; i++) soma = soma + parseInt(CPF.substring(i - 1, i)) * (12 - i);
		resto = (soma * 10) % 11;

		if ((resto == 10) || (resto == 11)) resto = 0;
		if (resto != parseInt(CPF.substring(10, 11))) //debugger;
			return false; //'resto' é o 2º dígito de validação do CPF

		return true;
	}

	this.validateCPF = validCPF;

	/*
	 * Função que faz a formatação de uma inscrição,
	 * seguindo o padrão de código genérico, CPF, CNPJ e
	 * inscrição genérica. Caso o parâmetro "force" for
	 * true, o método tenta forçar uma inscrição válida.
	 *
	 * Retorna um objeto com as seguintes propriedades:
	 *  * parameter //string de entrada contendo apenas números
	 *  * value     //string formatada conforme o tipo da inscrição
	 *  * tipo.cod  //char do código padrão do tipo da inscrição
	 *  * tipo.desc //string de descrição do tipo da inscrição
	 *  * valid     //boolean
	 *
	 * @param string inscricao
	 * @param boolean force
	 *
	 * @return object
	 */
	this.inscricaoFormat = function (inscricao, force) {
		if (inscricao === undefined || inscricao === null || inscricao === '')
			return {
				'parameter': '',
				'value': '',
				'tipo': {
					'cod': null,
					'desc': null
				},
				'valid': false
			};

		if (force === undefined || force === null)
			force = false;
		inscricao = inscricao.toString().replace(/[\D]+/g, '');

		var obj = {};
		obj.tipo = {};

		if (inscricao.length <= 5) {
			obj.tipo.cod = 'O';
			obj.tipo.desc = 'Código Genérico';
			obj.valid = true;
			obj.value = this.fill(inscricao, 5);
		} else {
			if (inscricao.length == 11) {
				obj.tipo.cod = 'F';
				obj.tipo.desc = 'Pessoa Física';
				obj.valid = this.validateCPF(inscricao);
				obj.value = inscricao.substr(0, 3) + '.' + inscricao.substr(3, 3) + '.' + inscricao.substr(6, 3) + '-' + inscricao.substr(9, 2);
			} else if (inscricao.length == 14) {
				obj.tipo.cod = 'J';
				obj.tipo.desc = 'Pessoa Jurídica';
				obj.valid = this.validateCNPJ(inscricao);
				obj.value = inscricao.substr(0, 2) + '.' + inscricao.substr(2, 3) + '.' + inscricao.substr(5, 3) + '/' + inscricao.substr(8, 4) + '-' + inscricao.substr(12, 2);
			} else force = true;

			if (force && obj.valid !== true) {
				obj.tipo.cod = 'I';
				obj.tipo.desc = 'Inscrição Internacional';

				if (inscricao.length <= 20) {
					obj.valid = true;
					obj.value = this.fill(inscricao, 20);
				} else {
					obj.valid = false;
					obj.value = inscricao.substr(0, 20);
				}
			}
		}

		obj.parameter = inscricao;
		return obj;
	};

	var CPFValidation = function (value, validationAttrs) {
		return {
			"valid": validCPF(value),
			"message": ScreenService.i18n('CPF Inválido')
		};
	};

	ValidationEngine.registerValidation('CPF', CPFValidation);

	var CNPJValidation = function (value, validationAttrs) {
		return {
			"valid": validCNPJ(value),
			"message": ScreenService.i18n('CNPJ Inválido')
		};
	};

	ValidationEngine.registerValidation('CNPJ', CNPJValidation);

	this.callRestEngine = function (dataSource, requestType, requestValue) {
		var defer = ZHPromise.defer();
		var params = {};

		switch (requestType) {
		case "Row":
			params = {
				"requestType": "Row",
				"row": requestValue
			};
			break;

		case "DataSet":
			params = {
				"requestType": "DataSet",
				"dataset": requestValue
			};
			break;

		case "FilterData":
			params = {
				"requestType": "FilterData",
				"filter": requestValue
			};
			break;

		default:
			return defer.reject("Erro na passagem de parâmetros para a requisição. O valor para requestType deve ser 'Row', 'DataSet' ou 'FilterData'.");
		}

		return restEngine.request(dataSource, params);
	};

	/*
	 * Função que muda a acessibilidade de uma ou mais
	 * widgets.
	 *
	 * @param array(objects) widgets
	 * @param boolean enabled
	 *
	 * Exemplos:
	 * General.changeWidgetAcessibility(widget.widgets, false); //Desabilita o acesso às widgets filhas
	 * General.changeWidgetAcessibility(widget.widgets, true);  //Habilita o acesso às widgets filhas e faz algo mais depois
	 */
	this.changeWidgetAcessibility = function (widgets, enabled) {
		widgets.forEach(function (widget) {
			widget.enabled = enabled;
		});

		templateManager.updateTemplate();
	};

	/*
	 * Função que altera o template de um form
	 * entre edição e visualização. Retorna um boolean
	 * que indica se a widget era do tipo form e a
	 * alteração foi realizada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.formWidgetSwitchTemplate = function (widget) {
		if ((widget.template != "widget/form.html") && (widget.template != "widget/form-view.html")) return false;

		if (!widget.templateToForm) widget.templateToForm = "widget/form.html";
		if (!widget.templateToView) widget.templateToView = "widget/form-view.html";

		if (widget.template == widget.templateToForm) widget.template = widget.templateToView;
		else if (widget.template == widget.templateToView) widget.template = widget.templateToForm;

		templateManager.updateTemplate();

		return true;
	};

	/*
	 * Função que coloca uma widget em modo de edição,
	 * bloqueando as demais widgets. Retorna um boolean
	 * que indica se a alteração de modo foi executada.
	 *
	 * @param object widget
	 *
	 * @return boolean
	 */
	this.widgetEdit = function (widget) {
		if (!widget.editing) {
			//Trata a edição de uma widget filha do tipo form
			if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
				var widgetsToDeactivate = widget.parent.widgets;

				this.changeWidgetAcessibility(
					widgetsToDeactivate.filter(function (subwidget) {
						return subwidget.name != widget.name;
					}).concat(widget.parent), //retorna um array com a widget parent e suas widgets filhas, exceto aquela que será editada
					false);
			} else //trata a edição dos demais casos
				this.changeWidgetAcessibility(widget.widgets, false);

			//propriedades a serem ignoradas na criação do backup
			var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];

			//faz backup da currentRow para permitir a recuperação dos dados
			widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
			widget.edit();
			return true;
		} else return false;
	};

	/*
	 * Procedimento que cancela o formulário de uma
	 * widget, seja este de inserção ou de edição, e
	 * desbloqueia as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetCancelForm = function (widget) {
		//trata o cancelamento de uma widget filha do tipo form
		if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
			var widgets = widget.parent.widgets;

			this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
		} else //trata o cancelamento dos demais casos
			this.changeWidgetAcessibility(widget.widgets, true);

		//caso a widget estiver em modo de inserção, remove a row e fecha o swipe
		if (widget.currentRow.__is_new) {
			var indexOfCurrentRow = widget.dataSource.data.indexOf(widget.currentRow);
			if (indexOfCurrentRow !== -1) {
				widget.dataSource.data.pop();
			}

			widget.list();

			ScreenService.closeSwipe();
		} else { //caso a widget estiver em modo de edição, recupera o backup e entra no modo de visualização
			Object.keys(widget.backupRow).forEach(function (key) {
				widget.currentRow[key] = widget.backupRow[key];
			}); //copia dados coluna por coluna para não perder referência do objeto da currentRow
			widget.view();
		}
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * própria widget. Provavelmente usado na action de
	 * 'voltar' do back_view.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToList = function (widget) {
		ScreenService.closeSwipe();
		widget.list();
	};

	/*
	 * Procedimento que fecha o swipe e volta ao grid da
	 * widget parent. Provavelmente usado na action de
	 * 'voltar' do back_list.
	 *
	 * @param object widget
	 */
	this.widgetGoBackToParent = function (widget) {
		ScreenService.closeSwipe().then(function () {
			widget.parent.list();
			widget.parent.activate();
		});
	};

	/*
	 * Procedimento que habilita inserção em uma widget,
	 * bloqueando as demais widgets.
	 *
	 * @param object widget
	 */
	this.widgetNewRow = function (widget) {
		this.changeWidgetAcessibility(widget.widgets, false);

		widget.newRow();
	};

	/*
	 * Função que salva os dados de uma widget e retorna
	 * para o modo adequado de visualização ou listagem.
	 * Faz exibição de mensagens de aviso para o usuário
	 * conforme padronização e desbloqueia as demais
	 * widgets.
	 *
	 * Basicamente, um saveCrud adaptado ao padrão.
	 *
	 * @param object widget
	 *
	 * @return promise
	 */
	this.widgetSave = function (widget) {
		var deferred = ZHPromise.defer();

		if (widget.isValid()) {
			widget.currentRow.changed = true;

			if (!widget.currentRow.__is_new)
				widget.currentRow.__is_new = false;
			else if (widget.paramsNewCode && widget.paramsNewCode.__is_new_code) {
				widget.currentRow.__using_unit = widget.paramsNewCode.__using_unit || false;
				widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
				widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
				widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
				widget.currentRow.__cast_number = widget.paramsNewCode.__cast_number || false;
                if (!widget.currentRow.__columns_to_return && widget.paramsNewCode && widget.paramsNewCode.__columns_to_return)
                	widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
			}

			widget.saveRow().then(function (response) {
				widget.currentRow.changed = false;
				delete widget.backupRow;

				if (this.formWidgetSwitchTemplate(widget) && widget.parent) {
					widget.view();

					var widgets = widget.parent.widgets;
					this.changeWidgetAcessibility(widgets.concat(widget.parent), true);
				} else {
					if (widget.currentRow.__is_new) {
						ScreenService.closeSwipe();
						widget.list();
					} else widget.view();

					this.changeWidgetAcessibility(widget.widgets, true);
				}

				if (widget.currentRow.__is_new) widget.currentRow.__is_new = false;
				else widget.reload();

				deferred.resolve(response);
			}.bind(this), function (error) {
				var errorMessage = 'Timeout';

				if (error.data) errorMessage = error.data.error;

				ScreenService.showMessage(errorMessage);
				deferred.reject(errorMessage);
			});
		} else deferred.reject("Preencha todos os campos corretamente!");

		return deferred;
	};

	/*
	 * Função para validar containers abertos a partir de outros containers.
	 * Se o container atual não foi aberto a partir do container pai, a
	 * callback é executada. Se nenhuma callback for passada nos parâmetros,
	 * o container pai é aberto.
	 *
	 * @param: object {parentContainerName, callback, scope, params}
	 *
	 * Exemplos:
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato',callback: ScreenService.showMessage, scope: ScreenService, params:['Teste']});
	 * General.checkParentContainer({parentContainerName: 'pla_prato#pla01300_prato'});
	 */
	this.checkParentContainer = function (containerProperties) {
		containers = templateManager.containers;
		if (!containers.hasOwnProperty(containerProperties.parentContainerName)) {
			if (angular.isFunction(containerProperties.callback)) {
				containerProperties.callback.apply(containerProperties.scope, containerProperties.params);
			} else {
				ScreenService.openWindow(containerProperties.parentContainerName);
			}
		}
	};

	var messagesWidget = {
		"id": 0,
		"name": "messagesWidget",
		"label": "Mensagens",
		"messagesDataset": "",
		"template": "widget/form.html",
		"container": {},
		"isVisible": true,
		"popupNoBlock": false,
		"fields": [],
		"actions": [],
		"events": []
	};

	var messagesFields = [{
		"label": "",
		"name": "CONTENT",
		"readOnly": true,
		"template": "field/textarea.html",
		"textRows": 14,
		"isVisible": true,
		"class": 12,
		"descriptionOrder": true,
		"showOnView": true,
		"showOnForm": true,
		"showOnList": true
    }];

	var messagesActions = [{
		"name": "cancelar",
		"label": "Cancelar",
		"idVisible": true,
		"hideIcon": true,
		"showAsAction": "back",
		"events": [{
			"name": "ActionEvent",
			"code": "ScreenService.closePopup();"
        }]
    }, {
		"name": "exportar",
		"label": "Exportar",
		"idVisible": true,
		"icon": "export",
		"showAsAction": "always",
		"events": [{
			"name": "ActionEvent",
			"code": "General.generateFileByDataSet(args.owner.widget.messagesDataset);"
        }]
    }];

	/* Função abre um popup com um Text Area a título de mostrar erros em lista.
	 * Recebe a widget de origem e um dataset com mensagens a serem exibidas.
	 * @params: widget, dataset, popupName
	 */
	this.showErrorMessages = function (widget, dataset, popupName) {
		messagesWidget.container = widget.container;
		messagesWidget.fields = messagesFields;
		messagesWidget.actions = messagesActions;

		var messagesWidgetInitialized = metaDataFactory.widgetFactory(messagesWidget);
		var content = '';
		if (popupName !== undefined)
			messagesWidgetInitialized.label = popupName;
		messagesWidgetInitialized.messagesDataset = dataset;
		dataset.forEach(function (message) {
			content += message + '\n';
		});
		messagesWidgetInitialized.getField('CONTENT').setValue(content);
		ScreenService.openPopup(messagesWidgetInitialized);
	};

	/* Gera arquivo básico baseado em linhas de dataSet.
	 * @params: dataset, object {fileName}
	 * !!! fileExtension: por enquanto somente TXT;
	 * Exemplo:
	 * General.generateFileByDataSet(dataset, {fileExtension: 'txt', fileName: 'meuArquivo'});
	 */

	this.generateFileByDataSet = function (dataset, fileProperties) {
		var content = "";
		var name = 'content';
		var extension = 'text/plain';

		if (fileProperties !== undefined) {
			if (fileProperties.fileName !== undefined)
				name = fileProperties.fileName;
		}

		dataset.forEach(function (line) {
			content += line + "\r\n";
		});

		var element = document.createElement('a');
		element.setAttribute('href', 'data:' + extension + ';charset=utf-8,' + encodeURIComponent(content));
		element.setAttribute('download', name);

		element.style.display = 'none';
		document.body.appendChild(element);

		element.click();

		document.body.removeChild(element);
	};

	/**
	 * Este método verifica se a widget foi filtrada usando o zh-grid-filter
	 * @param Widget widget
	 * @return boolean
	 */
	this.isWidgetFiltered = function (widget) {
		var isFiltered = true;
		var currentFilter;
		if(widget.hasOwnProperty('__filterPopup'))
			currentFilter = widget.__filterPopup.currentFilter;
		else
			currentFilter = widget.appliedFilter.FILTER_ROW;
			
		var requiredFields = widget.fields.filter(function (field){
			return isRequiredFieldInFilter(field) || isRequiredFieldWithoutFilterProperties(field);
		});
		if(requiredFields.length && Array.isArray(currentFilter)) {
			isFiltered = false;
		}
		return isFiltered;
	};

	function isRequiredFieldInFilter(field) {
		return field.filterProperties && field.filterProperties.validations && field.filterProperties.validations.required;
	}

	function isRequiredFieldWithoutFilterProperties(field) {
		return field.isVisible && field.validations && field.validations.required && !field.filterProperties;
	}

	/**
	 * Este método valida a inserção de registros na widget
	 * @param Widget widget
	 */
	this.newRow = function (widget) {
		if(this.isWidgetFiltered(widget)) {
			widget.newRow();
		} else {
			var message = "You can't add without filtering";
			ScreenService.showMessage(message);
		}
	};

    /*
     * Função que adiciona à currentRow o(s) valor(es)
     * da widget parent necessários para salvar o registro.
     *
     * @param object widget
     * @param object parents
     *
     * @return promise
     */

    this.saveWithParent = function(widget, parents) {
        if(Array.isArray(parents)) {
            parents.forEach(function(parent){
                widget.currentRow[parent] = widget.parent.currentRow[parent];
            });
            return this.widgetSave(widget);
        }
        else throw("\'Parents\' property should be an array, " + typeof parents + " given.");
    };
};

Configuration(function (ContextRegister) {
	ContextRegister.register('General', General);
});

// FILE: js/Util/LogDecorator.js
function LogDecorator(ZhuLogHelper) {
    /* Widget Getters */
    var getContentWidget = function(gridWidget) {
        return gridWidget.widgets[0].widgets[0];
    };

    /* Field Names */
    const ROUTE_FIELD_NAME              = 'ROUTE';
    const ROUTE_LABEL_FIELD_NAME        = 'ROUTE_LABEL';
    const TIME_FIELD_NAME               = 'TIMESTAMP';
    const USER_ID_FIELD_NAME            = 'USER_ID';
    const USER_NAME_FIELD_NAME          = 'USER_NAME';
    const CONTAINER_LABEL_FIELD_NAME    = 'CONTAINER_LABEL';
    const WIDGET_LABEL_FIELD_NAME       = 'WIDGET_LABEL';
    const CONTENT_FIELD_NAME            = 'CONTENT';
    const CONTEXT_DATA_FIELD_NAME       = 'CONTEXT_DATA';
    const USER_DATA_FIELD_NAME          = 'USER_DATA';
    const HTTP_RESPONSE_CODE_FIELD_NAME = 'HTTP_RESPONSE_CODE';
    const REQUEST_TYPE_FIELD_NAME       = 'REQUEST_TYPE';
    const REQUEST_ID_FIELD_NAME         = 'REQ_ID';
    const CONTENT_FILTER_FIELD_NAME     = 'contentFilter';
    const TIME_START_FIELD_NAME         = 'TIMESTAMP_START';
    const TIME_END_FIELD_NAME           = 'TIMESTAMP_END';
    const TYPE_FIELD_NAME               = 'TYPE';

    /* Field Labels */
    const USER_ID_FIELD_LABEL         = 'User Id';
    const USER_NAME_FIELD_LABEL       = 'User Name';
    const CONTAINER_LABEL_FIELD_LABEL = 'Screen';
    const WIDGET_LABEL_FIELD_LABEL    = 'Section Screen';
    const ROUTE_LABEL_FIELD_LABEL     = 'Action';

    /* Field Templates */
    const TEXT_EDIT_TEMPLATE = 'field/text-edit.html';
    const SELECT_NATIVE_TEMPLATE = 'field/select-native.html';
    const SELECT_MULTIPLE_TEMPLATE = 'field/select-multiple.html';

    var filterFieldsInAdvancedFieldGroup = [REQUEST_ID_FIELD_NAME, TYPE_FIELD_NAME, HTTP_RESPONSE_CODE_FIELD_NAME, CONTENT_FIELD_NAME];
    var fieldsNotUpdateTemplate = [ROUTE_LABEL_FIELD_NAME];

    var updateWidgetFields = function (widget) {
        var routeField          = ZhuLogHelper.getField(widget, ROUTE_FIELD_NAME);
        var userIdField         = Util.clone(routeField);
        var userNameField       = Util.clone(routeField);
        var containerLabelField = Util.clone(routeField);
        var widgetLabelField    = Util.clone(routeField);
        var routeLabelField     = Util.clone(routeField);

        userIdField.name  = USER_ID_FIELD_NAME;
        userIdField.label = USER_ID_FIELD_LABEL;

        userNameField.name  = USER_NAME_FIELD_NAME;
        userNameField.label = USER_NAME_FIELD_LABEL;

        containerLabelField.name  = CONTAINER_LABEL_FIELD_NAME;
        containerLabelField.label = CONTAINER_LABEL_FIELD_LABEL;

        widgetLabelField.name  = WIDGET_LABEL_FIELD_NAME;
        widgetLabelField.label = WIDGET_LABEL_FIELD_LABEL;

        routeLabelField.name  = ROUTE_LABEL_FIELD_NAME;
        routeLabelField.label = ROUTE_LABEL_FIELD_LABEL;

        ZhuLogHelper.addField (widget, userIdField,         1);
        ZhuLogHelper.addField (widget, userNameField,       2);
        ZhuLogHelper.moveField(widget, TIME_FIELD_NAME,     3);
        ZhuLogHelper.addField (widget, containerLabelField, 4);
        ZhuLogHelper.addField (widget, widgetLabelField,    5);
        ZhuLogHelper.addField (widget, routeLabelField,     6);
        ZhuLogHelper.moveField(widget, ROUTE_FIELD_NAME,    7);

        ZhuLogHelper.editField(widget, ROUTE_LABEL_FIELD_NAME, {
            valueField: ROUTE_FIELD_NAME,
            descriptionField: ROUTE_LABEL_FIELD_NAME,
            template: SELECT_NATIVE_TEMPLATE,
            maxWidth: '500px',
            dataSource: {
                data: window.routesTranslate || []
            },
            filterProperties: {
                template: SELECT_MULTIPLE_TEMPLATE,
                operator: 'IN'
            }
        });
    };

    var updateWidgetFieldsGridOnly = function(gridWidget) {
        gridWidget.fields.forEach(function(field) {
            if(field.name && !(filterFieldsInAdvancedFieldGroup.indexOf(field.name) != -1)) {
                field.fieldGroup = null;
            }

            if(field.filterProperties) {
                if(fieldsNotUpdateTemplate.indexOf(field.name) == -1) {
                    field.template = field.filterProperties.template;
                }
                if(field.filterProperties.template == TEXT_EDIT_TEMPLATE) {
                    field.filterProperties.operator = 'LIKE';
                }
            }
        });

        //General filter fields
        ZhuLogHelper.removeField(gridWidget, CONTENT_FILTER_FIELD_NAME);
        ZhuLogHelper.editField(gridWidget, USER_NAME_FIELD_NAME, {class: 4});
        ZhuLogHelper.editField(gridWidget, TIME_START_FIELD_NAME, {
            'class': 3,
            'datetime-lang': 'pt-BR',
            'datetime-dateFormat': 'dd/mm/yy',
            'datetime-timeFormat': 'hh:ii'
        });
        ZhuLogHelper.editField(gridWidget, TIME_END_FIELD_NAME, {
            'class': 3,
            'datetime-lang': 'pt-BR',
            'datetime-dateFormat': 'dd/mm/yy',
            'datetime-timeFormat': 'hh:ii'
        });
        ZhuLogHelper.editField(gridWidget, CONTAINER_LABEL_FIELD_NAME, {class: 3});
        ZhuLogHelper.editField(gridWidget, WIDGET_LABEL_FIELD_NAME, {class: 3});
        ZhuLogHelper.editField(gridWidget, ROUTE_LABEL_FIELD_NAME, {class: 3});
        ZhuLogHelper.editField(gridWidget, ROUTE_FIELD_NAME, {class: 3});

        //Advanced filter fields
        ZhuLogHelper.editField(gridWidget, REQUEST_ID_FIELD_NAME, {class: 4});
        ZhuLogHelper.editField(gridWidget, TYPE_FIELD_NAME, {class: 4});
        ZhuLogHelper.editField(gridWidget, HTTP_RESPONSE_CODE_FIELD_NAME, {class: 4});
        ZhuLogHelper.editField(gridWidget, CONTENT_FIELD_NAME, {
            isVisible: false,
            fieldGroup: 'advancedFilter',
            template : TEXT_EDIT_TEMPLATE,
            filterProperties: {
                isVisible: true,
                operator : 'LIKE',
                class    : 12
            }
        });

        ZhuLogHelper.moveField(gridWidget, TIME_START_FIELD_NAME, 3);
        ZhuLogHelper.moveField(gridWidget, TIME_END_FIELD_NAME,   4);
    };

    var updateWidgetFieldsFormOnly = function(formWidget) {
        ZhuLogHelper.removeField(formWidget, CONTEXT_DATA_FIELD_NAME);
        ZhuLogHelper.removeField(formWidget, USER_DATA_FIELD_NAME);

        ZhuLogHelper.editField(formWidget, USER_NAME_FIELD_NAME,          {class: 6});
        ZhuLogHelper.editField(formWidget, TIME_FIELD_NAME,               {class: 4});
        ZhuLogHelper.editField(formWidget, HTTP_RESPONSE_CODE_FIELD_NAME, {class: 2});
        ZhuLogHelper.editField(formWidget, REQUEST_TYPE_FIELD_NAME,       {class: 4});
        ZhuLogHelper.editField(formWidget, WIDGET_LABEL_FIELD_NAME,       {class: 6});
        ZhuLogHelper.editField(formWidget, CONTAINER_LABEL_FIELD_NAME,    {class: 6});
        ZhuLogHelper.editField(formWidget, ROUTE_FIELD_NAME,              {class: 6});
        ZhuLogHelper.editField(formWidget, ROUTE_LABEL_FIELD_NAME,        {class: 6});
    };

    var convertDateValue = function(value) {
        return "TO_DATE('" + value + "', 'DD/MM/YYYY HH24:MI')";
    };

    this.decorateWidget = function(gridWidget) {
        updateWidgetFields(gridWidget);
        updateWidgetFieldsGridOnly(gridWidget);

        var formWidget = getContentWidget(gridWidget);
        updateWidgetFields(formWidget);
        updateWidgetFieldsFormOnly(formWidget);

        gridWidget.currentRowDefaultFilter = {
            TIMESTAMP_START: moment().add(-1, 'days').format('DD/MM/YYYY') + ' 00:00',
            TIMESTAMP_END  : moment().format('DD/MM/YYYY') + ' 23:59'
        };
    };

    this.normalizeRows = function(rows) {
        rows = rows.map(function(row) {
            var userData  = JSON.parse(row.USER_DATA);
            row.USER_ID   = userData.CDOPERADOR;
            row.USER_NAME = userData.NMOPERADOR;

            var contextData     = JSON.parse(row.CONTEXT_DATA);
            row.CONTAINER_LABEL = contextData.containerLabel;
            row.WIDGET_LABEL    = contextData.widgetLabel;
            row.ROUTE_LABEL     = row.ROUTE;

            return row;
        });
        return rows;
    };

    this.normalizeFilters = function(filters) {
        filters = filters.map(function(filter) {
            if(filter.name == TIME_START_FIELD_NAME || filter.name == TIME_END_FIELD_NAME) {
                filter.value = convertDateValue(filter.value);
            }
            return filter;
        });
        return filters;
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LogDecorator', LogDecorator);
});


// FILE: js/Util/MenuBuilder.js
var MenuBuilder = function (restEngine, ZHPromise, templateManager, metaDataFactory, eventAggregator, $window, requestDataSourceEngine, ScreenService) {
    var localProductMenu;
    var attrToReplace = [
        "label",
        "showMenu",
        "showHeader",
        "showDescriptionOrder",
        "onRequestIdleTimeout",
        "mainWindow",
        "logLevel",
        "aclEnable",
        "showMenuSearch",
        "userDataActions",
        "reportServiceUrl",
        "reportsPath",
        "inDeveloperMode",
        "showDevtoolsInSameWindow",
        "shouldBackToMainWindowOnReload",
        "shortcutKeys"
    ];

    function callbackError(err) {
        if (err.data && err.data.error) {
            ScreenService.showMessage(err.data.error, 'ERROR');
        } else {
            ScreenService.showMessage(JSON.stringify(err), 'ERROR');
        }
    }

    var requestProductMenu = function () {
        var modulesPackage = $window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/containers.json").then(function (response) {
            var productObjMenu = response.data;
            var keys = Object.keys(productObjMenu);
            // get first project menu found
            return productObjMenu[keys[0]];
        });
    };

    var removeEmptyMenuGroups = function(groupMenus) {
        return groupMenus.filter(function(group) {
            return (group.menus && group.menus.length != 0) || group.name == "zhHistory";
        });
    };

    var orderMenus = function(menus) {
        menus.map(function(menu) {
            if(menu.menus) {
                menu.menus = orderMenus(menu.menus);
            }
            return menu;
        });

        return sortMenus(menus);
    };

    var sortMenus = function(menus) {
        return menus.sort(function(menuA, menuB) {
            if (menuA.order && menuB.order) {
                if (menuA.order < menuB.order) {
                    return -1;
                }
                if (menuA.order > menuB.order) {
                    return 1;
                }
            }
            if (!menuA.order && menuB.order) {
                return -1;
            }
            if (menuA.order && menuB.order) {
                return 1;
            }
            return 0;
        });
    };

    var prepareMenus = function (response) {
        var groupMenus = response.dataset.menu;
        window.routesTranslate = response.dataset.routesTranslate;
        var zhuLogWidget = templateManager.container ? templateManager.container.getWidget('zhuLog') : false;
        if(zhuLogWidget) {
            zhuLogWidget.getField('ROUTE_LABEL').dataSource.data = window.routesTranslate;
            $(window).trigger('resize.grid');
        }
        var groupMenuHistory = [templateManager.project.getMenu("zhHistory")];
        groupMenus = removeEmptyMenuGroups(groupMenus.concat(groupMenuHistory));
        templateManager.project.groupMenu = metaDataFactory.groupMenuFactory(orderMenus(groupMenus));

        //Force framework to apply ACL Rules
        var aux = {};
        Object.keys(templateManager.aclRules).forEach(function(ruleKey){
            aux[ruleKey] = templateManager.aclRules[ruleKey];
        });
        templateManager.aclRules = aux;
        //Force framework to apply ACL Rules

        eventAggregator.publish('menuBuilded');
    };

    var replaceProductAttrOnMenu = function (productMenu) {
        attrToReplace.forEach(function (attr) {
            templateManager.project[attr] = productMenu[attr];
        });
    };

    var setLocalProductMenu = function () {
        if (!$window.modulesPackage) {
            requestProductMenu().then(function (productMenu) {
                localProductMenu = productMenu;
            });
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    if(!window.notUseMicroservice) {
        eventAggregator.subscribe('loginSuccess', function (loginResponse) {
            if (!$window.modulesPackage && !$window.keepTitle) {
                replaceProductAttrOnMenu(localProductMenu);
                templateManager.updateTitle(templateManager.project.label);
            }
            var filter = [{
                "name": "CONCATENATE_GROUP_MENU",
                "operator" : "=",
                "value" :  true
            }, {
                "name": "VISIBLE_ELEMENTS",
                "operator" : "=",
                "value" :  loginResponse.VISIBLE_ELEMENTS || []
            }, {
                "name": "URL_ORIGIN",
                "operator" : "=",
                "value" :  window.location.origin
            }, {
                "name": "CURRENT_MODULE",
                "operator" : "=",
                "value" :  getCurrentModule()
            }, {
                "name": "ACL_ADMINISTRATOR",
                "operator" : "=",
                "value" :  loginResponse.ACL_ADMINISTRATOR
            } ];
            requestDataSourceEngine.getDataSource('lib#/lib_buildMenu', filter)
                .then(prepareMenus)
                .catch(callbackError);
        });

        setLocalProductMenu();
    }
};

Configuration(function (ContextRegister) {
    ContextRegister.register('MenuBuilder', MenuBuilder);
});

// FILE: js/Util/Messages.js
function LibMessages(){
    this.ALE_EXPIRED_SESSION = 'Atenção! Sua sessão expirou. Você será redirecionado para realizar o login novamente.';
    this.ALE_MIGRATION_IS_PENDENT = 'ATENÇÃO!<br/><br/>Esse produto foi atualizado recentemente, com necessidade de alterações na estrutura do banco de dados, sem que essas alterações tenham sido aplicadas.<br/>A utilização do produto nessa situação poderá acarretar erros.<br/><br/>Solicite ao administrador do sistema a regularização da alteração na estrutura do bando de dados.', 'INFORMATION';

    this.ERR_ATTEMPT_END = 'Tentativas esgotadas. Redirecionando...';
    this.ERR_ATTEMPT_PARTIAL = 'Senha incorreta.<br/>Tentativa %1/3';
    this.ERR_EXPIRED_SESSION_LOGIN = 'Sessão expirada. Redirecionando...';
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibMessages', LibMessages);
});


// FILE: js/Util/Utilities.js
function LibUtilities(ScreenService, eventAggregator, localStorageEngine, templateManager, requestDataSourceEngine, restEngine){
    var self = this;
    var validateExpiredSessionRoute = 'lib_validateExpiredSession';
    this.loginURI = LibConfig.loginURL;
    this.expiredSessionMethod = null;

    if(window.serviceUrl.indexOf('service') != -1) {
        this.loginURI += document.location.href;
    } else {
        this.loginURI += document.location.origin;
    }

    this.showErrorMessage = function(error) {
        var message = error;
        if(typeof error === 'object') {
            if(error.message) {
                message = error.message;
            } else if(error.data && error.data.error){
                message = error.data.error;
            }
        }

        if(!message) {
            message = 'Ocorreu um erro inesperado. Entre em contato com o administrador do sistema.';
        } else if(message.indexOf('Exception: Exception: ') != -1) {
            message = message.replace('Exception: Exception: ', '');
        }

        ScreenService.showMessage(message, 'error');
    };

    var getStrParamLoginUri = function(paramDescription, paramValue) {
        return paramValue || paramDescription == 'service' ? '&' + paramDescription + '=' + paramValue : '';
    };

    this.redirectToLogin = function(params) {
        var uri = window.serviceUrl.indexOf('service') != -1 ? self.loginURI.split('#')[0] : self.loginURI;
        params.forEach(function(param) {
            uri += getStrParamLoginUri(param.desc, param.value);
        });
        window.location = uri;
    };

    eventAggregator.subscribe("EventOnRequestStart", function (eventData) {
        var requestConfig = eventData.requestConfig;
        if (requestConfig && requestConfig.url && shouldAddRequestInfo(requestConfig.url) && templateManager.container) {
            requestConfig.data.origin = requestConfig.data.origin || {};
            var widget;
            var container = templateManager.loadingContainerJson;
            if(container) {
                widget = getFirstWidgetWithDataSourceRest(container.widgets);
                if(!widget) {
                    widget = container.widgets[0];
                }
            } else {
                container = templateManager.container;
                widget = container.getActiveWidget();
            }

            requestConfig.data.origin.containerLabel = container.label || container.name;
            requestConfig.data.origin.widgetLabel = widget.label || widget.name;
        }

        var route = requestConfig.url.split('/').pop();
        if(route != validateExpiredSessionRoute) {
            factoryExpiredSessionMethods();
        }
    });

    var shouldAddRequestInfo = function(url) {
        return !url.match(/.json/i) &&
               !url.match(/.html/i) &&
               !url.match(/.svg/i);
    };

    var getFirstWidgetWithDataSourceRest = function(widgets) {
        for(var i = 0; i < widgets.length; i++) {
            if(widgets[i].dataSource && widgets[i].dataSource.rest == true) {
                return widgets[i];
            } else {
                return getFirstWidgetWithDataSourceRest(widgets[i]);
            }
        }
    };

    var factoryExpiredSessionMethods = function() {
        if(templateManager.project && templateManager.project.onRequestIdleTimeout) {
            if(self.expiredSessionMethod) {
                clearTimeout(self.expiredSessionMethod);
            }
            self.expiredSessionMethod = Util.doAsync(validateExpiredSession, templateManager.project.onRequestIdleTimeout);
        }
    };

    var validateExpiredSession = function() {
        var timeout = templateManager.project.onRequestIdleTimeout;
        var filter = [ {
           name: 'TIMEOUT',
           value: timeout,
           operator: '='
        } ];
        
        requestDataSourceEngine.getDataSource('lib#/lib_validateExpiredSession', filter).then(function(response){
            if(response && response.dataset && response.dataset.expiredSessionInfo) {
                var expiredSessionInfo = response.dataset.expiredSessionInfo;
                if(expiredSessionInfo.IS_EXPIRED) {
                    eventAggregator.publish('LibExpiredSession');
                } else {
                    if(self.expiredSessionMethod) {
                        clearTimeout(self.expiredSessionMethod);
                    }
                    Util.doAsync(validateExpiredSession, expiredSessionInfo.TIMEOUT);
                }
            } else {
                self.showErrorMessage(response);
            }
        }).catch(self.showErrorMessage);
    };

    this.changeLoginResponseToOldParams = function(loginResponse) {
        if(loginResponse) {
            return {
                cdoperador:          loginResponse.USER_ID,
                nmoperador:          loginResponse.USER_NAME,
                dsemailoper:         loginResponse.EMAIL,
                nrorg:               loginResponse.ORGANIZATION_ID,
                nmorg:               loginResponse.TRADING_NAME,
                nmrazsocialorg:      loginResponse.COMPANY_NAME,
                perfilDeAcesso:      loginResponse.ACCESS_PROFILE,

                cdempresa:           loginResponse.CDEMPRESA     || null,
                cdfilialselecionada: loginResponse.CDFILIAL      || null,
                cdpictprat:          loginResponse.CDPICTPRAT    || null,
                cdpictprod:          loginResponse.CDPICTPROD    || null,
                ididioma:            loginResponse.IDIDIOMA      || null,
                nmacesso:            loginResponse.NMACESSO      || null,
                nmcomploper:         loginResponse.NMCOMPLOPER   || null,
                nmfilialselecionada: loginResponse.NMFILIAL      || null,
                nrestruttrab:        loginResponse.NRESTRUTTRAB  || null,
                nrorgtrab:           loginResponse.NRORGTRAB     || null,
                nrparcnegocio:       loginResponse.NRPARCNEGOCIO || null,

                logged:              loginResponse.LOGGED,
                senhaExpirada:       loginResponse.EXPIRED_PASSWORD,
                version:             loginResponse.VERSION
            };
        }
    };

    var getCurrentModule = function() {
        var currentModule = window.location.hash.split('/').filter(function (_) {
            return _.length;
        }).pop();
        if (currentModule && currentModule.length > 0) {
            currentModule = currentModule.split(/_|#/)[0];
        }
        return currentModule;
    };

    this.openMicroServiceUrl = function(args) {
        var containerName = args.owner.msContainerName;
        var currentModule, module;
        
        if(containerName.indexOf('#') == -1) {
            currentModule = module = '';
        } else {
            var subModule = containerName.split('#')[0];
            module        = subModule.split('_')[0];
            currentModule = getCurrentModule();   
        }

        if (module && currentModule && (module != currentModule)) {
            var filter = [ {
                name: 'TOKEN',
                value: localStorageEngine.getLocalVar('LIB_TOKEN'),
                operator: '='
            } ];

            requestDataSourceEngine.getDataSource(module + '#/lib_startSession', filter)
                .then(function(){
                    ScreenService.openLink(args.owner.url);
                }).catch(self.showErrorMessage);
        }
    };

    this.openLinkByMenu = function(menu) {
        if(typeof menu === 'string') {
            menu = templateManager.project.getMenu(menu);
        }

        var args = { owner: {
            msContainerName: menu.msContainerName,
            url: menu.url
        } };
        self.openMicroServiceUrl(args);
    };

    this.openWindow = function(windowName, newWindow) {
        requestModulesMetadata().then(function (modules) {
            var subModule = windowName.split('#')[0];
            var module = subModule.split('_')[0];
            var url = window.location.origin + '/' + modules[module].baseUrl + "/#/" + windowName;
            if (ScreenService.openLink) {
                ScreenService.openLink(url, newWindow);
            } else {
                ScreenService.openWindow(windowName);
            }
        });
    }; 
    
    var requestModulesMetadata = function () {
        var modulesPackage = window.modulesPackage || "modules";
        return restEngine.requestMetaData(modulesPackage + "#json/modules.json").then(function (response) {
            return response.data;
        });
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibUtilities', LibUtilities);
});


// FILE: js/Util/ViewFloatMask.js
var ViewFloatMask = function(maskEngine) {
    var keypressHandler = function (e, element, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var thousandsCode = opt.thousands.charCodeAt();
        var isThousands = (thousandsCode === e.which);
        var is_number = (e.which >= 48 && e.which <= 57);
        var is_dot = (e.which == 46);
        var is_comma = (e.which == 44);
        var is_minus = (e.which == 45);
        var hasSeparator = (element.val().indexOf(opt.decimal) != -1 || element.val().indexOf(opt.thousands) != -1);
        var hasNegative = (element.val().length > 0 || !opt.allowNegative);

        if ((hasSeparator && (is_dot || is_comma)) || (hasNegative && is_minus) || isThousands)
            e.preventDefault();

        if (is_dot || is_comma || is_number || is_minus)
            element.data('maskOptions', opt);
        else
            e.preventDefault();
    };

    var viewFloat = function(element, viewValue, params, callBack) {
        var value = viewValue || '';
        element.val(value);
        params.callback = callBack;
        params.showRound = !('showRound' in params) ? true : params.showRound;
        params.showSymbol = false;

        element.on("keypress", function(event) {
            keypressHandler(event, element, params);
        });

        element.on("blur", function() {
            element.text(unMaskViewFloat(element.val(), params));
            element.val(viewFloatFormat(element.val(), params));
            params.callback(element.text());
        });

        if (!!params.showRound) {
            element.on("focus", function() {
                var value = element.text();
                var viewValue = viewFloatFormat(element.val(), params);
                var currentValue = viewFloatFormat(element.text(), params);

                if (currentValue !== viewValue)
                    value = element.val();

                element.val(viewFloatFormat(value, params));
                element.select();
            });
        }
    };

    var viewFloatFormat = function(value, params) {
        var defaultOptions = Util.clone($.currency.i18n[$.currency.lang]);

        var valueFormated = "";
        try {
            var options = Util.extend(defaultOptions, (params || {}), true);
            valueFormated = $.currency.string(value, options);
        } catch (e) {
            valueFormated = value;
        }
        return valueFormated;
    };

    var unMaskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var fltValue = opt.allowZero && !opt.allowEmpty ? 0 : "";
        var valueFormated = value;

        if (!Util.isEmptyOrBlank(valueFormated)) {
            var decimal = opt.decimal;
            var thousands = opt.thousands;

            if (typeof valueFormated === "string") {
                if(opt.showSymbol)
                    valueFormated = valueFormated.replace(opt.symbol, '');

                if(thousands)
                    valueFormated = valueFormated.replace(new RegExp('\\' + thousands, 'g'), '');

                valueFormated = valueFormated.replace(new RegExp('\\' + decimal, 'g'), '.');
                valueFormated = removeInvalidChar(valueFormated);
            }
            fltValue = parseFloat(valueFormated);

            if ((!opt.allowZero && fltValue === 0) || $.currency.isInvalidValue(fltValue, opt))
                valueFormated = "";
        }
        return valueFormated;
    };

    var removeInvalidChar = function(value) {
        var index = value.indexOf('.');
        return index + 1 === value.length ? value.slice(0, index) : value;
    };

    var maskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);

        if (value)
            value = value.replace(new RegExp('\\' + '.', 'g'), opt.decimal);

        return value;
    };

    this.register = maskEngine.registerMask('viewFloat', viewFloat, viewFloatFormat);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ViewFloatMask', ViewFloatMask);
});

// FILE: js/Util/WidgetsMetaData.js
function WidgetsMetaData() {
    this.getExpiredSession = function() {
        return {
            "name": "EXPIRED_SESSION",
            "label": "Sessão Expirada",
            "isVisible": true,
            "showDescriptionOrder": false,
            "showAsModal": true,
            "template": "widget/form.html",
            "tekLibExpiredSession": true,
            "actions": [
                {
                    "name": "CONFIRM",
                    "label": "Confirmar",
                    "showAsAction": "never",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "LibLoginController.expiredSessionActionEventConfirm(args)"
                        }
                    ]
                },
                {
                    "name": "CANCEL",
                    "label": "Sair",
                    "showAsAction": "back",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "LibLoginController.expiredSessionActionEventCancel()"
                        }
                    ]
                }
            ],
            "events": [],
            "dataSource": [],
            "fields": [
                {
                    "name": "OPERATOR",
                    "label": "Operador",
                    "template": "field/text-edit.html",
                    "class": 12,
                    "readOnly": true,
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true
                },
                {
                    "name": "PASSWORD",
                    "label": "Senha",
                    "template": "field/password.html",
                    "class": 12,
                    "focused": true,
                    "validations":{
                        required: true
                    },
                    "isVisible": true,
                    "showOnList": true,
                    "showOnForm": true,
                    "events":[
                        {
                            "name": "FieldOnKeyup",
                            "code": "LibLoginController.expiredSessionFieldOnKeyupPassword(args)"
                        }
                    ]
                }
            ]
        };
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('WidgetsMetaData', WidgetsMetaData);
});


// FILE: js/Service/Login.js
function LibLoginService(restEngine, requestDataSourceEngine){
    var buildRequestObj = function(baseUrl, service, params) {
        return {
            service: service,
            params: params,
            typeRequest: "post",
            baseUrl: baseUrl
        };
    };

    this.requestToken = function(clientId, loginURL, bHash) {
        var requestObj = buildRequestObj(loginURL, "/lm_requestToken", { "CLIENT_ID": clientId, "BHASH":  bHash });
        return restEngine.post(requestObj).then(function(data){
            return data && data.dataset ? data.dataset.token : [];
        });
    };

    this.startSession = function(token, versionFile, productId) {
        var filter = [ {
            name: 'TOKEN',
            value: token,
            operator: '='
        }, {
            name: 'VERSION_FILE',
            value: versionFile,
            operator: '='
        }, {
            name: 'PRODUCT_ID',
            value: productId,
            operator: '='
        }, {
            name: 'IS_LOGGED',
            value: localStorage.getItem('IS_LOGGED') || false,
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('lib#/lib_startSession', filter).then(function(data){
            if(typeof data == 'string') {
                return {
                    isError : true,
                    message : data
                };
            }
            return data && data.dataset ? data.dataset.system : false;
        });
    };

    this.validateUser = function(cdOperator, password) {
        var filter = [ {
            name: 'USER',
            value: cdOperator,
            operator: '='
        }, {
            name: 'PASSWORD',
            value: password,
            operator: '='
        } ];

        return requestDataSourceEngine.getDataSource('lib#/lib_validateUser', filter);
    };

    this.getUriParams = function(title, theme, splashFavicon, image, product, settings, language, productId, service, ip, bHash) {
        if(service === undefined) {
            service = true;
        }

        var params = [];

        if(title)         params.push({desc: 'title',         value: title});
        if(theme)         params.push({desc: 'theme',         value: theme});
        if(image)         params.push({desc: 'image' ,        value: image});
        if(product)       params.push({desc: 'product',       value: product});
        if(settings)      params.push({desc: 'settings',      value: settings});
        if(language)      params.push({desc: 'language',      value: language});
        if(splashFavicon) params.push({desc: 'splashFavicon', value: splashFavicon});
        if(productId)     params.push({desc: 'productId',     value: productId});
                          params.push({desc: 'service',       value: service});
        if(ip)            params.push({desc: 'ip',            value: ip});
        if(bHash)         params.push({desc: 'bHash',         value: bHash});

        return params;
    };

    this.logout = function(){
        return requestDataSourceEngine.getDataSource('lib#/lib_logout', []);
    };

    this.getIpAddress = function(){
        return requestDataSourceEngine.getDataSource('lib#/lib_getIpAddress', []);
    };

    this.getModulesVersion = function(){
        return requestDataSourceEngine.getDataSource('lib#/lib_getModulesVersion', []);
    };

    this.generateModulesVersionFile = function() {
        return requestDataSourceEngine.setDataSource('lib#/lib_generateModulesVersionFile', [ { changed: true } ]);
    };

    this.downloadModulesVersionFile = function(fileName) {
        var serviceUrl = window.serviceUrl;
        window.open(serviceUrl + "/lib_downloadModulesVersionFile?requestType=DataSet&dataset[0][FILE_NAME]=" + fileName);
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginService', LibLoginService);
});



// FILE: js/Controller/Login.js
function LibLoginController(LibLoginService, LibUtilities, localStorageEngine, templateManager, eventAggregator, LibMessages,
                            ScreenService, metaDataFactory, WidgetsMetaData, ZhuAclController, ZhLogger, ZHPromise) {

    var loginUrlParams, loginUserId, loginUserName;

    var factoryExpiredSessionWidget = function() {
        return metaDataFactory.widgetFactory(WidgetsMetaData.getExpiredSession(), templateManager.container);
    };

    this.authenticate = function(title, theme, splashFavicon, image, product, productId,
                                 versionFile, testConnectionParams, language, settings, service) {
        var defer = ZHPromise.defer();
        var ip, bHash, setAclEventId;

        var setAclFunction = function() {
            setAcl(defer.promise, setAclEventId);
        };

        setAclEventId = eventAggregator.subscribe('onContainerOpen', setAclFunction);

        LibLoginService.getIpAddress().then(function(ipResponse) {
            if (ipResponse.dataset && ipResponse.dataset.ip) {
                ip    = ipResponse.dataset.ip.IP;
                bHash = ipResponse.dataset.ip.BHASH;
                var modulesVersionError = ipResponse.dataset.ip.MODULES_VERSION_ERROR;
                return ipResponseAndRequestToken(modulesVersionError, bHash);
            } else {
                return ZHPromise.reject(ipResponse);
            }
        }).then(function (tokenData) {
            loginUrlParams = LibLoginService.getUriParams(title, theme, splashFavicon, image, product,
                settings, language, productId, service, ip, bHash);
            return tokenResponseAndStartSession(tokenData, versionFile, productId);
        }).then(function (response) {
            return startSessionResponse(response);
        }).then(function (response) {
            defer.resolve(response);
        }).catch(function(error) {
            defer.reject(error);
            LibUtilities.showErrorMessage(error);
        });

        return defer.promise;
    };

    var ipResponseAndRequestToken = function(modulesVersionError, bHash) {
        if (modulesVersionError) {
            ZhLogger.warning(modulesVersionError);
        }
        return LibLoginService.requestToken(LibConfig.loginClientURL, LibConfig.loginURI, bHash);
    };

    var tokenResponseAndStartSession = function(tokenData, versionFile, productId) {
        if (tokenData.isError) {
            return ZHPromise.reject(tokenData.message);
        } else {
            if (tokenData && tokenData.length > 0) {
                localStorageEngine.setLocalVar('LIB_TOKEN', tokenData[0]);
                return LibLoginService.startSession(tokenData[0], versionFile, productId);
            } else {
                LibUtilities.redirectToLogin(loginUrlParams);
                return ZHPromise.when({});
            }
        }
    };

    var startSessionResponse = function(response) {
        if (response.isError) {
            return ZHPromise.reject(response.message);
        } else {
            if (!response.USER_ID) {
                localStorage.removeItem('IS_LOGGED');
                LibUtilities.redirectToLogin(loginUrlParams);
            } else if (response.SESSION_EXPIRED) {
                logoutExpiredSession(true);
            } else {
                localStorage.setItem('IS_LOGGED', true);
                loginUserId    = response.USER_ID;
                loginUserName  = response.USER_NAME;

                if(response.IS_PENDENT_MIGRATION) {
                    ScreenService.showMessage(ScreenService.i18n(LibMessages.ALE_MIGRATION_IS_PENDENT));
                }
            }

            return ZHPromise.when(response);
        }
    };

    var doLogout = function (params) {
        LibLoginService.logout().then(function () {
            localStorage.removeItem('IS_LOGGED');
            LibUtilities.redirectToLogin(params);
        }).catch(LibUtilities.showErrorMessage);
    };

    this.logout = function() {
        doLogout(loginUrlParams);
    };

    var setAcl = function (loginPromise, setAclEventId) {
        loginPromise.then(function(response) {
            if(!window.productApplyAcl) {
                ZhuAclController.setUserRules(response.ACCESS_PROFILE, window.projectId, response.USER_ID, response.ORGANIZATION_ID,
                    {
                        canConfigureAcl: response.ACL_ADMINISTRATOR,
                        rules          : response.ACL_ADMINISTRATOR ? [] : false,
                        defaultRule    : response.ACL_ADMINISTRATOR ? 'A' : false
                    }
                );
                templateManager.project.aclDefaultRule = 'A';
            }
            eventAggregator.unSubscribe('onContainerOpen', setAclEventId);
        }).catch(LibUtilities.showErrorMessage);
    };

    eventAggregator.subscribe("LibExpiredSession", function () {
        if(!templateManager.container.getActiveWidget().tekLibExpiredSession) {
            var expiredSessionWidget = factoryExpiredSessionWidget();
            expiredSessionWidget.attempts = 0;
            expiredSessionWidget.dataSource.data = [];
            expiredSessionWidget.newRow();
            expiredSessionWidget.currentRow.OPERATOR = loginUserId + ' - ' + loginUserName;
            ScreenService.openPopup(expiredSessionWidget);
            localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', true);

        }
    });

    var callDoLogout = function() {
        doLogout(loginUrlParams);
    };

    var logoutExpiredSession = function (afterLogin) {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        var loadingMessage = afterLogin ? LibMessages.ERR_EXPIRED_SESSION_LOGIN : LibMessages.ERR_ATTEMPT_END;
        ScreenService.changeLoadingMessage(loadingMessage);
        ScreenService.showLoader();
        Util.doAsync(callDoLogout, 5000);
    };

    this.expiredSessionActionEventConfirm = function(args) {
        var expiredSessionWidget = args.owner.widget ? args.owner.widget : args.owner.field.widget;
        if(expiredSessionWidget.isValid()) {
            var password   = expiredSessionWidget.currentRow.PASSWORD;
            LibLoginService.validateUser(loginUserId, password)
                .then(function() {
                    localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
                    ScreenService.closePopup();
                }).catch(function() {
                    if(expiredSessionWidget.attempts >= 2) {
                        ScreenService.showMessage(LibMessages.ERR_ATTEMPT_PARTIAL.replace('%1', ++expiredSessionWidget.attempts), 'ALERT');
                        logoutExpiredSession();
                    } else {
                        ScreenService.showMessage(LibMessages.ERR_ATTEMPT_PARTIAL.replace('%1', ++expiredSessionWidget.attempts), 'ALERT');
                    }
                });
        }
    };

    this.expiredSessionActionEventCancel = function() {
        localStorageEngine.setLocalVar('LOGIN_EXPIRED_SESSION', false);
        doLogout(loginUrlParams);
    };

    this.expiredSessionFieldOnKeyupPassword = function(args) {
        var key = window.event.keyCode;

        //13 = ENTER
        if(key == 13) {
            this.expiredSessionActionEventConfirm(args);
        }
    };

    this.downloadModulesVersionFile = function() {
        return LibLoginService.generateModulesVersionFile().then(function(response) {
            if(response && response.dataset) {
                var fileName = response.dataset.generatedFile[0];
                LibLoginService.downloadModulesVersionFile(fileName);
                return true;
            } else {
                LibUtilities.showErrorMessage(response);
                return false;
            }
        }).catch(LibUtilities.showErrorMessage);
    }
}

Configuration(function(ContextRegister){
    ContextRegister.register('LibLoginController', LibLoginController);
});
